@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2004.
+ * Copyright (c) Tony Bybell 2004-2007.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <stdio.h>
 
 #ifndef _MSC_VER
@@ -17,12 +18,13 @@
 #include <string.h>
 #include <stdlib.h>
 #include "ae2.h"
-#include "aet.h"
+#include "symbol.h"
 #include "vcd.h"
 #include "lxt.h"
 #include "lxt2_read.h"
 #include "fgetdynamic.h"
 #include "debug.h"
+#include "busy.h"
 
 /* 
  * select appropriate entry points based on if aet2
@@ -120,6 +122,7 @@ va_list ap;
 va_start(ap, format);
 vfprintf(stderr, format, ap);
 fprintf(stderr, "\n");
+va_end(ap);
 exit(255);
 }
 
@@ -137,6 +140,8 @@ if((!ae2_msg_suppress)||(sev))
 	fprintf(stderr, "AE2 %03d | ", sev);
 	vfprintf(stderr, format, ap);
 	fprintf(stderr, "\n");
+
+	va_end(ap);
 	}
 	else
 	{
@@ -148,6 +153,8 @@ if((!ae2_msg_suppress)||(sev))
 	fprintf(stderr, " %c\r", twirler[twirl_pos]);
 	twirl_pos = (twirl_pos+1) & 3;
 	did_twirl = 1;
+
+	va_end(ap);
 	}
 }
 
@@ -362,11 +369,13 @@ for(i=0;i<numfacs;i++)
 
 	if((regex_matches)&&(!aet2_rd_get_fac_process_mask(i))) continue;
 
-	len = ae2_read_symbol_name(ae2, idx, buf) - 1;	/* it returns the zero byte counted */
+	len = ae2_read_symbol_name(ae2, idx, buf);	/* previously had -1, but it no longer returns the zero byte counted */
+	buf[len] = 0;					/* only being defensive here, not really necessary */
+
 	if(fr[match_idx].length>1)
 		{
-		sprintf(buf+len, "[%d:%d]", 0, fr[match_idx].length-1);
-		str=malloc_2(strlen(buf)+1);
+		int len2 = sprintf(buf+len, "[%d:%d]", 0, fr[match_idx].length-1);
+		str=malloc_2(len + len2 + 1);
 		if(!alt_hier_delimeter)
 			{
 			strcpy(str, buf);
@@ -375,7 +384,7 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+	        s=symadd_name_exists(str,0);
 		prevsymroot = prevsym = NULL;
 		}
 		else
@@ -389,7 +398,7 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+	        s=symadd_name_exists(str,0);
 		prevsymroot = prevsym = NULL;
 		}
 		
@@ -453,7 +462,7 @@ if(fast_tree_sort)
 		build_tree_from_name(facs[i]->name, i);
 		}
 /* SPLASH */                            splash_sync(4, 5);
-	treeprune(treeroot);
+	treegraft(treeroot);
 
 	fprintf(stderr, AET2_RDLOAD"Sorting facility hierarchy tree.\n");
 	treesort(treeroot, NULL);
@@ -475,7 +484,7 @@ if(fast_tree_sort)
 		curnode=curnode->nextinaet;
 		while((ch=(*subst)))
 			{	
-			if(ch==hier_delimeter) { *subst=0x01; }	/* forces sort at hier boundaries */
+			if(ch==hier_delimeter) { *subst=VCDNAM_HIERSORT; }	/* forces sort at hier boundaries */
 			subst++;
 			}
 		}
@@ -491,7 +500,7 @@ if(fast_tree_sort)
 		subst=facs[i]->name;
 		while((ch=(*subst)))
 			{	
-			if(ch==0x01) { *subst=hier_delimeter; }	/* restore back to normal */
+			if(ch==VCDNAM_HIERSORT) { *subst=hier_delimeter; }	/* restore back to normal */
 			subst++;
 			}
 		}
@@ -507,7 +516,8 @@ if(fast_tree_sort)
 		build_tree_from_name(facs[i]->name, i);
 		}
 /* SPLASH */                            splash_sync(5, 5);
-	treeprune(treeroot);
+	treegraft(treeroot);
+	treesort(treeroot, NULL);
 	}
 
 min_time = first_cycle; max_time=last_cycle;
@@ -551,7 +561,16 @@ struct HistEnt *htemp = histent_calloc();
 struct lx2_entry *l2e = lx2_table+(*facidx);
 FACREF *f = fr+(*facidx);
 
-/* fprintf(stderr, "%Ld %d %s\n", *time, *facidx, *value); */
+static int busycnt = 0;
+
+busycnt++;
+if(busycnt==WAVE_BUSY_ITER)
+        {
+        busy_window_refresh();
+        busycnt = 0;
+        }
+
+/* fprintf(stderr, "%lld %d %s\n", *time, *facidx, *value); */
 
 if(f->length>1)        
         {
@@ -824,7 +843,9 @@ if(cnt>100)
 	fprintf(stderr, AET2_RDLOAD"Extracting %d traces\n", cnt);
 	}
 
+set_window_busy(NULL);
 ae2_iterator(min_time, max_time);
+set_window_idle(NULL);
 
 for(txidx=0;txidx<numfacs;txidx++)
 	{
@@ -891,3 +912,11 @@ for(txidx=0;txidx<numfacs;txidx++)
 
 #endif
 /* ...of AET2_IS_PRESENT */
+
+/*
+ * $Id: ae2.c,v 1.2 2007/04/20 01:39:00 gtkwave Exp $
+ * $Log: ae2.c,v $
+ * Revision 1.2  2007/04/20 01:39:00  gtkwave
+ * initial release
+ *
+ */
