@@ -6,10 +6,16 @@
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  */
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "aet.h"
+#include <string.h>
+#include "symbol.h"
 #include "lxt.h"
 #include "debug.h"
 #include "bsearch.h"
@@ -33,7 +39,7 @@ int hier_max_level=0;
 char *hier_extract(char *pnt, int levels)
 {
 int i, len;
-char ch, *pnt2;
+char ch, *pnt2, *esc;
 char only_nums_so_far=1;
 
 if(!pnt) return(NULL);
@@ -42,6 +48,10 @@ len=strlen(pnt);
 if(!len) return(pnt);
 
 if(levels<1) levels=1;
+if((esc=strchr(pnt, '\\'))) 
+	{
+	return((levels==1) ? esc : pnt); /* shortcut out on escape IDs: level=1, esc char else all */
+	}
 
 pnt2=pnt+len-1;
 ch=*pnt2;
@@ -858,7 +868,7 @@ for(i=0;i<traces.total;i++)
 	if((subst=t->name))
 	        while((ch=(*subst)))
         	        {
-        	        if(ch==hier_delimeter) { *subst=0x01; } /* forces sort at hier boundaries */
+        	        if(ch==hier_delimeter) { *subst=VCDNAM_HIERSORT; } /* forces sort at hier boundaries */
         	        subst++;
         	        }
 
@@ -895,7 +905,7 @@ for(i=0;i<traces.total;i++)
 	if((subst=t->name))
 	        while((ch=(*subst)))
         	        {
-        	        if(ch==0x01) { *subst=hier_delimeter; } /* restore hier */
+        	        if(ch==VCDNAM_HIERSORT) { *subst=hier_delimeter; } /* restore hier */
         	        subst++;
         	        }
         }
@@ -1052,3 +1062,12 @@ while(t)
 
 UpdateTracesVisible();
 }
+
+/*
+ * $Id: analyzer.c,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: analyzer.c,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
