@@ -12,7 +12,7 @@
 
 #include <gtk/gtk.h>
 #include <stdlib.h>
-#include "alloca.h"
+#include "wavealloca.h"
 #include "debug.h"
 
 typedef struct _SearchProgressData {
@@ -52,14 +52,24 @@ enum AnalyzerBits  { AN_0, AN_X, AN_Z, AN_1, AN_H, AN_U, AN_W, AN_L, AN_DASH, AN
 /* for writing out 4 state formats (read GHW, write LXT) */
 #define AN_STR4ST  "0XZ11XZ0XXXXXXXX"
 
+/* now the recoded "extra" values... */
+#define RCV_X (1 | (0<<1))
+#define RCV_Z (1 | (1<<1))
+#define RCV_H (1 | (2<<1))
+#define RCV_U (1 | (3<<1))
+#define RCV_W (1 | (4<<1))
+#define RCV_L (1 | (5<<1))
+#define RCV_D (1 | (6<<1))
+
+#define RCV_STR "XZHUWL-?"
+/*               01234567 */
+
 /* ^^^   Bit representation   ^^^ */
 
 
 typedef struct HistEnt
 {
 hptr next;	      /* next transition in history */
-TimeType time;        /* time of transition */
-unsigned char flags;  /* so far only set on glitch/real condition */
 
 union
   {
@@ -67,6 +77,8 @@ union
   char *h_vector;	/* pointer to a whole vector of h_val type bits */
   } v;
 
+TimeType time;        /* time of transition */
+unsigned char flags;  /* so far only set on glitch/real condition */
 } HistEnt;
 
 enum HistEntFlagBits
@@ -125,7 +137,11 @@ struct Node
     hptr     *harray;   /* fill this in when we make a trace.. contains  */
 			/*  a ptr to an array of histents for bsearching */
     int      numhist;	/* number of elements in the harray */
-    struct fac *mvlfac; /* for use with mvlsim aets */
+
+    union {
+      struct fac *mvlfac; /* for use with mvlsim aets */
+      struct vlist_t *mvlfac_vlist;
+    };
   };
 
 typedef struct BitAttributes
@@ -211,6 +227,7 @@ typedef struct TraceEnt
     unsigned is_alias : 1;	/* set when it's an alias (safe to free t->name then) */
     unsigned vector : 1;	/* 1 if bit vector, 0 if node */
     unsigned shift_drag_valid : 1; /* qualifies shift_drag above */
+    unsigned interactive_vector_needs_regeneration : 1; /* for interactive VCDs */
 
     unsigned int flags;		/* see def below in TraceEntFlagBits */
     union
@@ -225,7 +242,7 @@ enum TraceEntFlagBits
 { TR_HIGHLIGHT_B, TR_HEX_B, TR_DEC_B, TR_BIN_B, TR_OCT_B, 
   TR_RJUSTIFY_B, TR_INVERT_B, TR_REVERSE_B, TR_EXCLUDE_B,
   TR_BLANK_B, TR_SIGNED_B, TR_ASCII_B, TR_COLLAPSED_B, TR_FTRANSLATED_B, TR_PTRANSLATED_B,
-  TR_ANALOG_STEP_B, TR_ANALOG_INTERPOLATED_B, TR_ANALOG_BLANK_STRETCH_B
+  TR_ANALOG_STEP_B, TR_ANALOG_INTERPOLATED_B, TR_ANALOG_BLANK_STRETCH_B, TR_REAL_B
 };
  
 #define TR_HIGHLIGHT 		(1<<TR_HIGHLIGHT_B)
@@ -243,8 +260,9 @@ enum TraceEntFlagBits
 #define TR_ANALOG_STEP 		(1<<TR_ANALOG_STEP_B)
 #define TR_ANALOG_INTERPOLATED	(1<<TR_ANALOG_INTERPOLATED_B)
 #define TR_ANALOG_BLANK_STRETCH	(1<<TR_ANALOG_BLANK_STRETCH_B)
+#define TR_REAL			(1<<TR_REAL_B)
 
-#define TR_NUMMASK	(TR_ASCII|TR_HEX|TR_DEC|TR_BIN|TR_OCT|TR_SIGNED)
+#define TR_NUMMASK	(TR_ASCII|TR_HEX|TR_DEC|TR_BIN|TR_OCT|TR_SIGNED|TR_REAL)
 
 #define TR_COLLAPSED	(1<<TR_COLLAPSED_B)
 #define TR_ISCOLLAPSED	(TR_BLANK|TR_COLLAPSED)
@@ -293,4 +311,15 @@ extern Trptr	   shift_click_trace;
 char *hier_extract(char *pnt, int levels);
 extern int hier_max_level;
 
+/* vector matching */
+char *attempt_vecmatch(char *s1, char *s2);
+
 #endif
+
+/*
+ * $Id: analyzer.h,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: analyzer.h,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
