@@ -7,11 +7,12 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <math.h>
 #include "gtk12compat.h"
 #include "currenttime.h"
 #include "pixmaps.h"
-#include "aet.h"
+#include "symbol.h"
 #include "bsearch.h"
 #include "color.h"
 #include "strace.h"
@@ -58,7 +59,7 @@ if(flags&TR_REVERSE)
 	{
 	char *fwdpnt, *revpnt;
 
-	fwdpnt=bits;
+	fwdpnt=(char *)bits;
 	revpnt=newbuff+nbits+6;
 	for(i=0;i<3;i++) *(--revpnt)=xtab[0];
 	for(i=0;i<nbits;i++)
@@ -71,7 +72,7 @@ if(flags&TR_REVERSE)
 	{
 	char *fwdpnt, *fwdpnt2;
 
-	fwdpnt=bits;
+	fwdpnt=(char *)bits;
 	fwdpnt2=newbuff;
 	for(i=0;i<3;i++) *(fwdpnt2++)=xtab[0];
 	for(i=0;i<nbits;i++)
@@ -335,6 +336,54 @@ else if(flags&TR_SIGNED)
 		strcpy(os, "XXX");
 		}
 	}
+else if(flags&TR_REAL)
+	{
+	char *parse;
+
+	if(nbits==64)
+		{
+		UTimeType utt = LLDescriptor(0);
+		double d;
+
+		parse=newbuff+3;
+
+		for(i=0;i<nbits;i++)
+			{
+			char ch = AN_STR[(int)(*(parse++))];
+			if ((ch=='0')||(ch=='1'))
+				{
+				utt <<= 1;
+				if(ch=='1')
+					{
+					utt |= LLDescriptor(1);
+					}
+				}
+				else
+				{
+				goto rl_go_binary;
+				}
+			}
+
+		memcpy(&d, &utt, sizeof(double));
+		os=pnt=(char *)calloc_2(1,32);		
+		sprintf(os, "%.16g", d);
+		}
+		else
+		{
+rl_go_binary:	len=(nbits/1)+2+1;		/* %xxxxx */
+		os=pnt=(char *)calloc_2(1,len);
+		if(show_base) { *(pnt++)='%'; }
+	
+		parse=newbuff+3;
+	
+		for(i=0;i<nbits;i++)
+			{
+			*(pnt++)=AN_STR[(int)(*(parse++))];
+			}
+	
+		*(pnt++)=0x00;
+		}
+	}
 else	/* decimal when all else fails */
 	{
 	char *parse;
@@ -436,7 +485,7 @@ int vtype(Trptr t, char *vec)
 int vtype2(Trptr t, vptr v)
 {
 	int i, nbits, res;
-	char *vec=v->v;
+	char *vec=(char *)v->v;
 
 	if (vec == NULL)
 		return(1);
@@ -806,6 +855,54 @@ else if(flags&TR_SIGNED)
 		strcpy(os, "XXX");
 		}
 	}
+else if(flags&TR_REAL)
+	{
+	char *parse;
+
+	if(nbits==64)
+		{
+		UTimeType utt = LLDescriptor(0);
+		double d;
+
+		parse=newbuff+3;
+
+		for(i=0;i<nbits;i++)
+			{
+			char ch = AN_STR[(int)(*(parse++))];
+			if ((ch=='0')||(ch=='1'))
+				{
+				utt <<= 1;
+				if(ch=='1')
+					{
+					utt |= LLDescriptor(1);
+					}
+				}
+				else
+				{
+				goto rl_go_binary;
+				}
+			}
+
+		memcpy(&d, &utt, sizeof(double));
+		os=pnt=(char *)calloc_2(1,32);		
+		sprintf(os, "%.16g", d);
+		}
+		else
+		{
+rl_go_binary:	len=(nbits/1)+2+1;		/* %xxxxx */
+		os=pnt=(char *)calloc_2(1,len);
+		if(show_base) { *(pnt++)='%'; }
+	
+		parse=newbuff+3;
+	
+		for(i=0;i<nbits;i++)
+			{
+			*(pnt++)=AN_STR[(int)(*(parse++))];
+			}
+	
+		*(pnt++)=0x00;
+		}
+	}
 else	/* decimal when all else fails */
 	{
 	char *parse;
@@ -869,8 +966,7 @@ if(p)
 
 	buf[0] = 0;
 
-	fgets(buf, 1024, p->sin);
-	n = strlen(buf);
+	n = fgets(buf, 1024, p->sin) ? strlen(buf) : 0;
 	buf[n] = 0;
 
 	if(n)
@@ -971,7 +1067,7 @@ Ulong flags;
 int nbits;
 char *bits;
 char *pnt, *newbuff;
-int i, j, len;
+int i;
 char xtab[AN_COUNT];
 double mynan = strtod("NaN", NULL);
 double retval = mynan;
@@ -1122,8 +1218,8 @@ double convert_real(Trptr t, vptr v)
 Ulong flags;
 int nbits;
 unsigned char *bits;
-char *pnt, *newbuff;
-int i, j, len;
+char *newbuff;
+int i;
 char xtab[AN_COUNT];
 double mynan = strtod("NaN", NULL);
 double retval = mynan;
@@ -1149,7 +1245,7 @@ if(flags&TR_REVERSE)
 	{
 	char *fwdpnt, *revpnt;
 
-	fwdpnt=bits;
+	fwdpnt=(char *)bits;
 	revpnt=newbuff+nbits+6;
 	for(i=0;i<3;i++) *(--revpnt)=xtab[0];
 	for(i=0;i<nbits;i++)
@@ -1162,7 +1258,7 @@ if(flags&TR_REVERSE)
 	{
 	char *fwdpnt, *fwdpnt2;
 
-	fwdpnt=bits;
+	fwdpnt=(char *)bits;
 	fwdpnt2=newbuff;
 	for(i=0;i<3;i++) *(fwdpnt2++)=xtab[0];
 	for(i=0;i<nbits;i++)
@@ -1229,3 +1325,11 @@ free_2(newbuff);
 return(retval);
 }
 
+/*
+ * $Id: baseconvert.c,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: baseconvert.c,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
