@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2005.
+ * Copyright (c) Tony Bybell 1999-2007.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,8 +7,14 @@
  * of the License, or (at your option) any later version.
  */
 
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include "analyzer.h"
-#include "aet.h"
+#include "symbol.h"
 #include "lxt.h"
 #include "lx2.h"
 #include "lxt2_read.h"
@@ -18,13 +24,151 @@
 #include "strace.h"
 #include "translate.h"
 #include "ptranslate.h"
+#include "main.h"
+#include "busy.h"
 #include <stdlib.h>
 
+/*
+ * attempt to match top vs bottom rather than emit <Vector>
+ */
+char *attempt_vecmatch(char *s1, char *s2)
+{
+char *s;
+char *p1, *p2;
+char *n1=NULL, *n2=NULL;
+int n1len = 0, n2len = 0;
+char *pfx = NULL;
+int pfxlen = 0;
+int pfxstate = 0;
+char *sfx = NULL;
+int sfxlen = 0;
+int totlen;
+int idx;
+
+if((!s1)||(!*s1)||(!s2)||(!*s2)) return(NULL);
+
+if(!strcmp(s1, s2)) { s = malloc_2(strlen(s1)+1); strcpy(s, s1); return(s); }
+
+p1 = s1; p2 = s2;
+while(*p1 && *p2)
+	{
+	if(pfxstate==0)
+		{
+		if(*p1 == *p2)
+			{
+			pfx = p1;
+			pfxlen = p1 - s1 + 1;
+			p1++;
+			p2++;
+			continue;
+			}
+
+		if(!pfx) return(NULL);
+
+		if(isdigit(*p1)&&isdigit(*p2))
+			{
+			n1 = p1; n2 = p2;
+			while(*p1)
+				{
+				if(isdigit(*p1))
+					{
+					n1len++;
+					}
+					else
+					{
+					break;
+					}
+				p1++;
+				}
+
+			while(*p2)
+				{
+				if(isdigit(*p2))
+					{
+					n2len++;
+					}
+					else
+					{
+					break;
+					}
+				p2++;
+				}
+
+			if(*p1 && *p2)
+				{
+				pfxstate = 1;
+				sfx = p1;
+				continue;
+				}
+				else
+				{
+				break;
+				}
+			}
+		}
+
+	if(pfxstate==1)
+		{
+		if(*p1 == *p2)
+			{
+			sfxlen++;
+			p1++;
+			p2++;
+			continue;
+			}
+			else
+			{
+			return(NULL);
+			}
+		}
+
+	return(NULL);
+	}
+
+if((*p1)||(*p2)) return(NULL);
+
+while(pfxlen>1)	/* backup if matching a sequence like 20..24 where the 2 matches outside of the left bracket */
+	{
+	if(isdigit(s1[pfxlen-1]))
+		{
+		pfxlen--;
+		n1--; n1len++;
+		n2--; n2len++;
+		}
+		else
+		{
+		break;
+		}
+	}
+
+totlen = pfxlen + 1 + n1len + 1 + n2len + 1 + sfxlen + 1;
+s = malloc_2(totlen);
+memcpy(s, s1, pfxlen);
+idx = pfxlen;
+s[idx] = '[';
+idx++;
+memcpy(s+idx, n1, n1len);
+idx += n1len;
+s[idx] = ':';
+idx++;
+memcpy(s+idx, n2, n2len);
+idx += n2len;
+s[idx] = ']';
+idx++;
+if(sfxlen) { memcpy(s+idx, sfx, sfxlen); idx+=sfxlen; }
+s[idx] = 0;
+
+return(s);
+}
+
+
 /*
  * mvlfac resolution
  */
 void import_trace(nptr np)
 {
+set_window_busy(NULL);
+
 if(is_lxt)
 	{
 	import_lxt_trace(np);
@@ -39,6 +183,8 @@ else
 	fprintf(stderr, "Internal error with mvlfac trace handling, exiting.\n");
 	exit(255);
 	}
+
+set_window_idle(NULL);
 }
 
 
@@ -214,7 +360,7 @@ return(bitvec);
 /*
  * Make solitary traces from wildcarded signals...
  */
-int maketraces(char *str)
+int maketraces(char *str, int quick_return)
 {
 char *pnt, *wild;
 char ch, wild_active=0;
@@ -291,14 +437,15 @@ if(len)
 	{
 	wild=(char *)calloc_2(1,len+1);
 	memcpy(wild,str,len);
-	wave_regex_compile(wild);
+	wave_regex_compile(wild, WAVE_REGEX_WILD);
 
 	for(i=0;i<numfacs;i++)
 		{
-		if(wave_regex_match(facs[i]->name))
+		if(wave_regex_match(facs[i]->name, WAVE_REGEX_WILD))
 			{
 			AddNode(facs[i]->n,NULL);
 			made=~0;
+			if(quick_return) break;
 			}
 		}
 
@@ -393,10 +540,10 @@ if(len)
 		}
 		else
 		{
-		wave_regex_compile(wild);
+		wave_regex_compile(wild, WAVE_REGEX_WILD);
 		for(i=numfacs-1;i>=0;i--)	/* to keep vectors in little endian hi..lo order */
 			{
-			if(wave_regex_match(facs[i]->name))
+			if(wave_regex_match(facs[i]->name, WAVE_REGEX_WILD))
 				{
 				n[nodepnt++]=facs[i]->n;
 				if(nodepnt==512) { free_2(wild); goto ifnode; }
@@ -720,11 +867,13 @@ if(nodepnt)
 				{
 				if(!b->attribs)
 					{
-					strcpy(b->name=(char *)malloc_2(8+1),"<Vector>");
+					char *aname = attempt_vecmatch(s1, s2);
+					if(aname) b->name = aname; else { strcpy(b->name=(char *)malloc_2(8+1),"<Vector>"); }
 					}
 					else
 					{
-					strcpy(b->name=(char *)malloc_2(15+1),"<ComplexVector>");
+					char *aname = attempt_vecmatch(s1, s2);
+					if(aname) b->name = aname; else { strcpy(b->name=(char *)malloc_2(15+1),"<ComplexVector>"); }
 					}
 				}
 				else
@@ -924,11 +1073,13 @@ if(nodepnt)
 				{
 				if(!b->attribs)
 					{
-					strcpy(b->name=(char *)malloc_2(8+1),"<Vector>");
+					char *aname = attempt_vecmatch(s1, s2);
+					if(aname) b->name = aname; else { strcpy(b->name=(char *)malloc_2(8+1),"<Vector>"); }
 					}
 					else
 					{
-					strcpy(b->name=(char *)malloc_2(15+1),"<ComplexVector>");
+					char *aname = attempt_vecmatch(s1, s2);
+					if(aname) b->name = aname; else { strcpy(b->name=(char *)malloc_2(15+1),"<ComplexVector>"); }
 					}
 				}
 				else
@@ -1139,7 +1290,7 @@ for(;;)
 	c1=(unsigned char)*(s1++);
 	c2=(unsigned char)*(s2++);
 
-	if((!c1)&&(!c2)) return(0);
+	if(!(c1|c2)) return(0);	/* removes extra branch through logical or */
 	if((c1<='9')&&(c2<='9')&&(c2>='0')&&(c1>='0'))
 		{
 		u1=(int)(c1&15);
@@ -1337,7 +1488,8 @@ if((root1len!=root2len)||(!root1len)||(!root2len)||
 	{
 	if(symlo!=symhi)
 		{
-		strcpy(name=(char *)malloc_2(8+1),"<Vector>");
+		char *aname = attempt_vecmatch(s1, s2);
+		if(aname) name = aname; else { strcpy(name=(char *)malloc_2(8+1),"<Vector>"); }
 		}
 		else
 		{
@@ -1803,11 +1955,94 @@ if(n->expansion)
 
 /******************************************************************/
 
+/*
+ * attempt to synthesize bitwise on loader fail...caller must free return pnt
+ */
+static char *synth_blastvec(char *w)
+{
+char *mem = NULL;
+char *t;
+char *lbrack, *colon, *rbrack, *rname, *msbs, *lsbs;
+int wlen, bitlen, msb, lsb;
+int msbslen, lsbslen, maxnumlen;
+int i, siz;
+
+if(w)
+	{
+	if((lbrack = strrchr(w, '[')))
+	if((colon = strchr(lbrack+1, ':')))
+	if((rbrack = strchr(colon+1, ']')))
+		{
+		*lbrack = *colon = *rbrack = 0;
+		msbs = lbrack + 1;
+		lsbs = colon + 1;
+		rname = hier_extract(w, hier_max_level);
+
+		msb = atoi(msbs);
+		lsb = atoi(lsbs);
+		bitlen = (msb > lsb) ? (msb - lsb + 1) : (lsb - msb + 1);
+		if(bitlen > 1)
+			{
+			wlen = strlen(w);
+
+			msbslen = strlen(msbs);
+			lsbslen = strlen(lsbs);
+			maxnumlen = (msbslen > lsbslen) ? msbslen : lsbslen;
+
+			siz = 	1 + 				/* # */
+				strlen(rname) +			/* vector alias name */
+				1+				/*   */
+				1+				/* [ */
+				msbslen+			/* msb */
+				1+				/* : */
+				lsbslen+			/* lsb */
+				1+				/* ] */
+				1;				/*   */
+
+			siz +=  bitlen * (
+				wlen +				/* full bitname */
+				1+				/* [ */
+				maxnumlen+			/* individual bit */
+				1+				/* ] */
+				1				/*   */
+				);
+
+			mem = calloc_2(1, siz);
+			t = mem + sprintf(mem, "#%s[%d:%d] ", rname, msb, lsb);
+			
+			if(msb > lsb)
+				{
+				for(i = msb; i >= lsb; i--)
+					{
+					t += sprintf(t, "%s[%d]", w, i);
+					if(i!=lsb) t += sprintf(t, " ");
+					}
+				}
+				else
+				{
+				for(i = msb; i <= lsb; i++)
+					{
+					t += sprintf(t, "%s[%d]", w, i);
+					if(i!=lsb) t += sprintf(t, " ");
+					}
+				}
+
+			/* fprintf(stderr, "%d,%d: %s\n", siz, strlen(mem), mem); */
+			}
+
+		}
+	}
+
+return(mem);
+}
+
+/******************************************************************/
+
 /*
  * Parse a line of the wave file and act accordingly.. 
  * Returns nonzero if trace(s) added.
  */
-int parsewavline(char *w)
+int parsewavline(char *w, int depth)
 {
 int i;
 int len;
@@ -1971,6 +2206,49 @@ if((*w2=='#')||(*w2==':'))
 			free_2(b);
 			}
 		}
+		else if(!depth) /* don't try vectorized if we're re-entrant */
+		{
+		char *sp = strchr(w2, ' ');
+		char *lbrack;
+
+		if(sp)
+			{
+			*sp = 0;
+
+			lbrack = strrchr(w2, '[');
+
+			if(lbrack)
+				{
+				int made = 0;
+				char *w3;
+				char *rbrack           = strrchr(w2,   ']');
+				char *rightmost_lbrack = strrchr(sp+1, '[');
+
+				if(rbrack && rightmost_lbrack)
+					{
+					*rbrack = 0;
+
+					w3 = malloc_2(strlen(w2) + 1 + strlen(rightmost_lbrack+1) + 1);
+					sprintf(w3, "%s:%s", w2, rightmost_lbrack+1);
+
+					made = maketraces(w3,1);
+					free_2(w3);
+					}
+
+				if(0)	/* this is overkill for now with possible delay implications so commented out */
+				if(!made)
+					{
+					*lbrack = 0;
+					fprintf(stderr, "GTKWAVE | Attempting regex '%s' on missing stranded vector\n", w2);
+
+					w3 = malloc_2(1 + strlen(w2) + 5);
+					sprintf(w3, "^%s\\[.*", w2);
+					maketraces(w3, 1);
+					free_2(w3);
+					}
+				}
+			}
+		}
 	
 	return(v!=NULL);
 	}
@@ -2092,9 +2370,56 @@ else if(*w2=='^')
 			}
 		}
 	}
+else if (*w2 == '[')
+  {
+    /* Search for matching ']'.  */
+    w2++;
+    for (w = w2; *w; w++)
+      if (*w == ']')
+	break;
+    if (!*w)
+      return 0;
+    
+    *w++ = 0;
+    if (strcmp (w2, "size") == 0)
+      {
+	/* Main window size.  */
+	int x, y;
+	sscanf (w, "%d %d", &x, &y);
+	set_window_size (x, y);
+      }
+    else if (strcmp (w2, "pos") == 0)
+      {
+	/* Main window position.  */
+	int x, y;
+	sscanf (w, "%d %d", &x, &y);
+	set_window_xypos (x, y);
+      }
+    else
+      {
+	/* Unknown attribute.  Forget it.  */
+	return 0;
+      }
+  }
 	else
 	{
-	return(maketraces(w));
+	int rc = maketraces(w, 0);
+	if(rc)
+		{
+		return(rc);
+		}
+		else
+		{
+		char *newl = synth_blastvec(w);
+
+		if(newl)
+			{
+			rc = parsewavline(newl, depth+1);
+			free_2(newl);
+			}
+
+		return(rc);
+		}
 	}
 
 return(0);
@@ -2108,12 +2433,13 @@ return(0);
 /*
  * Make solitary traces from wildcarded signals...
  */
-void maketraces_lx2(char *str)
+int maketraces_lx2(char *str, int quick_return)
 {
 char *pnt, *wild;
 char ch, wild_active=0;
 int len;
 int i;
+int made = 0;
 
 pnt=str;
 while((ch=*pnt))
@@ -2134,25 +2460,27 @@ if(!wild_active)	/* short circuit wildcard evaluation with bsearch */
 		{
 		for(i=1;;i++)
 			{
-			if(str[i]==0) return;
+			if(str[i]==0) return(0);
 			if((str[i]==')')&&(str[i+1])) {i++; goto maketracespt2; }
 			}
-		return;
+		return(made);
 
 maketracespt2:
 		if((s=symfind(str+i)))
 			{
 			lx2_set_fac_process_mask(s->n);
+			made = ~0;
 			}
-		return;
+		return(made);
 		}
 		else
 		{
 		if((s=symfind(str)))
 			{
 			lx2_set_fac_process_mask(s->n);
+			made = ~0;
 			}
-		return;
+		return(made);
 		}
 	}
 
@@ -2172,13 +2500,15 @@ if(len)
 	{
 	wild=(char *)calloc_2(1,len+1);
 	memcpy(wild,str,len);
-	wave_regex_compile(wild);
+	wave_regex_compile(wild, WAVE_REGEX_WILD);
 
 	for(i=0;i<numfacs;i++)
 		{
-		if(wave_regex_match(facs[i]->name))
+		if(wave_regex_match(facs[i]->name, WAVE_REGEX_WILD))
 			{
 			lx2_set_fac_process_mask(facs[i]->n);
+			made = ~0;
+			if(quick_return) break;
 			}
 		}
 
@@ -2188,19 +2518,20 @@ if(len)
 if(!ch) break;
 str=pnt;
 }
-return;
+return(made);
 }
 
 
 /*
  * Create a vector from wildcarded signals...
  */
-void makevec_lx2(char *str)
+int makevec_lx2(char *str)
 {
 char *pnt, *pnt2, *wild=NULL;
 char ch, ch2, wild_active;
 int len;
 int i;
+int rc = 0;
 
 while(1)
 {
@@ -2248,6 +2579,7 @@ if(len)
 					if(s)
 						{
 						lx2_set_fac_process_mask(s->n);
+						rc = 1;
 						}
 					break;
 					}
@@ -2258,17 +2590,19 @@ if(len)
 			if((s=symfind(wild)))	
 				{
 				lx2_set_fac_process_mask(s->n);
+				rc = 1;
 				}
 			}
 		}
 		else
 		{
-		wave_regex_compile(wild);
+		wave_regex_compile(wild, WAVE_REGEX_WILD);
 		for(i=numfacs-1;i>=0;i--)	/* to keep vectors in little endian hi..lo order */
 			{
-			if(wave_regex_match(facs[i]->name))
+			if(wave_regex_match(facs[i]->name, WAVE_REGEX_WILD))
 				{
 				lx2_set_fac_process_mask(facs[i]->n);
+				rc = 1;
 				}
 			}
 		}
@@ -2278,6 +2612,8 @@ if(len)
 if(!ch) break;
 str=pnt;
 }
+
+return(rc);
 }
 
 
@@ -2285,19 +2621,20 @@ str=pnt;
  * Parse a line of the wave file and act accordingly.. 
  * Returns nonzero if trace(s) added.
  */
-void parsewavline_lx2(char *w)
+int parsewavline_lx2(char *w, int depth)
 {
+int made = 0;
 int i;
 int len;
 char *prefix, *suffix;
 char *w2;
 
-if(!(len=strlen(w))) return;
+if(!(len=strlen(w))) return(0);
 if(*(w+len-1)=='\n')
 	{
 	*(w+len-1)=0x00; /* strip newline if present */
 	len--;
-	if(!len) return;
+	if(!len) return(0);
 	}
 
 prefix=(char *)wave_alloca(len+1);
@@ -2307,7 +2644,7 @@ w2=w;
 while(1)
 {
 if(isspace(*w2)) { w2++; continue; }
-if(!(*w2)) return;	/* no args */
+if(!(*w2)) return(0);	/* no args */
 break;			/* start grabbing chars from here */
 }
 
@@ -2338,42 +2675,88 @@ if(*w2=='+')
 		{
 		for(i=1;;i++)
 			{
-			if(suffix[i]==0) return;
+			if(suffix[i]==0) return(0);
 			if((suffix[i]==')')&&(suffix[i+1])) {i++; goto aliaspt2; }
 			}
-		return;
+		return(0);
 
 aliaspt2:
 		s=symfind(suffix+i);
 		if(s)
 			{
 			lx2_set_fac_process_mask(s->n);
+			made = ~0;
 			}
-		return;
+		return(made);
 		}
 		else
 		{
 		if((s=symfind(suffix)))
 			{
 			lx2_set_fac_process_mask(s->n);
+			made = ~0;
 			}
-		return;
+		return(made);
 		}
 	}
 else
 if((*w2=='#')||(*w2==':'))
 	{
+	int rc;
+
 	/* handle bitvec, parsing extra time info and such is inefficient but ok for ":" case */
 	w2=w2+strlen(prefix);
 	while(1)
 		{
 		if(isspace(*w2)) { w2++; continue; }
-		if(!(*w2)) return;	/* no more args */	
+		if(!(*w2)) return(0);	/* no more args */	
 		break;			/* start grabbing chars from here */
 		}
 
-	makevec_lx2(w2);
-	return;
+	rc = makevec_lx2(w2);
+	if((!rc)&&(!depth))		/* don't try vectorized if we're re-entrant */
+		{
+		char *sp = strchr(w2, ' ');
+		char *lbrack;
+
+		if(sp)
+			{
+			*sp = 0;
+
+			lbrack = strrchr(w2, '[');
+
+			if(lbrack)
+				{
+				char *w3;
+				char *rbrack           = strrchr(w2,   ']');
+				char *rightmost_lbrack = strrchr(sp+1, '[');
+
+				if(rbrack && rightmost_lbrack)
+					{
+					*rbrack = 0;
+
+					w3 = malloc_2(strlen(w2) + 1 + strlen(rightmost_lbrack+1) + 1);
+					sprintf(w3, "%s:%s", w2, rightmost_lbrack+1);
+
+					made = maketraces_lx2(w3,1);
+					free_2(w3);
+					}
+
+				if(0)	/* this is overkill for now with possible delay implications so commented out */
+				if(!made)
+					{
+					*lbrack = 0;
+
+					w3 = malloc_2(1 + strlen(w2) + 5);
+					sprintf(w3, "^%s\\[.*", w2);
+					maketraces_lx2(w3, 1);
+					free_2(w3);
+					}
+				}
+			}
+		}
+
+	return(made);
 	}
 else
 if(*w2=='!')
@@ -2388,10 +2771,20 @@ else if(*w2=='^')
 	}
 	else
 	{
-	maketraces_lx2(w);
+        made = maketraces_lx2(w, 0);
+        if(!made)
+                {
+                char *newl = synth_blastvec(w);
+
+		if(newl)
+			{
+	                made = parsewavline_lx2(newl, depth+1);
+	                free_2(newl);
+			}
+                }
 	}
 
-return;
+return(made);
 }
 
 
@@ -2399,3 +2792,11 @@ return;
 /* LX2 variants */
 /****************/
 
+/*
+ * $Id: bitvec.c,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: bitvec.c,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
