@@ -7,9 +7,10 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include "analyzer.h"
 #include "currenttime.h"
-#include "aet.h"
+#include "symbol.h"
 #include "bsearch.h"
 #include "strace.h"
 
@@ -43,11 +44,13 @@ return(rv);
 
 int bsearch_timechain(TimeType key)
 {
+void *bsearch_dummy_rv;
+
 max_compare_time_tc=-2+shift_timebase; max_compare_pos_tc=NULL; 
 
 if(!timearray) return(-1);
 
-bsearch(&key, timearray, timearray_size, sizeof(TimeType), compar_timechain);
+bsearch_dummy_rv = bsearch(&key, timearray, timearray_size, sizeof(TimeType), compar_timechain);
 if((!max_compare_pos_tc)||(max_compare_time_tc<shift_timebase)) 
 	{
 	max_compare_pos_tc=timearray; /* aix bsearch fix */
@@ -87,9 +90,11 @@ return(rv);
 
 hptr bsearch_node(nptr n, TimeType key)
 {
+void *bsearch_dummy_rv;
+
 max_compare_time=-2+shift_timebase; max_compare_pos=NULL; max_compare_index=NULL;
 
-bsearch(&key, n->harray, n->numhist, sizeof(hptr), compar_histent);
+bsearch_dummy_rv = bsearch(&key, n->harray, n->numhist, sizeof(hptr), compar_histent);
 if((!max_compare_pos)||(max_compare_time<shift_timebase)) 
 	{
 	max_compare_pos=n->harray[1]; /* aix bsearch fix */
@@ -131,9 +136,11 @@ return(rv);
 
 vptr bsearch_vector(bvptr b, TimeType key)
 {
+void *bsearch_dummy_rv;
+
 vmax_compare_time=-2+shift_timebase; vmax_compare_pos=NULL; vmax_compare_index=NULL;
 
-bsearch(&key, b->vectors, b->numregions, sizeof(vptr), compar_vectorent);
+bsearch_dummy_rv = bsearch(&key, b->vectors, b->numregions, sizeof(vptr), compar_vectorent);
 if((!vmax_compare_pos)||(vmax_compare_time<shift_timebase)) 
 	{
 	vmax_compare_pos=b->vectors[1]; /* aix bsearch fix */
@@ -178,13 +185,14 @@ return(key-obj);
 
 char *bsearch_trunc(char *ascii, int maxlen)
 {
+void *bsearch_dummy_rv;
 int len;
 
 if((maxlen<=0)||(!ascii)||(!(len=strlen(ascii)))) return(NULL);
 
 maxlen_trunc=0; maxlen_trunc_pos=NULL;
 
-bsearch(&maxlen, trunc_asciibase=ascii, len, sizeof(char), compar_trunc);
+bsearch_dummy_rv = bsearch(&maxlen, trunc_asciibase=ascii, len, sizeof(char), compar_trunc);
 return(maxlen_trunc_pos);
 }
 
@@ -208,3 +216,12 @@ if ((!ascii)||(!strlen(ascii))) return(NULL);
 rc=(struct symbol **)bsearch(ascii, facs, numfacs, sizeof(struct symbol *), compar_facs);
 if(rc) return(*rc); else return(NULL);
 }
+
+/*
+ * $Id: bsearch.c,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: bsearch.c,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
