@@ -19,9 +19,16 @@ struct symbol *bsearch_facs(char *ascii);
 extern int maxlen_trunc;
 extern TimeType shift_timebase, shift_timebase_default_for_add;
 
-hptr *max_compare_index;
-vptr *vmax_compare_index;
+extern hptr *max_compare_index;
+extern vptr *vmax_compare_index;
 
 #endif
 
+/*
+ * $Id: bsearch.h,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: bsearch.h,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
 
