@@ -39,7 +39,23 @@ extern int color_w;
 extern int color_dashfill;
 extern int color_dash;
 
+extern int color_white;
+extern int color_black;
+extern int color_ltgray;
+extern int color_normal;
+extern int color_mdgray;
+extern int color_dkgray;
+extern int color_dkblue;
+
 GdkGC *alloc_color(GtkWidget *widget, int tuple, GdkGC *fallback);	/* tuple is encoded as 32bit: --RRGGBB (>=0 is valid) */
 
 #endif
 
+/*
+ * $Id: color.h,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: color.h,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
