@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2005.
+ * Copyright (c) Tony Bybell 1999-2007.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,13 +7,15 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <gtk/gtk.h>
 #include "currenttime.h"
-#include "aet.h"
+#include "symbol.h"
 
-char is_vcd=0;
+char is_vcd=0, partial_vcd=0;
 
 char use_maxtime_display=1;
+char use_frequency_delta=0;
 static GtkWidget *max_or_marker_label=NULL;
 static GtkWidget *base_or_curtime_label=NULL;
    
@@ -118,6 +120,30 @@ if(i)
 }
 
 
+void reformat_time_as_frequency(char *buf, TimeType val, char dim)
+{
+char *pnt;
+int offset;
+double k;
+static const double negpow[] = { 1.0, 1.0e-3, 1.0e-6, 1.0e-9, 1.0e-12, 1.0e-15 };
+
+pnt=strchr(time_prefix, (int)dim);
+if(pnt) { offset=pnt-time_prefix; } else offset=0;
+
+if(val)
+	{
+	k = 1 / ((double)val * negpow[offset]);
+
+	sprintf(buf, "%e Hz", k);
+	}
+	else
+	{
+	strcpy(buf, "-- Hz");
+	}
+
+}
+
+
 void reformat_time_blackout(char *buf, TimeType val, char dim)
 {
 char *pnt;
@@ -158,6 +184,24 @@ if(i)
 
 void update_markertime(TimeType val)
 {
+#if !defined _MSC_VER && !defined __MINGW32__ 
+if(anno_ctx)
+	{
+	if(val >= 0)
+		{
+		anno_ctx->marker_set = 0;	/* avoid race on update */
+		anno_ctx->marker = val / time_scale;
+		reformat_time(anno_ctx->time_string, val, time_dimension);
+
+		anno_ctx->marker_set = 1;
+		}
+		else
+		{
+		anno_ctx->marker_set = 0;
+		}
+	}
+#endif
+
 if(!use_maxtime_display)
 	{
 	if(val>=0)
@@ -169,18 +213,47 @@ if(!use_maxtime_display)
 			if(val>=0)
 				{
 				*(maxtext+1)='+';
-				reformat_time(maxtext+2, val, time_dimension);
+				if(use_frequency_delta)
+					{
+					reformat_time_as_frequency(maxtext+2, val, time_dimension);
+					}
+					else
+					{
+					reformat_time(maxtext+2, val, time_dimension);
+					}
 				}
 				else
 				{
-				reformat_time(maxtext+1, val, time_dimension);
+				if(use_frequency_delta)
+					{
+					reformat_time_as_frequency(maxtext+1, val, time_dimension);
+					}
+					else
+					{
+					reformat_time(maxtext+1, val, time_dimension);
+					}
 				}
 			}
 		else if(tims.lmbcache>=0) 
 			{
 			val-=tims.lmbcache; /* do delta instead */
-			*maxtext='+';
-			reformat_time(maxtext+1, val, time_dimension);
+
+			if(use_frequency_delta)
+				{
+				reformat_time_as_frequency(maxtext, val, time_dimension);
+				}
+			else
+				{
+				if(val>=0)
+					{
+					*maxtext='+';
+					reformat_time(maxtext+1, val, time_dimension);
+					}
+					else
+					{
+					reformat_time(maxtext, val, time_dimension);
+					}
+				}
 			}
 		else
 			{
@@ -244,6 +317,7 @@ GtkWidget *
 create_time_box(void)
 {
 GtkWidget *mainbox;
+GtkWidget *eventbox;
 
 max_or_marker_label=(use_maxtime_display)
 	? gtk_label_new(maxtime_label_text)
@@ -276,6 +350,8 @@ if(tims.baseline<0)
 	}
 
 mainbox=gtk_vbox_new(FALSE, 0);
+eventbox=gtk_event_box_new();
+gtk_container_add(GTK_CONTAINER(eventbox), mainbox);
 
 gtk_box_pack_start(GTK_BOX(mainbox), max_or_marker_label, TRUE, FALSE, 0);
 gtk_widget_show(max_or_marker_label);
@@ -286,8 +362,9 @@ gtk_box_pack_start(GTK_BOX(mainbox), base_or_curtime_label, TRUE, FALSE, 0);
 gtk_widget_show(base_or_curtime_label);
 gtk_box_pack_start(GTK_BOX(mainbox), curtimewid, TRUE, FALSE, 0);
 gtk_widget_show(curtimewid);
-   
-return(mainbox);
+gtk_widget_show(mainbox);   
+
+return(eventbox);
 }
    
 
@@ -357,3 +434,12 @@ switch(scale)
         default:                                        time_dimension = 'n'; break;
         }
 }
+
+/*
+ * $Id: currenttime.c,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: currenttime.c,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
