@@ -12,6 +12,7 @@
  * debug.c 01feb99ajb
  * malloc debugs added on 13jul99ajb
  */
+#include <config.h>
 #include "debug.h"
 
 #undef free_2
@@ -287,3 +288,12 @@ return(tmpspace);
 
 #endif
 }
+
+/*
+ * $Id: debug.c,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: debug.c,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
