@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2006.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -10,17 +10,20 @@
 #ifndef FGET_DYNAMIC_H
 #define FGET_DYNAMIC_H
 
-#include <stdio.h>
-#include <stdlib.h>
-#include "alloca.h"
-
-struct alloc_bytechain
-{
-char val;
-struct alloc_bytechain *next;
-};
+#include "vlist.h"
 
 char *fgetmalloc(FILE *handle);
+char *fgetmalloc_stripspaces(FILE *handle);
+
 extern int fgetmalloc_len;
 
 #endif
+
+/*
+ * $Id: fgetdynamic.h,v 1.2 2007/04/20 02:08:11 gtkwave Exp $
+ * $Log: fgetdynamic.h,v $
+ * Revision 1.2  2007/04/20 02:08:11  gtkwave
+ * initial release
+ *
+ */
+
