@@ -1,5 +1,5 @@
 /*  GHDL Wavefile reader interface.
-    Copyright (C) 2005 Tristan Gingold and Tony Bybell
+    Copyright (C) 2005-2006 Tristan Gingold and Tony Bybell
 
     GHDL is free software; you can redistribute it and/or modify it under
     the terms of the GNU General Public License as published by the Free
@@ -17,10 +17,23 @@
     02111-1307, USA.
 */
 
+#include <config.h>
 #include "ghw.h"
 
 /************************ splay ************************/
 
+/* 
+ * NOTE: 
+ * a GHW tree's "which" is not the same as a gtkwave "which"
+ * in that gtkwave's points to the facs[] array and
+ * GHW's functions as an alias handle.  The following
+ * vars are used to resolve those differences...
+ */
+static struct Node **nxp;
+static struct symbol *sym_head = NULL, *sym_curr = NULL;
+static int sym_which = 0;
+
+
 /*
  * pointer splay
  */
@@ -93,7 +106,7 @@ static ghw_Tree * ghw_insert(void *i, ghw_Tree * t, int val, struct symbol *sym)
     ghw_Tree * n;
     int dir;
     
-    n = (ghw_Tree *) calloc(1, sizeof (ghw_Tree));
+    n = (ghw_Tree *) calloc_2(1, sizeof (ghw_Tree));
     if (n == NULL) {
 	fprintf(stderr, "ghw_insert: ran out of memory, exiting.\n");
 	exit(255);
@@ -119,7 +132,7 @@ static ghw_Tree * ghw_insert(void *i, ghw_Tree * t, int val, struct symbol *sym)
 	return n;
     } else { /* We get here if it's already in the tree */
              /* Don't add it again                      */
-	free(n);
+	free_2(n);
 	return t;
     }
 }
@@ -157,8 +170,8 @@ for(i=0;i<numfacs;i++)
 	{
 	if(psr)
 		{
-		char *fstr1 = facs[i-1]->n->nname;
-		char *fstr2 = facs[i]->n->nname;
+		char *fstr1 = facs[i-1]->name;
+		char *fstr2 = facs[i]->name;
 		int p1 = strand_pnt(fstr1);
 		int p2 = strand_pnt(fstr2);
 
@@ -241,7 +254,7 @@ static void incinerate_whichcache_tree(ghw_Tree *t)
 if(t->left) incinerate_whichcache_tree(t->left);
 if(t->right) incinerate_whichcache_tree(t->right);
 
-free(t);
+free_2(t);
 }
 
 
@@ -263,7 +276,7 @@ for(i=0;i<numfacs;i++)
         if((len=strlen(subst=curnode->name))>longestname) longestname=len;
         while((ch=(*subst)))
                 { 
-                if(ch==hier_delimeter) { *subst=0x01; } /* forces sort at hier boundaries */
+                if(ch==hier_delimeter) { *subst=VCDNAM_HIERSORT; } /* forces sort at hier boundaries */
                 subst++;
                 }
         }
@@ -286,7 +299,7 @@ for(i=0;i<numfacs;i++)
         subst=facs[i]->name;
         while((ch=(*subst)))
                 {
-                if(ch==0x01) { *subst=hier_delimeter; } /* restore back to normal */
+                if(ch==VCDNAM_HIERSORT) { *subst=hier_delimeter; } /* restore back to normal */
                 subst++;
                 }
         }
@@ -347,7 +360,7 @@ build_hierarchy_array (struct ghw_handler *h, union ghw_type *arr, int dim,
 	    sprintf(asbuf, "%s%c%d", pfx, dim == 0 ? '[' : ',', v);
             name = strdup(asbuf);
 	    build_hierarchy_array (h, arr, dim + 1, name, res, sig);
-	    free(name);
+	    free_2(name);
 	    if (v == r->right)
 	      break;
 	    if (r->dir == 0)
@@ -357,6 +370,33 @@ build_hierarchy_array (struct ghw_handler *h, union ghw_type *arr, int dim,
 	  }
       }
       return;
+
+    case ghdl_rtik_type_e8:
+      {
+	int32_t v;
+	char *name;
+	struct ghw_range_e8 *r;
+	struct tree *last;
+	
+	last = NULL;
+	r = &arr->sa.rngs[dim]->e8;
+	v = r->left;
+	while (1)
+	  {
+	    sprintf(asbuf, "%s%c%d", pfx, dim == 0 ? '[' : ',', v);
+            name = strdup(asbuf);
+	    build_hierarchy_array (h, arr, dim + 1, name, res, sig);
+	    free_2(name);
+	    if (v == r->right)
+	      break;
+	    if (r->dir == 0)
+	      v++;
+	    else
+	      v--;
+	  }
+      }
+      return;
+
     default:
       fprintf (stderr, "build_hierarchy_array: unhandled type %d\n",
 	       idx->kind);
@@ -369,6 +409,7 @@ build_hierarchy_type (struct ghw_handler *h, union ghw_type *t,
 		      const char *pfx, unsigned int **sig)
 {
   struct tree *res;
+  struct symbol *s;
 
   switch (t->kind)
     {
@@ -381,17 +422,30 @@ build_hierarchy_type (struct ghw_handler *h, union ghw_type *t,
     case ghdl_rtik_type_i64:
     case ghdl_rtik_type_p32:
     case ghdl_rtik_type_p64:
+
+      s = calloc(1, sizeof(struct symbol));
+      if(!sym_head)
+		{
+		sym_head = sym_curr = s;
+		}
+		else
+		{
+		sym_curr->next = s; sym_curr = s;
+		}
+
       nbr_sig_ref++;
-      res = (struct tree *) calloc(1, sizeof (struct tree));
-      res->name = (char *)pfx;
+      res = (struct tree *) calloc_2(1, sizeof (struct tree) + strlen(pfx));
+      strcpy(res->name, (char *)pfx);
       res->which = *(*sig)++;
+
+      s->n = nxp[res->which];
       return res;
     case ghdl_rtik_subtype_array:
     case ghdl_rtik_subtype_array_ptr:
       {
 	struct tree *r;
-	res = (struct tree *) calloc(1, sizeof (struct tree));
-	res->name = (char *)pfx;
+	res = (struct tree *) calloc_2(1, sizeof (struct tree) + strlen(pfx));
+	strcpy(res->name, (char *)pfx);
 	res->which = -1;
 	r = res;
 	build_hierarchy_array (h, t, 0, "", &res, sig);
@@ -405,8 +459,8 @@ build_hierarchy_type (struct ghw_handler *h, union ghw_type *t,
 	struct tree *c;
 	int i;
 
-	res = (struct tree *) calloc(1, sizeof (struct tree));
-	res->name = (char *)pfx;
+	res = (struct tree *) calloc_2(1, sizeof (struct tree) + strlen(pfx));
+	strcpy(res->name, (char *)pfx);
 	res->which = -1;
 
 	last = NULL;
@@ -444,25 +498,41 @@ build_hierarchy (struct ghw_handler *h, struct ghw_hie *hie)
     case ghw_hie_generate_for:
     case ghw_hie_generate_if:
     case ghw_hie_package:
-      t = (struct tree *) calloc(1, sizeof (struct tree));
       if (hie->kind == ghw_hie_generate_for)
 	{
 	  char buf[128];
 	  int name_len, buf_len;
+	  char *n;
 
 	  ghw_get_value (buf, sizeof (buf),
 			 hie->u.blk.iter_value, hie->u.blk.iter_type);
 	  name_len = strlen (hie->name);
 	  buf_len = strlen (buf);
-	  t->name = malloc(2 + buf_len + name_len + 1);
-	  memcpy (t->name, hie->name, name_len);
-	  t->name[name_len] = '[';
-	  memcpy (t->name + name_len + 1, buf, buf_len);
-	  t->name[name_len + buf_len + 1] = ']';
-	  t->name[name_len + buf_len + 2] = 0;
+
+          t = (struct tree *) calloc_2(1, sizeof (struct tree) + (2 + buf_len + name_len));
+	  n = t->name;
+
+	  memcpy (n, hie->name, name_len);
+	  n += name_len;
+	  *n++ = '[';
+	  memcpy (n, buf, buf_len);
+	  n += buf_len;
+	  *n++ = ']';
+	  *n = 0;
 	}
       else
-	t->name = (char *)hie->name;
+        {
+          if(hie->name)
+		{
+          	t = (struct tree *) calloc_2(1, sizeof (struct tree) + strlen(hie->name));
+          	strcpy(t->name, (char *)hie->name);
+		}
+		else
+		{
+          	t = (struct tree *) calloc_2(1, sizeof (struct tree));
+		}
+        }
+
       t->which = -1;
       prev = NULL;
       for (ch = hie->u.blk.child; ch != NULL; ch = ch->brother)
@@ -526,26 +596,23 @@ static void
 create_facs (struct ghw_handler *h)
 {
   int i;
+  struct symbol *s = sym_head;
 
-  nbr_sig_ref++;
   numfacs = nbr_sig_ref;
-  facs=(struct symbol **)malloc(numfacs*sizeof(struct symbol *));
+  facs=(struct symbol **)malloc_2(numfacs*sizeof(struct symbol *));
 
-  for (i = 0; i < nbr_sig_ref; i++)
+  i = 0;
+  while(s)
+	{
+	facs[i++] = s;
+	s = s->next;
+	}
+
+  for (i = 0; i < h->nbr_sigs; i++)
     {
-      struct symbol *s;
-      struct Node *n;
+      struct Node *n = nxp[i];
 
-      /* Create symbol.  */
-      s = (struct symbol *)calloc(1,sizeof(struct symbol));
-      facs[i] = s;
-      s->name = NULL;
-
-      /* Create node.  */
-      n = (struct Node *)calloc(1,sizeof(struct Node));
-      s->n = n;
-      n->nname = s->name;
-      if (i < h->nbr_sigs && h->sigs[i].type)
+      if (h->sigs[i].type)
 	switch (h->sigs[i].type->kind)
 	  {
 	  case ghdl_rtik_type_b2:
@@ -577,19 +644,6 @@ create_facs (struct ghw_handler *h)
 	  }
     }
 
-facs[0]->name = WAVE_GHW_DUMMYFACNAME;
-facs[0]->n->nname = facs[0]->name;
-if(!facs[0]->n->curr)
-    	{
-      	hptr he=histent_calloc();
-      	he->flags = 0;
-      	he->time=-1;
-      	he->v.h_vector=NULL;
-
-      	facs[0]->n->head.next=he;
-      	facs[0]->n->curr=he;
-      	facs[0]->n->head.time = -2;
-    	}
 }
 
 static char *fac_name;
@@ -597,6 +651,8 @@ static int fac_name_len;
 static int fac_name_max;
 static int last_fac;
 
+
+
 static void
 set_fac_name_1 (struct tree *t)
 {
@@ -605,7 +661,7 @@ set_fac_name_1 (struct tree *t)
       int prev_len = fac_name_len;
 
       /* Complete the name.  */
-      if (t->name != NULL)
+      if(t->name[0]) /* originally (t->name != NULL) when using pointers */
 	{
 	  int len;
 	  
@@ -615,7 +671,7 @@ set_fac_name_1 (struct tree *t)
 	      fac_name_max *= 2;
 	      if (fac_name_max <= len + fac_name_len)
 		fac_name_max = len + fac_name_len + 1;
-	      fac_name = realloc(fac_name, fac_name_max);
+	      fac_name = realloc_2(fac_name, fac_name_max);
 	    }
           if(t->name[0] != '[')
 		{
@@ -633,32 +689,15 @@ set_fac_name_1 (struct tree *t)
 
       if (t->which >= 0)
 	{
-	  struct symbol *orig;
-	  
-	  /* Set a name to a fac, or duplicate a fac.  */
-	  orig = facs[t->which];
-	  if (orig->name == NULL)
-	    {
-	      struct symbol *s;
+        struct symbol *s = sym_head;
 
-	      orig->name = strdup (fac_name);
-	      orig->n->nname = orig->name;
-	      s = symadd (fac_name, hash (fac_name));
-	      s->n = facs[t->which]->n;
-	    }
-	  else
-	    {
-	      struct symbol *ns = facs[last_fac];
+	s->name = strdup (fac_name);
+	s->n = nxp[t->which];
+	if(!s->n->nname) s->n->nname = s->name;
 
-	      ns->name = strdup (fac_name);
-	      ns->n->nname = ns->name;
-	      ns->n->ext = orig->n->ext;
-	      ns->n->head = orig->n->head;
-	      ns->n->curr = orig->n->curr;
+	t->which = sym_which++; /* patch in gtkwave "which" as node is correct */
 
-	      t->which = last_fac;
-	      last_fac++;
-	    }
+	sym_head = sym_head->next;
 	}
 
       if (t->child)
@@ -676,7 +715,7 @@ set_fac_name (struct ghw_handler *h)
   if (fac_name_max == 0)
     {
       fac_name_max = 1024;
-      fac_name = malloc(fac_name_max);
+      fac_name = malloc_2(fac_name_max);
     }
   fac_name_len = 3;
   memcpy (fac_name, "top", 4);
@@ -685,9 +724,8 @@ set_fac_name (struct ghw_handler *h)
 }
 
 static void
-add_history (struct ghw_handler *h, struct symbol *s, int sig_num)
+add_history (struct ghw_handler *h, struct Node *n, int sig_num)
 {
-  struct Node *n = s->n;
   struct HistEnt *he;
   struct ghw_sig *sig = &h->sigs[sig_num];
   union ghw_type *sig_type = sig->type;
@@ -785,7 +823,7 @@ add_history (struct ghw_handler *h, struct symbol *s, int sig_num)
       break;
     case ghdl_rtik_type_f64:
       {
-	double *d = malloc(sizeof (double));
+	double *d = malloc_2(sizeof (double));
 	*d = sig->val->f64;
 	he->v.h_vector = (char *)d;
         is_vector = 1;
@@ -821,7 +859,7 @@ add_history (struct ghw_handler *h, struct symbol *s, int sig_num)
                 {
                 if(n->curr->v.h_vector &&
                    sig_type->kind != ghdl_rtik_type_b2 &&
-                   sig_type->kind != ghdl_rtik_type_e8) free(n->curr->v.h_vector);
+                   sig_type->kind != ghdl_rtik_type_e8) free_2(n->curr->v.h_vector);
                 n->curr->v.h_vector = he->v.h_vector;
                 /* can't free up this "he" because of block allocation so assume it's dead */
                 }
@@ -831,6 +869,17 @@ add_history (struct ghw_handler *h, struct symbol *s, int sig_num)
 		}
 	return;
 	}
+	else /* look for duplicate dumps of same value at adjacent times */
+	{
+	if(!is_vector)
+		{
+		if(n->curr->v.h_val == he->v.h_val)
+			{
+			return;
+	                /* can't free up this "he" because of block allocation so assume it's dead */
+			}
+		}
+	}
 
   n->curr->next=he;
   n->curr=he;
@@ -846,8 +895,7 @@ add_tail (struct ghw_handler *h)
   for (i = 0; i < h->nbr_sigs; i++)
     {
       struct ghw_sig *sig = &h->sigs[i];
-      struct symbol *s = facs[i];
-      struct Node *n = s->n;
+      struct Node *n = nxp[i];
       struct HistEnt *he;
       
       if (sig->type == NULL || n == NULL || !n->curr)
@@ -872,7 +920,7 @@ read_traces (struct ghw_handler *h)
   int i;
   enum ghw_res res;
   
-  list = malloc((numfacs + 1) * sizeof (int));
+  list = malloc_2((numfacs + 1) * sizeof (int));
   
   while (1)
     {
@@ -881,7 +929,7 @@ read_traces (struct ghw_handler *h)
 	{
 	case ghw_res_error:
 	case ghw_res_eof:
-	  free(list);
+	  free_2(list);
 	  return;
 	case ghw_res_ok:
 	case ghw_res_other:
@@ -892,7 +940,7 @@ read_traces (struct ghw_handler *h)
 	  /* printf ("Time is "TTFormat"\n", h->snap_time); */
 
 	  for (i = 0; i < h->nbr_sigs; i++)
-	    add_history (h, facs[i], i);
+	    add_history (h, nxp[i], i);
 	  break;
 	case ghw_res_cycle:
 	  while (1)
@@ -906,7 +954,7 @@ read_traces (struct ghw_handler *h)
 		    max_time = h->snap_time;
 		  
 		  for (i = 0; (sig = list[i]) != 0; i++)
-		    add_history (h, facs[sig], sig);
+		    add_history (h, nxp[sig], sig);
 		}
 	      res = ghw_read_cycle_next (h);
 	      if (res != 1)
@@ -931,6 +979,8 @@ TimeType
 ghw_main(char *fname)
 {
   struct ghw_handler handle;
+  int i;
+  int rc;
 
   if(!hier_was_explicitly_set) /* set default hierarchy split char */
     {
@@ -938,9 +988,9 @@ ghw_main(char *fname)
     }
 
  handle.flag_verbose = 0;
- if (ghw_open (&handle, fname) < 0)
+ if ((rc=ghw_open (&handle, fname)) < 0)
    {
-     fprintf (stderr, "Error opening ghw file '%s'.\n", fname);
+     fprintf (stderr, "Error opening ghw file '%s', rc=%d.\n", fname, rc);
      exit(1);
    }
 
@@ -957,14 +1007,46 @@ ghw_main(char *fname)
  max_time = 0;
 
  nbr_sig_ref = 0;
+
+ nxp =(struct Node **)calloc_2(handle.nbr_sigs, sizeof(struct Node *));
+ for(i=0;i<handle.nbr_sigs;i++)
+	{
+        nxp[i] = (struct Node *)calloc_2(1,sizeof(struct Node));
+	}
+
  treeroot = build_hierarchy (&handle, handle.hie);
  /* GHW does not contains a 'top' name.
     FIXME: should use basename of the file.  */
+
  create_facs (&handle);
  read_traces (&handle);
  add_tail (&handle);
+ 
  set_fac_name (&handle);
- treeroot->name = "top";
+ free(nxp); nxp = NULL;
+
+ /* fix up names on aliased nodes via cloning... */
+ for(i=0;i<numfacs;i++)
+	{
+	if(strcmp(facs[i]->name, facs[i]->n->nname))
+		{
+		struct Node *n = malloc(sizeof(struct Node));
+		memcpy(n, facs[i]->n, sizeof(struct Node));
+		facs[i]->n = n;
+		n->nname = facs[i]->name;
+		}
+	}
+
+ /* treeroot->name = "top"; */
+ {
+ const char *base_hier = "top";
+
+ struct tree *t = calloc_2(1, sizeof(struct tree) + strlen(base_hier));
+ memcpy(t, treeroot, sizeof(struct tree));
+ strcpy(t->name, base_hier);
+ free_2(treeroot);
+ treeroot = t;
+ }
 
  ghw_close (&handle);
 
@@ -988,3 +1070,11 @@ ghw_main(char *fname)
 
 /*******************************************************************************/
 
+/*
+ * $Id: ghw.c,v 1.2 2007/04/20 02:08:12 gtkwave Exp $
+ * $Log: ghw.c,v $
+ * Revision 1.2  2007/04/20 02:08:12  gtkwave
+ * initial release
+ *
+ */
+
