@@ -17,6 +17,7 @@
     02111-1307, USA.
 */
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -24,6 +25,12 @@
 
 #include "ghwlib.h"
 
+#if HAVE_TRIO
+#include <trio.h>
+#define snprintf trio_snprintf
+#define printf trio_printf
+#endif
+
 int
 ghw_open (struct ghw_handler *h, const char *filename)
 {
@@ -35,7 +42,43 @@ ghw_open (struct ghw_handler *h, const char *filename)
 
   if (fread (hdr, sizeof (hdr), 1, h->stream) != 1)
     return -1;
-  /* Check magic.  */
+
+  /* Check magic...also check if there is a compression layer on top of the file */
+  if (!memcmp (hdr, "\x1f\x8b", 2))
+	{
+	char *p = malloc(strlen("gzip -cd ") + strlen(filename) + 1);
+	sprintf(p, "gzip -cd %s", filename);
+	fclose(h->stream);
+	h->stream = popen(p, "r");
+	free(p);
+
+	  if (h->stream == NULL)
+	    return -1;
+
+	h->stream_ispipe = 1;
+	if (fread (hdr, sizeof (hdr), 1, h->stream) != 1)
+	    return -1;
+	}
+  else if (!memcmp (hdr, "BZ", 2))
+	{
+	char *p = malloc(strlen("bzip2 -cd ") + strlen(filename) + 1);
+	sprintf(p, "bzip2 -cd %s", filename);
+	fclose(h->stream);
+	h->stream = popen(p, "r");
+	free(p);
+
+	  if (h->stream == NULL)
+	    return -1;
+
+	h->stream_ispipe = 1;
+	if (fread (hdr, sizeof (hdr), 1, h->stream) != 1)
+	    return -1;
+	}
+  else
+	{
+	h->stream_ispipe = 0;
+	}
+
   if (memcmp (hdr, "GHDLwave\n", 9) != 0)
     return -2;
   /* Check version.  */
@@ -124,7 +167,7 @@ ghw_read_byte (struct ghw_handler *h, unsigned char *res)
 int
 ghw_read_uleb128 (struct ghw_handler *h, uint32_t *res)
 {
-  unsigned int r = 0;
+  uint32_t r = 0;
   unsigned int off = 0;
 
   while (1)
@@ -202,7 +245,7 @@ ghw_read_f64 (struct ghw_handler *h, double *res)
 const char *
 ghw_read_strid (struct ghw_handler *h)
 {
-  unsigned int id;
+  uint32_t id;
   if (ghw_read_uleb128 (h, &id) != 0)
     return NULL;
   return h->str_table[id];
@@ -211,7 +254,7 @@ ghw_read_strid (struct ghw_handler *h)
 union ghw_type *
 ghw_read_typeid (struct ghw_handler *h)
 {
-  unsigned int id;
+  uint32_t id;
   if (ghw_read_uleb128 (h, &id) != 0)
     return NULL;
   return h->types[id - 1];
@@ -306,9 +349,9 @@ ghw_read_str (struct ghw_handler *h)
 
   if (hdr[0] != 0 || hdr[1] != 0 || hdr[2] != 0 || hdr[3] != 0)
     return -1;
-  h->nbr_str = ghw_get_i32 (h, &hdr[4]);
+  h->nbr_str = ghw_get_i32 (h, (unsigned char *)&hdr[4]);
   h->nbr_str++;
-  h->str_size = ghw_get_i32 (h, &hdr[8]);
+  h->str_size = ghw_get_i32 (h, (unsigned char *)&hdr[8]);
   h->str_table = (char **)malloc ((h->nbr_str + 1) * sizeof (char *));
   h->str_content = (char *)malloc (h->str_size + h->nbr_str + 1);
 
@@ -426,6 +469,13 @@ get_range_length (union ghw_range *rng)
 	return (rng->i32.left - rng->i32.right + 1);
       else
 	return (rng->i32.right - rng->i32.left + 1);
+
+    case ghdl_rtik_type_e8:
+      if (rng->e8.dir)
+	return (rng->e8.left - rng->e8.right + 1);
+      else
+	return (rng->e8.right - rng->e8.left + 1);
+
     default:
       fprintf (stderr, "get_range_length: unhandled kind %d\n", rng->kind);
       abort ();
@@ -443,7 +493,7 @@ ghw_read_type (struct ghw_handler *h)
 
   if (hdr[0] != 0 || hdr[1] != 0 || hdr[2] != 0 || hdr[3] != 0)
     return -1;
-  h->nbr_types = ghw_get_i32 (h, &hdr[4]);
+  h->nbr_types = ghw_get_i32 (h, (unsigned char *)&hdr[4]);
   h->types = (union ghw_type **)
     malloc (h->nbr_types * sizeof (union ghw_type *));
 
@@ -467,7 +517,7 @@ ghw_read_type (struct ghw_handler *h)
 	    e->kind = t;
 	    e->wkt = ghw_wkt_unknown;
 	    e->name = ghw_read_strid (h);
-	    if (ghw_read_uleb128 (h, &e->nbr) != 0)
+	    if (ghw_read_uleb128 (h, (uint32_t *)&e->nbr) != 0)
 	      return -1;
 	    e->lits = (const char **) malloc (e->nbr * sizeof (char *));
 	    if (h->flag_verbose > 1)
@@ -549,7 +599,7 @@ ghw_read_type (struct ghw_handler *h)
 	    arr->kind = t;
 	    arr->name = ghw_read_strid (h);
 	    arr->el = ghw_read_typeid (h);
-	    if (ghw_read_uleb128 (h, &arr->nbr_dim) != 0)
+	    if (ghw_read_uleb128 (h, (uint32_t *)&arr->nbr_dim) != 0)
 	      return -1;
 	    arr->dims = (union ghw_type **)
 	      malloc (arr->nbr_dim * sizeof (union ghw_type *));
@@ -593,7 +643,7 @@ ghw_read_type (struct ghw_handler *h)
 	    rec = malloc (sizeof (struct ghw_type_record));
 	    rec->kind = t;
 	    rec->name = ghw_read_strid (h);
-	    if (ghw_read_uleb128 (h, &rec->nbr_fields) != 0)
+	    if (ghw_read_uleb128 (h, (uint32_t *)&rec->nbr_fields) != 0)
 	      return -1;
 	    rec->el = malloc
 	      (rec->nbr_fields * sizeof (struct ghw_record_element));
@@ -671,7 +721,7 @@ ghw_read_signal (struct ghw_handler *h, unsigned int *sigs, union ghw_type *t)
     case ghdl_rtik_type_e32:
     case ghdl_rtik_subtype_scalar:
       {
-	unsigned int sig_el;
+	uint32_t sig_el;
 
 	if (ghw_read_uleb128 (h, &sig_el) < 0)
 	  return -1;
@@ -789,11 +839,11 @@ ghw_read_hie (struct ghw_handler *h)
 
   if (hdr[0] != 0 || hdr[1] != 0 || hdr[2] != 0 || hdr[3] != 0)
     return -1;
-  nbr_scopes = ghw_get_i32 (h, &hdr[4]);
+  nbr_scopes = ghw_get_i32 (h, (unsigned char *)&hdr[4]);
   /* Number of declared signals (which may be composite).  */
-  nbr_sigs = ghw_get_i32 (h, &hdr[8]);
+  nbr_sigs = ghw_get_i32 (h, (unsigned char *)&hdr[8]);
   /* Number of basic signals.  */
-  h->nbr_sigs = ghw_get_i32 (h, &hdr[12]);
+  h->nbr_sigs = ghw_get_i32 (h, (unsigned char *)&hdr[12]);
 
   if (h->flag_verbose)
     printf ("%d scopes, %d signals, %d signal elements\n",
@@ -817,7 +867,7 @@ ghw_read_hie (struct ghw_handler *h)
     {
       int t;
       struct ghw_hie *el;
-      unsigned int str;
+      uint32_t str;
 
       t = fgetc (h->stream);
       if (t == EOF)
@@ -1109,7 +1159,7 @@ ghw_read_snapshot (struct ghw_handler *h)
 
   if (hdr[0] != 0 || hdr[1] != 0 || hdr[2] != 0 || hdr[3] != 0)
     return -1;
-  h->snap_time = ghw_get_i64 (h, &hdr[4]);
+  h->snap_time = ghw_get_i64 (h, (unsigned char *)&hdr[4]);
   if (h->flag_verbose > 1)
     printf ("Time is %lld fs\n", h->snap_time);
 
@@ -1143,7 +1193,7 @@ ghw_read_cycle_start (struct ghw_handler *h)
   if (fread (hdr, sizeof (hdr), 1, h->stream) != 1)
     return -1;
 
-  h->snap_time = ghw_get_i64 (h, hdr);
+  h->snap_time = ghw_get_i64 (h, (unsigned char *)hdr);
   return 0;
 }
 
@@ -1214,16 +1264,31 @@ ghw_read_cycle_end (struct ghw_handler *h)
   return 0;
 }
 
+static const char *
+ghw_get_lit (union ghw_type *type, int e)
+{
+  if (e >= type->en.nbr || e < 0)
+    return "??";
+  else
+    return type->en.lits[e];
+}
+
+static void
+ghw_disp_lit (union ghw_type *type, int e)
+{
+  printf ("%s (%d)", ghw_get_lit (type, e), e);
+}
+
 void
 ghw_disp_value (union ghw_val *val, union ghw_type *type)
 {
   switch (ghw_get_base_type (type)->kind)
     {
     case ghdl_rtik_type_b2:
-      printf ("%s (%d)", type->en.lits[val->b2], val->b2);
+      ghw_disp_lit (type, val->b2);
       break;
     case ghdl_rtik_type_e8:
-      printf ("%s (%d)", type->en.lits[val->e8], val->e8);
+      ghw_disp_lit (type, val->e8);
       break;
     case ghdl_rtik_type_i32:
       printf ("%d", val->i32);
@@ -1261,10 +1326,18 @@ ghw_get_value (char *buf, int len, union ghw_val *val, union ghw_type *type)
 	}
       break;
     case ghdl_rtik_type_e8:
-      if (val->b2 <= type->en.nbr)
+      if (val->e8 <= type->en.nbr)
 	{
-	  strncpy (buf, type->en.lits[val->e8], len - 1);
-	  buf[len - 1] = 0;
+	  /* XXX : without the if() is this a programming error? */
+	  if(type->en.lits)
+		{
+	  	strncpy (buf, type->en.lits[val->e8], len - 1);
+	  	buf[len - 1] = 0;
+		}
+		else
+		{
+		snprintf (buf, len, "?%d", val->e8);
+		}
 	}
       else
 	{
@@ -1570,7 +1643,7 @@ ghw_close (struct ghw_handler *h)
 {
   if (h->stream)
     {
-      fclose (h->stream);
+      if(h->stream_ispipe) { pclose (h->stream); } else { fclose (h->stream); }
       h->stream = NULL;
     }
 }
@@ -1582,10 +1655,14 @@ ghw_get_dir (int is_downto)
 }
 
 void
-ghw_disp_range (union ghw_range *rng)
+ghw_disp_range (union ghw_type *type, union ghw_range *rng)
 {
   switch (rng->kind)
     {
+    case ghdl_rtik_type_e8:
+      printf ("%s %s %s", ghw_get_lit (type, rng->e8.left),
+	      ghw_get_dir (rng->e8.dir), ghw_get_lit (type, rng->e8.right));
+      break;
     case ghdl_rtik_type_i32:
     case ghdl_rtik_type_p32:
       printf ("%d %s %d",
@@ -1657,7 +1734,7 @@ ghw_disp_type (struct ghw_handler *h, union ghw_type *t)
 	printf ("subtype %s is ", s->name);
 	ghw_disp_typename (h, s->base);
 	printf (" range ");
-	ghw_disp_range (s->rng);
+	ghw_disp_range (s->base, s->rng);
 	printf (";\n");
       }
       break;
@@ -1692,7 +1769,7 @@ ghw_disp_type (struct ghw_handler *h, union ghw_type *t)
 	  {
 	    if (i != 0)
 	      printf (", ");
-	    ghw_disp_range (a->rngs[i]);
+	    ghw_disp_range ((union ghw_type *)a->base, a->rngs[i]);
 	  }
 	printf (");\n");
       }
@@ -1725,3 +1802,12 @@ ghw_disp_types (struct ghw_handler *h)
   for (i = 0; i < h->nbr_types; i++)
     ghw_disp_type (h, h->types[i]);
 }
+
+/*
+ * $Id: ghwlib.c,v 1.2 2007/04/20 02:08:12 gtkwave Exp $
+ * $Log: ghwlib.c,v $
+ * Revision 1.2  2007/04/20 02:08:12  gtkwave
+ * initial release
+ *
+ */
+
