@@ -6,10 +6,7 @@
 #define WAVE_GTKIFE(a,b,c,d,e) {a,b,c,d,e,NULL}
 #define WAVE_GDK_GET_POINTER(a,b,c,bi,ci,d)  gdk_window_get_pointer(a,bi,ci,d)
 #define WAVE_GDK_GET_POINTER_COPY x=xi; y=yi;
-#define WAVE_GTK_SIGFONT gdk_font_load("-*-courier-*-r-*-*-14-*-*-*-*-*-*-*"); \
-	if(!signalfont) { fprintf(stderr, "Could not load signalfont courier 14, exiting!\n"); exit(255); }
-#define WAVE_GTK_WAVEFONT gdk_font_load("-*-courier-*-r-*-*-10-*-*-*-*-*-*-*"); \
-	if(!wavefont) { fprintf(stderr, "Could not load wavefont courier 10, exiting!\n"); exit(255); }
+
 #define WAVE_GTK_SFUNCAST(x) ((void (*)(GtkWidget *, gpointer))(x))
 
 #else
@@ -31,3 +28,11 @@
 
 #endif
 
+/*
+ * $Id: gtk12compat.h,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: gtk12compat.h,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
