@@ -20,6 +20,7 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <config.h>
 #include "lxt2_read.h"
 
 /****************************************************************************/
@@ -807,9 +808,9 @@ if(!(lt->handle=fopen(name, "rb")))
 
 	setvbuf(lt->handle, (char *)NULL, _IONBF, 0);	/* keeps gzip from acting weird in tandem with fopen */
 
-	fread(&id, 2, 1, lt->handle);
-	fread(&version, 2, 1, lt->handle);
-	fread(&lt->granule_size, 1, 1, lt->handle);
+	if(!fread(&id, 2, 1, lt->handle)) { id = 0; }
+	if(!fread(&version, 2, 1, lt->handle)) { id = 0; }
+	if(!fread(&lt->granule_size, 1, 1, lt->handle)) { id = 0; }
 	
 	if(lxt2_rd_get_16(&id,0) != LXT2_RD_HDRID)
 		{
@@ -833,19 +834,20 @@ if(!(lt->handle=fopen(name, "rb")))
 		}
 	else
 		{
+		size_t rcf;
 		int rc;
 		char *m;
 		off_t pos, fend;
 		int t;
 		struct lxt2_rd_block *b;
 
-		fread(&lt->numfacs, 4, 1, lt->handle);			lt->numfacs = lxt2_rd_get_32(&lt->numfacs,0);
-		fread(&lt->numfacbytes, 4, 1, lt->handle);		lt->numfacbytes = lxt2_rd_get_32(&lt->numfacbytes,0);
-		fread(&lt->longestname, 4, 1, lt->handle);		lt->longestname = lxt2_rd_get_32(&lt->longestname,0);
-		fread(&lt->zfacnamesize, 4, 1, lt->handle);		lt->zfacnamesize = lxt2_rd_get_32(&lt->zfacnamesize,0);
-		fread(&lt->zfacname_predec_size, 4, 1, lt->handle); 	lt->zfacname_predec_size = lxt2_rd_get_32(&lt->zfacname_predec_size,0);
-		fread(&lt->zfacgeometrysize, 4, 1, lt->handle);		lt->zfacgeometrysize = lxt2_rd_get_32(&lt->zfacgeometrysize,0);
-		fread(&lt->timescale, 1, 1, lt->handle);		/* no swap necessary */
+		rcf=fread(&lt->numfacs, 4, 1, lt->handle);		lt->numfacs = rcf ? lxt2_rd_get_32(&lt->numfacs,0) : 0;
+		rcf=fread(&lt->numfacbytes, 4, 1, lt->handle);		lt->numfacbytes = rcf ? lxt2_rd_get_32(&lt->numfacbytes,0) : 0;
+		rcf=fread(&lt->longestname, 4, 1, lt->handle);		lt->longestname = rcf ? lxt2_rd_get_32(&lt->longestname,0) : 0;
+		rcf=fread(&lt->zfacnamesize, 4, 1, lt->handle);		lt->zfacnamesize = rcf ? lxt2_rd_get_32(&lt->zfacnamesize,0) : 0;
+		rcf=fread(&lt->zfacname_predec_size, 4, 1, lt->handle); lt->zfacname_predec_size = rcf ? lxt2_rd_get_32(&lt->zfacname_predec_size,0) : 0;
+		rcf=fread(&lt->zfacgeometrysize, 4, 1, lt->handle);	lt->zfacgeometrysize = rcf ? lxt2_rd_get_32(&lt->zfacgeometrysize,0) : 0;
+		rcf=fread(&lt->timescale, 1, 1, lt->handle);		if(!rcf) lt->timescale = 0; /* no swap necessary */
 
 		fprintf(stderr, LXT2_RDLOAD"%d facilities\n", lt->numfacs);
 		pos = ftello(lt->handle);
@@ -948,10 +950,10 @@ if(!(lt->handle=fopen(name, "rb")))
 
 			b=calloc(1, sizeof(struct lxt2_rd_block));
 		
-			fread(&b->uncompressed_siz, 4, 1, lt->handle);			b->uncompressed_siz = lxt2_rd_get_32(&b->uncompressed_siz,0);
-			fread(&b->compressed_siz, 4, 1, lt->handle);			b->compressed_siz = lxt2_rd_get_32(&b->compressed_siz,0);
-			fread(&b->start, 8, 1, lt->handle);				b->start = lxt2_rd_get_64(&b->start,0);
-			fread(&b->end, 8, 1, lt->handle);				b->end = lxt2_rd_get_64(&b->end,0);
+			rcf = fread(&b->uncompressed_siz, 4, 1, lt->handle);	b->uncompressed_siz = rcf ? lxt2_rd_get_32(&b->uncompressed_siz,0) : 0;
+			rcf = fread(&b->compressed_siz, 4, 1, lt->handle);	b->compressed_siz = rcf ? lxt2_rd_get_32(&b->compressed_siz,0) : 0;
+			rcf = fread(&b->start, 8, 1, lt->handle);		b->start = rcf ? lxt2_rd_get_64(&b->start,0) : 0;
+			rcf = fread(&b->end, 8, 1, lt->handle);			b->end = rcf ? lxt2_rd_get_64(&b->end,0) : 0;
 			pos = ftello(lt->handle);
 			fseeko(lt->handle, pos, SEEK_SET);
 			/* fprintf(stderr, LXT2_RDLOAD"block gzip start at pos %d (0x%08x)\n", pos, pos); */
@@ -1457,6 +1459,7 @@ struct lxt2_rd_block *bcutoff=NULL, *bfinal=NULL;
 int striped_kill = 0;
 unsigned int real_uncompressed_siz = 0;
 unsigned char gzid[2];
+int i;
 
 if(lt)
 	{
@@ -1466,6 +1469,11 @@ if(lt)
 	b = lt->block_head;
 	blk=0;
 
+	for(i=0;i<lt->numfacs;i++)
+		{
+		if(lt->value[i]) lt->value[i][0] = 0;
+		}
+
 	while(b)
 		{
 		if((!b->mem)&&(!b->short_read_ignore)&&(!b->exclude_block))
@@ -1483,7 +1491,7 @@ if(lt)
 
 			fseeko(lt->handle, b->filepos, SEEK_SET);
 			gzid[0]=gzid[1]=0;
-			fread(&gzid, 2, 1, lt->handle);
+			if(!fread(&gzid, 2, 1, lt->handle)) { gzid[0] = gzid[1] = 0; }
 			fseeko(lt->handle, b->filepos, SEEK_SET);
 
 			if((striped_kill = (gzid[0]!=0x1f)||(gzid[1]!=0x8b)))
@@ -1505,10 +1513,12 @@ if(lt)
 
 				while(iter!=0xFFFFFFFF)
 					{
+					size_t rcf;
+
 					clen = unclen = iter = 0;				
-					fread(&clen, 4, 1, lt->handle);		clen = lxt2_rd_get_32(&clen,0);
-					fread(&unclen, 4, 1, lt->handle);	unclen = lxt2_rd_get_32(&unclen,0);
-					fread(&iter, 4, 1, lt->handle);		iter = lxt2_rd_get_32(&iter,0);
+					rcf = fread(&clen, 4, 1, lt->handle);	clen = rcf ? lxt2_rd_get_32(&clen,0) : 0;
+					rcf = fread(&unclen, 4, 1, lt->handle);	unclen = rcf ? lxt2_rd_get_32(&unclen,0) : 0;
+					rcf = fread(&iter, 4, 1, lt->handle);	iter = rcf ? lxt2_rd_get_32(&iter,0) : 0;
 
 					fspos += 12;
 					if((iter==0xFFFFFFFF)||(lt->process_mask_compressed[iter/LXT2_RD_PARTIAL_SIZE]))
@@ -1519,14 +1529,14 @@ if(lt)
 							zbuff = malloc(zlen = clen * 2);
 							}
 
-						fread(zbuff, clen, 1, lt->handle);
+						if(!fread(zbuff, clen, 1, lt->handle)) { clen = 0; }
 
 						strm.avail_in = clen-10;
 						strm.avail_out = unclen;
 						strm.total_in = strm.total_out = 0;
 						strm.zalloc = NULL; strm.zfree = NULL; strm.opaque = NULL;
-						strm.next_in = zbuff+10;
-						strm.next_out = pnt;
+						strm.next_in = (unsigned char *)(zbuff+10);
+						strm.next_out = (unsigned char *)(pnt);
 
 						rc = inflateInit2(&strm, -MAX_WBITS);
 						while (Z_OK == (rc = inflate(&strm, Z_NO_FLUSH)));
@@ -1716,4 +1726,11 @@ if(lt)
 return(blk);
 }
 
+/*
+ * $Id: lxt2_read.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: lxt2_read.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
 
