@@ -31,14 +31,16 @@
 
 #ifndef _MSC_VER
 #include <unistd.h>
+#if HAVE_INTTYPES_H
 #include <inttypes.h>
+#endif
 #else
 typedef long off_t;
 #include <windows.h>
 #include <io.h>
 #endif
 
-#if defined _MSC_VER || defined __MINGW32__
+#ifndef HAVE_FSEEKO
 #define fseeko fseek
 #define ftello ftell
 #endif
@@ -279,3 +281,12 @@ unsigned int			lxt2_rd_limit_time_range(struct lxt2_rd_trace *lt, lxtint64_t str
 unsigned int			lxt2_rd_unlimit_time_range(struct lxt2_rd_trace *lt);
 
 #endif
+
+/*
+ * $Id: lxt2_read.h,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: lxt2_read.h,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
