@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 Tony Bybell.
+ * Copyright (c) 2003-2007 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,6 +20,11 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#ifdef _AIX
+#pragma alloca
+#endif
+
+#include <config.h>
 #include "lxt2_write.h"
 
 
@@ -555,6 +560,11 @@ if(s->namlen<3) return;
 lastch--;
 while(lastch!=s->name)
 	{
+        if(*lastch=='.')
+                {
+                return; /* MTI SV [0.3] notation for implicit vars */
+                }
+
 	if(*lastch=='[')
 		{
 		*lastch=0x00;	
@@ -1039,8 +1049,10 @@ for(cnt = 0; cnt < lt->break_header_size; cnt += sizeof(buf))
 		seg = sizeof(buf);
 		}
 
-	fread(buf, seg, 1, clone);
-	fwrite(buf, seg, 1, f2);
+	if(fread(buf, seg, 1, clone))
+		{
+		if(!fwrite(buf, seg, 1, f2)) break; /* write error! */
+		}
 	}
 
 fclose(clone);
@@ -1414,7 +1426,7 @@ if((lt->timegranule>=lt->maxgranule)||(do_finalize)||(early_flush))
 	lxt2_wr_emit_u64(lt, (lt->firsttime>>32)&0xffffffff, lt->firsttime&0xffffffff);
 	lxt2_wr_emit_u64(lt, (lt->lasttime>>32)&0xffffffff, lt->lasttime&0xffffffff);
 
-	/* fprintf(stderr, "start: %Ld, end %Ld\n", lt->firsttime, lt->lasttime); */
+	/* fprintf(stderr, "start: %lld, end %lld\n", lt->firsttime, lt->lasttime); */
 
 	lt->timegranule=0;
 	lt->numblock++;
@@ -1787,13 +1799,13 @@ while(s->aliased_to)	/* find root alias if exists */
 valuelen = strlen(value);	/* ensure string is proper length */
 if(valuelen == s->len)
 	{
-	vfix = alloca(s->len+1);
+	vfix = wave_alloca(s->len+1);
 	strcpy(vfix, value);
 	value = vfix;
 	}
 	else
 	{
-	vfix = alloca(s->len+1);
+	vfix = wave_alloca(s->len+1);
 
 	if(valuelen < s->len)
 		{
@@ -2167,3 +2179,12 @@ if(lt)
 	sprintf(lt->zmode, "wb%d", depth);
 	}
 }
+
+/*
+ * $Id: lxt2_write.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: lxt2_write.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
