@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 Tony Bybell.
+ * Copyright (c) 2003-7 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,8 +20,14 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <config.h>
 #include "lxt2_read.h"
+
+#if HAVE_GETOPT_H
 #include <getopt.h>
+#endif
+
+#include <time.h>
 
 int flat_earth = 0;
 extern void free_hier(void);
@@ -80,12 +86,12 @@ void vcd_callback(struct lxt2_rd_trace **lt, lxtint64_t *pnt_time, lxtint32_t *p
 {
 struct lxt2_rd_geometry *g = lxt2_rd_get_fac_geometry(*lt, *pnt_facidx);
 
-/* fprintf(stderr, "%Ld %d %s\n", *pnt_time, *pnt_facidx, *pnt_value); */
+/* fprintf(stderr, "%lld %d %s\n", *pnt_time, *pnt_facidx, *pnt_value); */
 
 if(vcd_prevtime != *pnt_time)
 	{
 	vcd_prevtime = *pnt_time;
-	printf("#%Ld\n", *pnt_time);
+	printf("#%lld\n", *pnt_time);
 	}
 
 if(!(*pnt_value)[0])
@@ -142,7 +148,8 @@ if(lt)
 	int numfacs;
 	char time_dimension;
 	int time_scale = 1;
-	char scale;
+	signed char scale;
+	time_t walltime;
 	
 	numfacs = lxt2_rd_get_num_facs(lt);
 	lxt2_rd_set_fac_process_mask_all(lt);
@@ -175,6 +182,11 @@ if(lt)
                 default:                                        time_dimension = 'n'; break;
                 }
 
+        time(&walltime);
+        printf("$date\n");
+        printf("\t%s",asctime(localtime(&walltime)));
+        printf("$end\n");
+        printf("$version\n\tlxt2vcd\n$end\n");
 	printf("$timescale %d%c%c $end\n", time_scale, time_dimension, !scale ? ' ' : 's');
 
 	for(i=0;i<numfacs;i++)
@@ -205,7 +217,7 @@ if(lt)
                         {
                        	if(g->len==1)
                        		{
-                                if((g->msb!=-1)&&(g->msb!=0))
+                                if(g->msb!=-1)
                                 	{
                                         printf("$var wire 1 %s %s[%d] $end\n", vcdid(newindx), netname, g->msb);
                                         }
@@ -216,7 +228,14 @@ if(lt)
 				}
                                 else
                                 {
-                                printf("$var wire %d %s %s[%d:%d] $end\n", g->len, vcdid(newindx), netname, g->msb, g->lsb);
+                                if(!(g->flags & LXT2_RD_SYM_F_INTEGER))
+                                        {
+                                        printf("$var wire %d %s %s[%d:%d] $end\n", g->len, vcdid(newindx), netname, g->msb, g->lsb);
+                                        }
+                                        else
+                                        {
+                                        printf("$var integer %d %s %s $end\n", g->len, vcdid(newindx), netname);
+                                        }
                                 }
                         }
 		}
@@ -236,7 +255,7 @@ if(lt)
 
 	if(vcd_prevtime!=lxt2_rd_get_end_time(lt))
 		{
-		printf("#%Ld\n", lxt2_rd_get_end_time(lt));
+		printf("#%lld\n", lxt2_rd_get_end_time(lt));
 		}
 
 	lxt2_rd_close(lt);
@@ -284,6 +303,8 @@ char *lxname=NULL;
 int c;
 int rc;
 
+WAVE_LOCALE_FIX
+
 while (1)
         {
         int option_index = 0;
@@ -356,3 +377,12 @@ free(lxname);
 
 return(rc);
 }
+
+/*
+ * $Id: lxt2vcd.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: lxt2vcd.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
