@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-5 Tony Bybell.
+ * Copyright (c) 2001-7 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,6 +20,7 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <config.h>
 #include "lxt_write.h"
 
 /*
@@ -618,6 +619,11 @@ if(s->namlen<3) return;
 lastch--;
 while(lastch!=s->name)
 	{
+	if(*lastch=='.')
+		{
+		return;	/* MTI SV [0.3] notation for implicit vars */
+		}
+
 	if(*lastch=='[')
 		{
 		*lastch=0x00;	
@@ -860,7 +866,7 @@ switch(numbytes_trans&3)
 	case 3: lt->lt_emit_u32(lt, numtrans); break;
 	}
 
-/* printf("Clock finish for '%s' at %Ld ending with '%c' for %d repeats over a switch delta of %d\n", 
+/* printf("Clock finish for '%s' at %lld ending with '%c' for %d repeats over a switch delta of %d\n", 
 	s->name, lt->timeval, s->clk_prevval, s->clk_numtrans - LT_CLKPACK, s->clk_delta); */
 s->clk_prevtrans = ULLDescriptor(~0);
 s->clk_numtrans = 0;
@@ -961,7 +967,7 @@ switch(numbytes_trans&3)
 	case 3: lt->lt_emit_u32(lt, numtrans); break;
 	}
 
-/* printf("Clock finish for '%s' at %Ld ending with '%08x' for %d repeats over a switch delta of %Ld\n", 
+/* printf("Clock finish for '%s' at %lld ending with '%08x' for %d repeats over a switch delta of %lld\n", 
 	s->name, lt->timeval, s->clk_prevval, s->clk_numtrans - LT_CLKPACK_M, s->clk_delta); */
 s->clk_prevtrans = ULLDescriptor(~0);
 s->clk_numtrans = 0;
@@ -1668,6 +1674,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 	unsigned int last_change_delta;
 
 	if((lt->clock_compress)&&(s->rows==0))
+	{
 	if((len>1)&&(len<=32))
 		{
 		int ivalue = value;
@@ -1707,7 +1714,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 						s->clk_prevval1 = s->clk_prevval;
 						s->clk_prevval  = ivalue;
 
-						/* printf("Clock value '%08x' for '%s' at %Ld (#%d)\n", ivalue, s->name, lt->timeval, s->clk_numtrans); */
+						/* printf("Clock value '%08x' for '%s' at %lld (#%d)\n", ivalue, s->name, lt->timeval, s->clk_numtrans); */
 						return(1);
 						}
 					}
@@ -1808,6 +1815,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 
 		s->clk_prevval = ivalue + '0';
 		}
+	}
 
 	/* normal trace handling */
 
@@ -2274,6 +2282,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 	int len = ((s->flags)&LT_SYM_F_INTEGER) ? 32 : s->len;
 
 	if((lt->clock_compress)&&(s->rows==0))
+	{
 	if((len>1)&&(len<=32))
 		{
 		int legal = 0;
@@ -2337,7 +2346,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 						s->clk_prevval1 = s->clk_prevval;
 						s->clk_prevval  = ivalue;
 
-						/* printf("Clock value '%08x' for '%s' [len=%d] at %Ld (#%d)\n", 
+						/* printf("Clock value '%08x' for '%s' [len=%d] at %lld (#%d)\n", 
 							ivalue, s->name, len, lt->timeval, s->clk_numtrans); */
 						return(1);
 						}
@@ -2403,7 +2412,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 						{
 						s->clk_prevval = value[0];
 
-						/* printf("Clock value '%c' for '%s' at %Ld (#%d)\n", value[0], s->name, lt->timeval, s->clk_numtrans); */
+						/* printf("Clock value '%c' for '%s' at %lld (#%d)\n", value[0], s->name, lt->timeval, s->clk_numtrans); */
 						return(1);
 						}
 					}
@@ -2437,6 +2446,7 @@ if(!(s->flags&(LT_SYM_F_DOUBLE|LT_SYM_F_STRING)))
 
 		s->clk_prevval = value[0];
 		}
+	}
 
 	/* normal trace handling */
 
@@ -2802,3 +2812,12 @@ if((lt)&&(lt->dumpoff_active))
 	lt->dumpoff_active = 0;
 	}
 }
+
+/*
+ * $Id: lxt_write.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: lxt_write.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
