@@ -20,6 +20,7 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -165,9 +166,11 @@ return(rv);
 
 static int bsearch_timechain(int key)
 {
+void *bsearch_dummy_rv;
+
 max_compare_time_tc=-1; max_compare_pos_tc=-1; 
 
-bsearch((void *)(long)key, (void *)(long)tableposition, total_cycles, 8, compar_timechain);
+bsearch_dummy_rv = bsearch((void *)(long)key, (void *)(long)tableposition, total_cycles, 8, compar_timechain);
 if((!max_compare_pos_tc)||(max_compare_time_tc<0)) 
         {
         max_compare_pos_tc=tableposition; /* aix bsearch fix */
@@ -309,8 +312,8 @@ for(;;)
 		{
 		opcode=get_byte(pos);
 
-		fwrite((void *)lcindex, sizeof(unsigned int), 1, thandle);
-		fwrite((void *)&pos, sizeof(unsigned int), 1, thandle);
+		if(!fwrite((void *)lcindex, sizeof(unsigned int), 1, thandle)) { fprintf(stderr, "write error, exiting!\n"); exit(255); }
+		if(!fwrite((void *)&pos, sizeof(unsigned int), 1, thandle)) { fprintf(stderr, "write error, exiting!\n"); exit(255); }
 		fsiz+=(2*sizeof(unsigned int));
 
 		/* fprintf(stderr, MVLHDR"Pos: %08x, time=%d, sym=%s, opcode=%02x\n",  pos, tmval, facs[lcindex[0]].name,opcode); */
@@ -799,8 +802,8 @@ lt_set_initial_value(lt, '0');
 fseek(thandle, -2*(sizeof(unsigned int)), SEEK_CUR);
 for(;;)
 	{
-	fread((void *)&which, sizeof(unsigned int), 1, thandle);
-	fread((void *)&pos, sizeof(unsigned int), 1, thandle);
+	if(!fread((void *)&which, sizeof(unsigned int), 1, thandle)) which = 0;
+	if(!fread((void *)&pos, sizeof(unsigned int), 1, thandle)) pos = 0;
 	tmvalchk=bsearch_timechain(pos);
 	if(tmval!=tmvalchk)
 		{
@@ -827,6 +830,8 @@ int main(int argc, char **argv)
 int fd;
 int i, doclock=0, dochg=0;
 
+WAVE_LOCALE_FIX
+
 if(argc<3)
         {
         fprintf(stderr, "Usage:\n------\n%s filename.aet filename.lxt [-clockpack][-chgpack]\n", argv[0]);
@@ -887,3 +892,11 @@ fprintf(stderr, MVLHDR"Finished, exiting.\n");
 exit(0);
 }
 
+/*
+ * $Id: mvl2lxt.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: mvl2lxt.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
