@@ -20,6 +20,7 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -322,9 +323,11 @@ return(rv);
 
 static int bsearch_timechain(int key)
 {
+void *bsearch_dummy_rv;
+
 max_compare_time_tc=-1; max_compare_pos_tc=-1; 
 
-bsearch((void *)(long)key, (void *)(long)tableposition, total_cycles, 8, compar_timechain);
+bsearch_dummy_rv = bsearch((void *)(long)key, (void *)(long)tableposition, total_cycles, 8, compar_timechain);
 if((!max_compare_pos_tc)||(max_compare_time_tc<0)) 
         {
         max_compare_pos_tc=tableposition; /* aix bsearch fix */
@@ -466,8 +469,8 @@ for(;;)
 		{
 		opcode=get_byte(pos);
 
-		fwrite((void *)lcindex, sizeof(unsigned int), 1, thandle);
-		fwrite((void *)&pos, sizeof(unsigned int), 1, thandle);
+		if(!fwrite((void *)lcindex, sizeof(unsigned int), 1, thandle)) { fprintf(stderr, "write error, exiting!\n"); exit(255); }
+		if(!fwrite((void *)&pos, sizeof(unsigned int), 1, thandle)) { fprintf(stderr, "write error, exiting!\n"); exit(255); }
 		fsiz+=(2*sizeof(unsigned int));
 
 		/* fprintf(stderr, MVLHDR"Pos: %08x, time=%d, sym=%s, opcode=%02x\n",  pos, tmval, facs[lcindex[0]].name,opcode); */
@@ -1000,8 +1003,8 @@ for(i=0;i<numfacs;i++)		/* init all facs to zero.. */
 fseek(thandle, -2*(sizeof(unsigned int)), SEEK_CUR);
 for(;;)
 	{
-	fread((void *)&which, sizeof(unsigned int), 1, thandle);
-	fread((void *)&pos, sizeof(unsigned int), 1, thandle);
+	if(!fread((void *)&which, sizeof(unsigned int), 1, thandle)) which = 0;
+	if(!fread((void *)&pos, sizeof(unsigned int), 1, thandle)) pos = 0;
 	tmvalchk=bsearch_timechain(pos);
 	if(tmval!=tmvalchk)
 		{
@@ -1028,6 +1031,8 @@ int main(int argc, char **argv)
 {
 int fd;
 
+WAVE_LOCALE_FIX
+
 if(argc!=2)
         {
         fprintf(stderr, "Usage:\n------\n%s filename\n\nVCD is then output to stdout.\n", argv[0]);
@@ -1406,3 +1411,12 @@ while(offs)
 }
 }
 #endif
+
+/*
+ * $Id: mvl2vcd.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: mvl2vcd.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
