@@ -33,6 +33,7 @@
  * 161001ajb: added support for awan format 0xa0 aets
  *
  */
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -249,6 +250,11 @@ if(t->msb==newlsb)
 
 /****************************************************************************/
 
+void exit_read_error(void)
+{
+fprintf(stderr, "read error, exiting!\n"); exit(255);
+}
+
 static int hdr_process(void)
 {
 int ch;
@@ -264,8 +270,10 @@ if((aettype!=0xc0)&&(aettype!=0xa0))
 	}
 
 fseek(handle, 0x14, SEEK_SET);		/* get sim date/time */
-fread(buf, sizeof(char), 8, handle); buf[8]=0;
-fread(buf+9, sizeof(char), 8, handle); buf[17]=0;
+if(!fread(buf, sizeof(char), 8, handle)) { exit_read_error(); }
+buf[8]=0;
+if(!fread(buf+9, sizeof(char), 8, handle)) { exit_read_error(); }
+buf[17]=0;
 printf("$version\n");
 printf("\tAET created at %s on %s.\n", buf+9, buf);
 
@@ -279,8 +287,10 @@ multiplier=get_long();
 if(aettype==0xc0)
 	{
 	fseek(handle, 0x58, SEEK_SET);		/* get model date/time */
-	fread(buf, sizeof(char), 8, handle); buf[8]=0;
-	fread(buf+9, sizeof(char), 8, handle); buf[17]=0;
+	if(!fread(buf, sizeof(char), 8, handle)) { exit_read_error(); }
+	buf[8]=0;
+	if(!fread(buf+9, sizeof(char), 8, handle)) { exit_read_error(); }
+	buf[17]=0;
 	pos=18;
 	while((ch=get_byte())) buf[pos++]=ch;	
 	buf[pos]=0;
@@ -319,7 +329,7 @@ for(i=0;i<numfacs;i++)                  /* pull in facinfo entries */
        	txfacs[i].namindx=get_long();           /* +12 = NAME POS IN NAME TABLE */
        	}   
 
-fread(facnames, sizeof(char), facbytes, handle);
+if(!fread(facnames, sizeof(char), facbytes, handle)) { exit_read_error(); }
 for(i=0;i<numfacs;i++) output_varname(txfacs+i);/* dump the names with hierarchy */
 printf("$enddefinitions $end\n\n");
 return(0);
@@ -335,6 +345,8 @@ int tm=0;
 int offset=0;
 int i, n;
 
+WAVE_LOCALE_FIX
+
 if(argc!=2)
 	{
 	fprintf(stderr, "Usage:\n------\n%s filename\n\nVCD is then output to stdout.\n", argv[0]);
@@ -457,3 +469,12 @@ fini:
 fclose(handle);
 exit(0);
 }
+
+/*
+ * $Id: tex2vcd.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: tex2vcd.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
