@@ -39,7 +39,7 @@
 #endif
 
 
-#define SYMPRIME 65519
+#define SYMPRIME 500009
 #define WAVE_DECOMPRESSOR "gzip -cd "	/* zcat alone doesn't cut it for AIX */
 
 
@@ -113,13 +113,13 @@ struct vcdsymbol *next;
 struct lt_symbol *ltsym;
 char *name;
 char *id;
-unsigned int nid;
-int vartype;
-int msi, lsi;
-int size;
 char *value;
 struct queuedevent *ev; /* only if vartype==V_EVENT */
 struct Node **narray;
+unsigned int nid;
+int msi, lsi;
+int size;
+unsigned char vartype;
 };
 
 
@@ -141,3 +141,12 @@ TimeType vcd_main(char *fname, char *lxname, int dostats, int doclock, int docha
 void append_vcd_slisthier(char *str);
 
 #endif
+
+/*
+ * $Id: v2l_analyzer.h,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: v2l_analyzer.h,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
