@@ -31,7 +31,13 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+
+#ifndef _AIX
+#if HAVE_GETOPT_H
 #include <getopt.h>
+#endif
+#endif
+
 #include "v2l_debug_lxt2.h"
 
 #ifndef _MSC_VER
@@ -39,7 +45,7 @@
 #endif
 
 
-#define SYMPRIME 65519
+#define SYMPRIME 500009
 #define WAVE_DECOMPRESSOR "gzip -cd "	/* zcat alone doesn't cut it for AIX */
 
 
@@ -113,13 +119,13 @@ struct vcdsymbol *next;
 void *ltsym;
 char *name;
 char *id;
-unsigned int nid;
-int vartype;
-int msi, lsi;
-int size;
 char *value;
 struct queuedevent *ev; /* only if vartype==V_EVENT */
 struct Node **narray;
+unsigned int nid;
+int msi, lsi;
+int size;
+unsigned char vartype;
 };
 
 
@@ -141,3 +147,12 @@ TimeType vcd_main(char *fname, char *lxname);
 void append_vcd_slisthier(char *str);
 
 #endif
+
+/*
+ * $Id: v2l_analyzer_lxt2.h,v 1.2 2007/04/20 02:08:19 gtkwave Exp $
+ * $Log: v2l_analyzer_lxt2.h,v $
+ * Revision 1.2  2007/04/20 02:08:19  gtkwave
+ * initial release
+ *
+ */
+
