@@ -24,6 +24,7 @@
  * debug.c 01feb99ajb
  * malloc debugs added on 13jul99ajb
  */
+#include <config.h>
 #include "v2l_debug.h"
 
 
@@ -201,3 +202,11 @@ while((ch=*(str++)))
 return(nflag?(-val):val);
 }
 
+/*
+ * $Id: v2l_debug.c,v 1.2 2007/04/20 02:08:19 gtkwave Exp $
+ * $Log: v2l_debug.c,v $
+ * Revision 1.2  2007/04/20 02:08:19  gtkwave
+ * initial release
+ *
+ */
+
