@@ -93,17 +93,23 @@ TimeType atoi_64(char *str);
  * memory leaks because of it.  you have been warned.
  */
 
-#ifndef	__NetBSD__
-#ifndef __sun__
-#ifndef alloca
-#define alloca __alloca
-#endif
-#else
+#include <stdlib.h>
+#if HAVE_ALLOCA_H
 #include <alloca.h>
+#elif defined(__GNUC__)
+#define alloca __builtin_alloca
+#elif defined(_MSC_VER)
+#include <malloc.h>
+#define alloca _alloca
 #endif
-#endif
-
 #define wave_alloca alloca
-
-
 #endif
+
+/*
+ * $Id: v2l_debug.h,v 1.2 2007/04/20 02:08:19 gtkwave Exp $
+ * $Log: v2l_debug.h,v $
+ * Revision 1.2  2007/04/20 02:08:19  gtkwave
+ * initial release
+ *
+ */
+
