@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Tony Bybell.
+ * Copyright (c) 2001-2007 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,13 @@
  * lxt conversion added		20nov01ajb
  */
 
+
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include "v2l_analyzer.h"
 
 #undef VCD_BSEARCH_IS_PERFECT           /* bsearch is imperfect under linux, but OK under AIX */
@@ -103,18 +110,24 @@ static struct vcdsymbol **sorted=NULL;
 static struct vcdsymbol **indexed=NULL;
 
 enum VarTypes { V_EVENT, V_PARAMETER,
-		V_INTEGER, V_REAL, V_REG, V_SUPPLY0,
-		V_SUPPLY1, V_TIME, V_TRI, V_TRIAND, V_TRIOR,
-		V_TRIREG, V_TRI0, V_TRI1, V_WAND, V_WIRE, V_WOR, V_PORT,
-		V_END, V_LB, V_COLON, V_RB, V_STRING };
+                V_INTEGER, V_REAL, V_REAL_PARAMETER=V_REAL, V_REG, V_SUPPLY0,
+                V_SUPPLY1, V_TIME, V_TRI, V_TRIAND, V_TRIOR,
+                V_TRIREG, V_TRI0, V_TRI1, V_WAND, V_WIRE, V_WOR, V_PORT, V_IN=V_PORT, V_OUT=V_PORT, V_INOUT=V_PORT,
+                V_END, V_LB, V_COLON, V_RB, V_STRING };
 
-char *vartypes[]={ "event", "parameter",
-		"integer", "real", "reg", "supply0",
-		"supply1", "time", "tri", "triand", "trior",
-		"trireg", "tri0", "tri1", "wand", "wire", "wor", "port",
-		"$end", "", "", "", ""};
+static char *vartypes[]={ "event", "parameter",
+                "integer", "real", "real_parameter", "reg", "supply0",
+                "supply1", "time", "tri", "triand", "trior",
+                "trireg", "tri0", "tri1", "wand", "wire", "wor", "port", "in", "out", "inout",
+                "$end", "", "", "", ""};
 
-#define NUM_VTOKENS 19
+static const unsigned char varenums[] = {  V_EVENT, V_PARAMETER,
+                V_INTEGER, V_REAL, V_REAL_PARAMETER, V_REG, V_SUPPLY0,
+                V_SUPPLY1, V_TIME, V_TRI, V_TRIAND, V_TRIOR,
+                V_TRIREG, V_TRI0, V_TRI1, V_WAND, V_WIRE, V_WOR, V_PORT, V_IN, V_OUT, V_INOUT,
+                V_END, V_LB, V_COLON, V_RB, V_STRING };
+
+#define NUM_VTOKENS 23  
 
 static int numsyms=0;
 
@@ -284,7 +297,7 @@ if(numsyms)
         {
         vcd_distance = vcd_maxid - vcd_minid + 1;
                          
-        if(vcd_distance <= 4 * 1024 * 1024)
+        if(vcd_distance <= 8 * 1024 * 1024)
                 {
                 indexed = (struct vcdsymbol **)calloc_2(vcd_distance, sizeof(struct vcdsymbol *));
                         
@@ -321,6 +334,13 @@ if(numsyms)
                         alias_vs_normal_symadd(sorted[i], root_v);
                         }
                 }
+
+        v=vcdsymroot;
+        while(v) 
+                {
+		free(v->name); v->name = NULL;
+                v=v->next;
+                }
         }
 }
 
@@ -424,7 +444,7 @@ return(T_UNKNOWN_KEY);
 }
 
 static int var_prevch=0;
-static int get_vartoken_patched(void)
+static int get_vartoken_patched(int match_kw)
 {
 int ch;
 int i, len=0;
@@ -465,12 +485,13 @@ for(yytext[len++]=ch;;yytext[len++]=ch)
         }
 yytext[len]=0;  /* terminator */
                 
+if(match_kw)
 for(i=0;i<NUM_VTOKENS;i++)
         {
         if(!strcmp(yytext,vartypes[i]))
                 {
                 if(ch<0) { free_2(varsplit); varsplit=NULL; }
-                return(i);
+                return(varenums[i]);
                 }
         }
 
@@ -480,14 +501,14 @@ return(V_STRING);
 }
 
 
-static int get_vartoken(void)
+static int get_vartoken(int match_kw)
 {
 int ch;
 int i, len=0;
 
 if(varsplit)
         {
-        int rc=get_vartoken_patched();
+        int rc=get_vartoken_patched(match_kw);
         if(rc!=V_END) return(rc);  
         var_prevch=0;
         }
@@ -511,6 +532,11 @@ if(!var_prevch)
 if(ch=='[') return(V_LB);
 if(ch==':') return(V_COLON);
 if(ch==']') return(V_RB);  
+
+if(ch=='#')     /* for MTI System Verilog '$var reg 64 >w #implicit-var###VarElem:ram_di[0.0] [63:0] $end' style declarations */
+        {       /* debussy simply escapes until the space */
+        yytext[len++]= '\\';
+        }
         
 for(yytext[len++]=ch;;yytext[len++]=ch)
         {
@@ -549,11 +575,12 @@ if((varsplit)&&(yytext[len-1]==']'))
         varsplit=NULL;
         }
         
+if(match_kw)
 for(i=0;i<NUM_VTOKENS;i++)
         {
         if(!strcmp(yytext,vartypes[i]))
                 {
-                return(i);
+                return(varenums[i]);
                 }
         }
 
@@ -981,11 +1008,8 @@ for(;;)
 			if((vtok==T_END)||(vtok==T_EOF)) break;
 			time_scale=atoi_64(yytext);
 			if(!time_scale) time_scale=1;
-			switch(time_scale)
-				{
-				case 10:	timelogadjust = +1;	break;
-				case 100:	timelogadjust = +2;	break;
-				}
+			else if (time_scale ==  10)	timelogadjust = +1;
+			else if (time_scale == 100)	timelogadjust = +2;
 
 			for(i=0;i<yylen;i++)
 				{
@@ -1080,7 +1104,7 @@ for(;;)
 
 			var_prevch=0;
 			varsplit=NULL;
-			vtok=get_vartoken();
+			vtok=get_vartoken(1);
 			if(vtok>V_PORT) goto bail;
 
 			v=(struct vcdsymbol *)calloc_2(1,sizeof(struct vcdsymbol));
@@ -1089,7 +1113,7 @@ for(;;)
 
 			if(vtok==V_PORT)
 				{
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok==V_STRING)
 					{
 					v->size=atoi_64(yytext);
@@ -1098,11 +1122,11 @@ for(;;)
 					else 
 					if(vtok==V_LB)
 					{
-					vtok=get_vartoken();
+					vtok=get_vartoken(1);
 					if(vtok==V_END) goto err;
 					if(vtok!=V_STRING) goto err;
 					v->msi=atoi_64(yytext);
-					vtok=get_vartoken();
+					vtok=get_vartoken(0);
 					if(vtok==V_RB)
 						{
 						v->lsi=v->msi;
@@ -1111,10 +1135,10 @@ for(;;)
 						else
 						{
 						if(vtok!=V_COLON) goto err;
-						vtok=get_vartoken();
+						vtok=get_vartoken(0);
 						if(vtok!=V_STRING) goto err;
 						v->lsi=atoi_64(yytext);
-						vtok=get_vartoken();
+						vtok=get_vartoken(0);
 						if(vtok!=V_RB) goto err;
 
 						if(v->msi>v->lsi)
@@ -1138,7 +1162,7 @@ for(;;)
 				if(v->nid < vcd_minid) vcd_minid = v->nid;
 				if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				if(slisthier_len)
 					{
@@ -1155,10 +1179,9 @@ for(;;)
 				}
 				else	/* regular vcd var, not an evcd port var */
 				{
-				vtok=get_vartoken();
+				vtok=get_vartoken(1);
 				if(vtok==V_END) goto err;
 				v->size=atoi_64(yytext);
-				if(!v->size) v->size=1;
 				vtok=get_strtoken();
 				if(vtok==V_END) goto err;
 				v->id=(char *)malloc_2(yylen+1);
@@ -1168,7 +1191,7 @@ for(;;)
 				if(v->nid < vcd_minid) vcd_minid = v->nid;
 				if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				if(slisthier_len)
 					{
@@ -1183,27 +1206,29 @@ for(;;)
 					strcpy(v->name,yytext);
 					}
 				
-				vtok=get_vartoken();
+				vtok=get_vartoken(1);
 				if(vtok==V_END) goto dumpv;
 				if(vtok!=V_LB) goto err;
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				v->msi=atoi_64(yytext);
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok==V_RB)
 					{
 					v->lsi=v->msi;
 					goto dumpv;
 					}
 				if(vtok!=V_COLON) goto err;
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				v->lsi=atoi_64(yytext);
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_RB) goto err;
 				}
 
 			dumpv:
+                        if(v->size == 0) { v->vartype = V_REAL; } /* MTI fix */
+
 			if(v->vartype==V_REAL)
 				{
 				v->size=1;		/* override any data we parsed in */
@@ -1855,6 +1880,8 @@ int main(int argc, char **argv)
 {
 int i, dostats=0, doclock=0, dochg=0, dodict=0, linear=0;
 
+WAVE_LOCALE_FIX
+
 if(argc>=3)
         {
 	for(i=3;i<argc;i++)
@@ -1887,3 +1914,12 @@ if(argc>=3)
         }
 exit(0);
 }
+
+/*
+ * $Id: vcd2lxt.c,v 1.2 2007/04/20 02:08:19 gtkwave Exp $
+ * $Log: vcd2lxt.c,v $
+ * Revision 1.2  2007/04/20 02:08:19  gtkwave
+ * initial release
+ *
+ */
+
