@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 Tony Bybell.
+ * Copyright (c) 2003-2007 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,13 +20,19 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#if !defined _MSC_VER && !defined __MINGW32__ && !defined __CYGWIN__
+#include <config.h>
+
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+#undef HAVE_RPC_XDR_H
+#endif
+
+#if HAVE_RPC_XDR_H 
 #include <rpc/types.h>
 #include <rpc/xdr.h>
 #endif
 #include "vzt_read.h"
 
-#ifdef __FreeBSD__
+#ifdef HAVE_FCNTL_H 
 #include <fcntl.h>
 #endif
 
@@ -43,8 +49,14 @@ struct vzt_rd_trace *lt;
 struct vzt_rd_block *b;
 };
 
+struct vzt_synvec_chain
+{
+vztint32_t num_entries;
+vztint32_t chain[1];
+};
 
-#if !defined _MSC_VER && !defined __MINGW32__  && !defined __CYGWIN__ && !defined __FreeBSD__
+
+#ifdef PTHREAD_CREATE_DETACHED
 _VZT_RD_INLINE static int vzt_rd_pthread_mutex_init(struct vzt_rd_trace *lt, pthread_mutex_t *mutex, const pthread_mutexattr_t *mutexattr)
 {
 if(lt->pthreads) { pthread_mutex_init(mutex, mutexattr); }
@@ -181,16 +193,25 @@ return((m1<<24)|(m2<<16)|(m3<<8)|m4);
 }
 
 
-static vztint32_t vzt_rd_get_v32(char **mm)
+static vztint32_t vzt_rd_get_v32(char **mmx)
 {
-signed char *c = *mm;
-signed char *beg = c;
+union {
+  signed char **mm;
+  char **mmu;
+} u;
+
+signed char *c;
+signed char *beg;
 vztint32_t val;
 
+u.mmu = mmx;
+c = *u.mm;
+beg = c;
+
 if(*c>=0)
 	{
 	while(*c>=0) c++;
-	*mm = c+1;
+	*u.mm = c+1;
 
 	val = (vztint32_t)(*c&0x7f);
 	do
@@ -201,23 +222,32 @@ if(*c>=0)
 	}
 	else
 	{
-	*mm = c+1;
+	*u.mm = c+1;
 	val = (vztint32_t)(*c&0x7f);
 	}
 
 return(val);
 }
 
-static vztint64_t vzt_rd_get_v64(char **mm)
+static vztint64_t vzt_rd_get_v64(char **mmx)
 {
-signed char *c = *mm;
-signed char *beg = c;
+union {
+  signed char **mm;
+  char **mmu;
+} u;
+
+signed char *c;
+signed char *beg;
 vztint64_t val;
 
+u.mmu = mmx;
+c = *u.mm;
+beg = c;
+
 if(*c>=0)
 	{
 	while(*c>=0) c++;
-	*mm = c+1;
+	*u.mm = c+1;
 
 	val = (vztint64_t)(*c&0x7f);
 	do
@@ -229,13 +259,15 @@ if(*c>=0)
 	else
 	{
 	val = (vztint64_t)(*c&0x7f);
-	*mm = c+1;
+	*u.mm = c+1;
 	}
 
 return(val);
 }
 
-_VZT_RD_INLINE static vztint32_t vzt_rd_get_v32_rvs(char **mm)
+
+#if 0
+_VZT_RD_INLINE static vztint32_t vzt_rd_get_v32_rvs(signed char **mm)
 {
 signed char *c = *mm;
 vztint32_t val;
@@ -251,7 +283,7 @@ while(*c>=0)
 *mm = c;
 return(val);
 }
-
+#endif
 
 /****************************************************************************/
 
@@ -396,7 +428,7 @@ if(b->rle)
 				if(curr_dec_word == num_sections) goto iloop;
 				}
 			}
-		iloop: 1; /* deliberate...only provides a jump target to loop bottom */
+		iloop: i+=0; /* deliberate...only provides a jump target to loop bottom */
 		}
 
 	goto bpcalc;
@@ -519,22 +551,51 @@ for(;word<row_size;word++)
 		{
 		change_msk = 0;
 
-		if(b->multi_state)
+		if(!(lt->flags[facidx]&VZT_RD_SYM_F_SYNVEC))
 			{
-			for(i=0;i<len;i++)
+			if(b->multi_state)
 				{
-				valpnt = b->change_dict + (b->vindex[vindex_offset+i] * row_size + word);
-				valpnt_x = b->change_dict + (b->vindex[vindex_offset_x+i] * row_size + word);
-				change_msk |= *valpnt;
-				change_msk |= *valpnt_x;
+				for(i=0;i<len;i++)
+					{
+					valpnt = b->change_dict + (b->vindex[vindex_offset+i] * row_size + word);
+					valpnt_x = b->change_dict + (b->vindex[vindex_offset_x+i] * row_size + word);
+					change_msk |= *valpnt;
+					change_msk |= *valpnt_x;
+					}
+				}
+				else
+				{
+				for(i=0;i<len;i++)
+					{
+					valpnt = b->change_dict + (b->vindex[vindex_offset+i] * row_size + word);
+					change_msk |= *valpnt;
+					}
 				}
 			}
 			else
 			{
-			for(i=0;i<len;i++)
+			if(b->multi_state)
 				{
-				valpnt = b->change_dict + (b->vindex[vindex_offset+i] * row_size + word);
-				change_msk |= *valpnt;
+				for(i=0;i<len;i++)
+					{
+					vindex_offset = lt->vindex_offset[facidx+i];
+					vindex_offset_x = vindex_offset + lt->total_values;
+
+					valpnt = b->change_dict + (b->vindex[vindex_offset] * row_size + word);
+					valpnt_x = b->change_dict + (b->vindex[vindex_offset_x] * row_size + word);
+					change_msk |= *valpnt;
+					change_msk |= *valpnt_x;
+					}
+				}
+				else
+				{
+				for(i=0;i<len;i++)
+					{
+					vindex_offset = lt->vindex_offset[facidx+i];
+
+					valpnt = b->change_dict + (b->vindex[vindex_offset] * row_size + word);
+					change_msk |= *valpnt;
+					}
 				}
 			}
 
@@ -555,7 +616,6 @@ return(old_time_offset);
 int vzt_rd_fac_value(struct vzt_rd_trace *lt, struct vzt_rd_block *b, vztint32_t time_offset, vztint32_t facidx, char *value)
 {
 vztint32_t len = lt->len[facidx];
-vztint32_t vindex_offset = lt->vindex_offset[facidx];
 int i;
 int word = time_offset / 32;
 int bit  = time_offset & 31;
@@ -566,27 +626,66 @@ vztint32_t *val_base;
 if((time_offset>b->num_time_ticks)||(facidx>lt->numrealfacs)) return(0);
 
 val_base = b->val_dict + word;
-if(b->multi_state)
+
+if(!(lt->flags[facidx]&VZT_RD_SYM_F_SYNVEC))
 	{
-	vztint32_t vindex_offset_x = lt->vindex_offset[facidx] + lt->total_values;
-	vztint32_t *valpnt_x;
-	int which;
+	vztint32_t vindex_offset = lt->vindex_offset[facidx];
 
-	for(i=0;i<len;i++)
+	if(b->multi_state)
 		{
-		valpnt   = val_base + (b->vindex[vindex_offset++] * row_size);
-		valpnt_x = val_base + (b->vindex[vindex_offset_x++] * row_size);
-
-		which = (((*valpnt_x >> bit) & 1) << 1) | ((*valpnt >> bit) & 1);
-		value[i] = "01xz"[which];
+		vztint32_t vindex_offset_x = vindex_offset + lt->total_values;
+		vztint32_t *valpnt_x;
+		int which;
+	
+		for(i=0;i<len;i++)
+			{
+			valpnt   = val_base + (b->vindex[vindex_offset++] * row_size);
+			valpnt_x = val_base + (b->vindex[vindex_offset_x++] * row_size);
+	
+			which = (((*valpnt_x >> bit) & 1) << 1) | ((*valpnt >> bit) & 1);
+			value[i] = "01xz"[which];
+			}
+		}
+		else
+		{
+		for(i=0;i<len;i++)
+			{
+			valpnt = val_base + (b->vindex[vindex_offset++] * row_size);
+			value[i] = '0' | ((*valpnt >> bit) & 1);
+			}
 		}
 	}
 	else
 	{
-	for(i=0;i<len;i++)
+	vztint32_t vindex_offset;
+
+	if(b->multi_state)
 		{
-		valpnt = val_base + (b->vindex[vindex_offset++] * row_size);
-		value[i] = '0' | ((*valpnt >> bit) & 1);
+		vztint32_t vindex_offset_x;
+		vztint32_t *valpnt_x;
+		int which;
+	
+		for(i=0;i<len;i++)
+			{
+			vindex_offset = lt->vindex_offset[facidx+i];
+			vindex_offset_x = vindex_offset + lt->total_values;
+
+			valpnt   = val_base + (b->vindex[vindex_offset] * row_size);
+			valpnt_x = val_base + (b->vindex[vindex_offset_x] * row_size);
+	
+			which = (((*valpnt_x >> bit) & 1) << 1) | ((*valpnt >> bit) & 1);
+			value[i] = "01xz"[which];
+			}
+		}
+		else
+		{
+		for(i=0;i<len;i++)
+			{
+			vindex_offset = lt->vindex_offset[facidx+i];
+
+			valpnt = val_base + (b->vindex[vindex_offset] * row_size);
+			value[i] = '0' | ((*valpnt >> bit) & 1);
+			}
 		}
 	}
 value[i] = 0;
@@ -597,8 +696,10 @@ return(1);
 static void vzt_rd_double_xdr(char *pnt, char *buf)
 {
 int j;
-#if !defined _MSC_VER && !defined __MINGW32__ && !defined __CYGWIN__
+#if HAVE_RPC_XDR_H 
 XDR x;
+#else
+const vztint32_t endian_matchword = 0x12345678;
 #endif
 double d;
 char xdrdata[8];
@@ -617,14 +718,21 @@ for(j=0;j<64;j++)
 		}
 	}
 
-#if !defined _MSC_VER && !defined __MINGW32__  && !defined __CYGWIN__
+#if HAVE_RPC_XDR_H 
 xdrmem_create(&x, xdrdata, sizeof(xdrdata), XDR_DECODE);						
 xdr_double(&x, &d);
 #else
 /* byte ordering in windows is reverse of XDR (on x86, that is) */
-for(j=0;j<8;j++)
+if(*((char *)&endian_matchword) == 0x78)
 	{
-	((char *)&d)[j] = xdrdata[7-j];
+	for(j=0;j<8;j++)
+		{
+		((char *)&d)[j] = xdrdata[7-j];
+		}
+	}
+	else
+	{
+	memcpy(&d, xdrdata, sizeof(double)); /* big endian, don't bytereverse */
 	}
 #endif
 
@@ -653,7 +761,7 @@ return(spnt);
 int vzt_rd_process_block_linear(struct vzt_rd_trace *lt, struct vzt_rd_block *b)
 {
 int i, i2;
-int idx;
+vztint32_t idx;
 char *pnt=lt->value_current_sector, *pnt2=lt->value_previous_sector;
 char buf[32];
 char *bufpnt;
@@ -720,7 +828,7 @@ return(1);
 int vzt_rd_process_block(struct vzt_rd_trace *lt, struct vzt_rd_block *b)
 {
 int i, i2;
-int idx;
+vztint32_t idx;
 char *pnt=lt->value_current_sector, *pnt2=lt->value_previous_sector;
 char buf[32];
 char *bufpnt;
@@ -1113,7 +1221,14 @@ if(lt)
 		int idx = facidx/8;
 		int bitpos = facidx&7;
 
-		lt->process_mask[idx] |= (1<<bitpos);
+		if(!(lt->flags[facidx]&VZT_RD_SYM_F_ALIAS))
+			{
+			lt->process_mask[idx] |= (1<<bitpos);
+			}
+			else
+			{
+			lt->process_mask[idx] &= (~(1<<bitpos));
+			}
 		}
 	rc=1;
 	}
@@ -1145,11 +1260,17 @@ return(rc);
 _VZT_RD_INLINE int vzt_rd_set_fac_process_mask_all(struct vzt_rd_trace *lt)
 {
 int rc=0;
+int i;
 
 if(lt)
 	{
 	memset(lt->process_mask, 0xff, (lt->numfacs+7)/8);
 	rc=1;
+
+	for(i=0;i<lt->numfacs;i++)
+		{
+		if((!lt->len[i])||(lt->flags[i]&VZT_RD_SYM_F_ALIAS)) vzt_rd_clr_fac_process_mask(lt, i);
+		}
 	}
 
 return(rc);
@@ -1233,7 +1354,7 @@ static int vzt_rd_is_gzip_type(FILE *handle)
 unsigned char cbuf[2] = { 0, 0 };
 
 off_t off = ftello(handle);
-fread(cbuf, 1, 2, handle);
+if(!fread(cbuf, 1, 2, handle)) { cbuf[0] = cbuf[1] = 0; }
 fseeko(handle, off, SEEK_SET);
 
 return((cbuf[0] == 0x1f) && (cbuf[1] == 0x8b));
@@ -1568,9 +1689,9 @@ if(!(lt->handle=fopen(name, "rb")))
 
 	setvbuf(lt->handle, (char *)NULL, _IONBF, 0);	/* keeps gzip from acting weird in tandem with fopen */
 
-	fread(&id, 2, 1, lt->handle);
-	fread(&version, 2, 1, lt->handle);
-	fread(&lt->granule_size, 1, 1, lt->handle);
+	if(!fread(&id, 2, 1, lt->handle)) { id = 0; }
+	if(!fread(&version, 2, 1, lt->handle)) { id = 0; }
+	if(!fread(&lt->granule_size, 1, 1, lt->handle)) { id = 0; }
 	
 	if(vzt_rd_get_16(&id,0) != VZT_RD_HDRID)
 		{
@@ -1594,6 +1715,7 @@ if(!(lt->handle=fopen(name, "rb")))
 		}
 	else
 		{
+		size_t rcf;
 		int rc;
 		char *m;
 		off_t pos, fend;
@@ -1602,13 +1724,13 @@ if(!(lt->handle=fopen(name, "rb")))
 
 		vzt_rd_pthread_mutex_init(lt, &lt->mutex, NULL);
 
-		fread(&lt->numfacs, 4, 1, lt->handle);			lt->numfacs = vzt_rd_get_32(&lt->numfacs,0);
-		fread(&lt->numfacbytes, 4, 1, lt->handle);		lt->numfacbytes = vzt_rd_get_32(&lt->numfacbytes,0);
-		fread(&lt->longestname, 4, 1, lt->handle);		lt->longestname = vzt_rd_get_32(&lt->longestname,0);
-		fread(&lt->zfacnamesize, 4, 1, lt->handle);		lt->zfacnamesize = vzt_rd_get_32(&lt->zfacnamesize,0);
-		fread(&lt->zfacname_predec_size, 4, 1, lt->handle); 	lt->zfacname_predec_size = vzt_rd_get_32(&lt->zfacname_predec_size,0);
-		fread(&lt->zfacgeometrysize, 4, 1, lt->handle);		lt->zfacgeometrysize = vzt_rd_get_32(&lt->zfacgeometrysize,0);
-		fread(&lt->timescale, 1, 1, lt->handle);		/* no swap necessary */
+		rcf = fread(&lt->numfacs, 4, 1, lt->handle);		lt->numfacs = rcf ? vzt_rd_get_32(&lt->numfacs,0) : 0;
+		rcf = fread(&lt->numfacbytes, 4, 1, lt->handle);	lt->numfacbytes = rcf ? vzt_rd_get_32(&lt->numfacbytes,0) : 0;
+		rcf = fread(&lt->longestname, 4, 1, lt->handle);	lt->longestname = rcf ? vzt_rd_get_32(&lt->longestname,0) : 0;
+		rcf = fread(&lt->zfacnamesize, 4, 1, lt->handle);	lt->zfacnamesize = rcf ? vzt_rd_get_32(&lt->zfacnamesize,0) : 0;
+		rcf = fread(&lt->zfacname_predec_size, 4, 1, lt->handle);lt->zfacname_predec_size = rcf ? vzt_rd_get_32(&lt->zfacname_predec_size,0) : 0;
+		rcf = fread(&lt->zfacgeometrysize, 4, 1, lt->handle);	lt->zfacgeometrysize = rcf ? vzt_rd_get_32(&lt->zfacgeometrysize,0) : 0;
+		rcf = fread(&lt->timescale, 1, 1, lt->handle);		if(!rcf) lt->timescale = 0; /* no swap necessary */
 
 		fprintf(stderr, VZT_RDLOAD"%d facilities\n", lt->numfacs);
 		pos = ftello(lt->handle);
@@ -1686,21 +1808,26 @@ if(!(lt->handle=fopen(name, "rb")))
 		lt->flags = malloc(lt->numfacs * sizeof(vztint32_t));
 		lt->len = malloc(lt->numfacs * sizeof(vztint32_t));
 		lt->vindex_offset = malloc(lt->numfacs * sizeof(vztint32_t));
+		lt->longest_len = 32; /* big enough for decoded double in vzt_rd_process_block_single_factime() */
 
 		for(i=0;i<lt->numfacs;i++)
 			{
+			int j;
+
 			lt->rows[i] = vzt_rd_get_32(m+i*16, 0);
 			lt->msb[i] = vzt_rd_get_32(m+i*16, 4);
 			lt->lsb[i] = vzt_rd_get_32(m+i*16, 8);
-			lt->flags[i] = vzt_rd_get_32(m+i*16, 12);
+			lt->flags[i] = vzt_rd_get_32(m+i*16, 12) & VZT_RD_SYM_MASK; /* strip out unsupported bits */
+
+			j = (!(lt->flags[i] & VZT_RD_SYM_F_ALIAS)) ? i : vzt_rd_get_alias_root(lt, i);
 
 			if(!(lt->flags[i] & (VZT_RD_SYM_F_INTEGER|VZT_RD_SYM_F_STRING|VZT_RD_SYM_F_DOUBLE)))
 				{
-				lt->len[i] = (lt->msb[i] <= lt->lsb[i]) ? (lt->lsb[i] - lt->msb[i] + 1) : (lt->msb[i] - lt->lsb[i] + 1);
+				lt->len[i] = (lt->msb[j] <= lt->lsb[j]) ? (lt->lsb[j] - lt->msb[j] + 1) : (lt->msb[j] - lt->lsb[j] + 1);
 				}
 				else
 				{
-				lt->len[i] = (lt->flags[i] & (VZT_RD_SYM_F_INTEGER|VZT_RD_SYM_F_STRING)) ? 32 : 64;
+				lt->len[i] = (lt->flags[j] & (VZT_RD_SYM_F_INTEGER|VZT_RD_SYM_F_STRING)) ? 32 : 64;
 				}
 
 			if(lt->len[i] > lt->longest_len)
@@ -1709,9 +1836,6 @@ if(!(lt->handle=fopen(name, "rb")))
 				}
 			}
 
-		lt->value_current_sector = malloc(lt->longest_len + 1);
-		lt->value_previous_sector = malloc(lt->longest_len + 1);
-
 		vindex_offset = 0; /* offset in value table */
 		for(lt->numrealfacs=0; lt->numrealfacs<lt->numfacs; lt->numrealfacs++)
 			{
@@ -1727,10 +1851,15 @@ if(!(lt->handle=fopen(name, "rb")))
 
 		if(lt->numrealfacs > lt->numfacs) lt->numrealfacs = lt->numfacs;
 
+		lt->value_current_sector = malloc(lt->longest_len + 1);
+		lt->value_previous_sector = malloc(lt->longest_len + 1);
+
 		free(m);
 
 		for(;;)
 			{
+			size_t rcf;
+
 			fseeko(lt->handle, 0L, SEEK_END);
 			fend=ftello(lt->handle);
 			if(pos>=fend) break;
@@ -1739,11 +1868,12 @@ if(!(lt->handle=fopen(name, "rb")))
 			/* fprintf(stderr, VZT_RDLOAD"seeking to block at %d (0x%08x)\n", pos, pos); */
 
 			b=calloc(1, sizeof(struct vzt_rd_block));
+			b->last_rd_value_idx = ~0;
 		
-			fread(&b->uncompressed_siz, 4, 1, lt->handle);			b->uncompressed_siz = vzt_rd_get_32(&b->uncompressed_siz,0);
-			fread(&b->compressed_siz, 4, 1, lt->handle);			b->compressed_siz = vzt_rd_get_32(&b->compressed_siz,0);
-			fread(&b->start, 8, 1, lt->handle);				b->start = vzt_rd_get_64(&b->start,0);
-			fread(&b->end, 8, 1, lt->handle);				b->end = vzt_rd_get_64(&b->end,0);
+			rcf = fread(&b->uncompressed_siz, 4, 1, lt->handle);	b->uncompressed_siz = rcf ? vzt_rd_get_32(&b->uncompressed_siz,0) : 0;
+			rcf = fread(&b->compressed_siz, 4, 1, lt->handle);	b->compressed_siz = rcf ? vzt_rd_get_32(&b->compressed_siz,0) : 0;
+			rcf = fread(&b->start, 8, 1, lt->handle);		b->start = rcf ? vzt_rd_get_64(&b->start,0) : 0;
+			rcf = fread(&b->end, 8, 1, lt->handle);			b->end = rcf ? vzt_rd_get_64(&b->end,0) : 0;
 			pos = ftello(lt->handle);
 
 			if((b->rle = (b->start > b->end)))
@@ -1820,6 +1950,182 @@ if(!(lt->handle=fopen(name, "rb")))
 return(lt);
 }
 
+/*
+ * experimental, only really useful for vztminer right now...
+ */
+void vzt_rd_vectorize(struct vzt_rd_trace *lt)
+{
+if((!lt)||(lt->vectorize))
+	{
+	return;
+	}
+	else
+	{
+	int old_longest_len = lt->longest_len;
+	int pmxlen = 31;
+	char *pbuff = malloc(pmxlen+1);
+	char *pname;
+	int plen, plen2;
+	int i, j;	
+	int pidx;
+	int num_after_combine = lt->numfacs;
+	int num_synvecs = 0;
+	int num_synalias = 0;
+	struct vzt_synvec_chain **synvec_chain = calloc(lt->numfacs, sizeof(struct vzt_synvec_chain *));
+
+	for(i=0;i<lt->numfacs-1;i++)
+		{
+		if(lt->len[i] != 1) continue;
+
+		pname = vzt_rd_get_facname(lt, i);
+		plen = strlen(pname);
+		if(plen > pmxlen) 
+			{
+			free(pbuff);
+			pbuff = malloc(plen+1);
+			}
+	
+		memcpy(pbuff, pname, plen);
+		pbuff[plen] = 0;
+		pidx = lt->msb[i];
+
+		for(j=i+1;j<lt->numfacs-1;j++)
+			{
+			pname = vzt_rd_get_facname(lt, j);
+			plen2 = strlen(pname);
+			if((plen != plen2) || (strcmp(pbuff, pname)) || (lt->len[j] != 1) || ((pidx != lt->msb[j]-1) && (pidx != lt->msb[j]+1)))
+				{
+				i = j-1;
+				break;
+				}
+	
+			pidx = lt->msb[j];
+			lt->len[i] += lt->len[j];
+			lt->lsb[i] = lt->lsb[j];
+			lt->len[j] = 0;
+			num_after_combine--;
+	
+			if(lt->len[i] > lt->longest_len)
+				{
+				lt->longest_len = lt->len[i];
+				}
+			}
+		}
+
+	for(i=lt->numrealfacs;i<lt->numfacs;i++)
+		{
+		if(lt->flags[i] & VZT_RD_SYM_F_ALIAS)	/* not necessary, only for sanity */
+			{
+			int j = vzt_rd_get_alias_root(lt, i);	
+			int k, l;
+	
+			if(lt->len[i])
+				{
+				if((lt->len[i]==1) && (lt->len[j]==1))
+					{
+					/* nothing to do, single bit alias */
+					continue;
+					}
+
+				if(lt->len[i]==lt->len[j])
+					{
+					int nfm1 = lt->numfacs-1;
+					if((i != nfm1) && (j != nfm1))
+						{
+						if(lt->len[i+1] && lt->len[j+1])
+							{
+							/* nothing to do, multi-bit alias */
+							continue;
+							}
+						}
+					}
+
+				if(1)	/* seems like this doesn't need to be conditional (for now) */
+					{
+					lt->vindex_offset[i] = lt->vindex_offset[j];
+					for(k=1;k<lt->len[i];k++)
+						{
+						lt->vindex_offset[k+i] = lt->vindex_offset[vzt_rd_get_alias_root(lt, k+i)];
+						}
+
+					if(synvec_chain[j])
+						{
+						int alias_found = 0;
+
+						for(k=0;k<synvec_chain[j]->num_entries;k++)
+							{
+							vztint32_t idx = synvec_chain[j]->chain[k];
+			
+							if(lt->len[i] == lt->len[idx])
+								{
+								for(l=0;l<lt->len[i];l++)
+									{
+									if(lt->vindex_offset[idx+l] != lt->vindex_offset[i+l]) break;
+									}
+
+								if(l == (lt->len[i]))
+									{
+									lt->rows[i] = idx;			/* already exists */
+									num_synalias++;
+									alias_found = 1;
+									break;
+									}
+								}
+							}
+
+						if(!alias_found)
+							{
+							synvec_chain[j] = realloc(synvec_chain[j], sizeof(struct vzt_synvec_chain) +
+									synvec_chain[j]->num_entries * sizeof(vztint32_t));
+	
+							synvec_chain[j]->chain[synvec_chain[j]->num_entries++] = i;
+							lt->flags[i] |= VZT_RD_SYM_F_SYNVEC;
+							lt->flags[i] &= ~VZT_RD_SYM_F_ALIAS;
+							lt->numrealfacs = i+1;				/* bump up to end */
+							num_synvecs++;
+							}
+						}
+						else
+						{
+						synvec_chain[j] = malloc(sizeof(struct vzt_synvec_chain));
+						synvec_chain[j]->num_entries = 1;
+						synvec_chain[j]->chain[0] = i;
+
+						lt->flags[i] |= VZT_RD_SYM_F_SYNVEC;
+						lt->flags[i] &= ~VZT_RD_SYM_F_ALIAS;
+						lt->numrealfacs = i+1;				/* bump up to end */
+						num_synvecs++;
+						}
+					}
+				}
+			}
+		}
+
+	for(i=0;i<lt->numfacs;i++)
+		{
+		if(synvec_chain[i]) free(synvec_chain[i]);
+		}
+
+	free(synvec_chain);
+
+	fprintf(stderr, VZT_RDLOAD"%d facilities (after vectorization)\n", num_after_combine);
+	if(num_synvecs)
+		{
+		fprintf(stderr, VZT_RDLOAD"%d complex vectors synthesized\n", num_synvecs);
+		if(num_synalias) fprintf(stderr, VZT_RDLOAD"%d complex aliases synthesized\n", num_synalias);
+		}
+	fprintf(stderr, VZT_RDLOAD"\n");
+
+	if(lt->longest_len != old_longest_len)
+		{
+		free(lt->value_current_sector); free(lt->value_previous_sector);
+
+		lt->value_current_sector = malloc(lt->longest_len + 1);
+		lt->value_previous_sector = malloc(lt->longest_len + 1);
+		}
+	}
+}
+
 
 struct vzt_rd_trace *vzt_rd_init(const char *name)
 {
@@ -1882,3 +2188,154 @@ if(lt)
 	free(lt);	
 	}
 }
+
+
+/*******************************************/
+
+/*
+ * read single fac value at a given time
+ */
+static char *vzt_rd_process_block_single_factime(struct vzt_rd_trace *lt, struct vzt_rd_block *b, vztint64_t simtime, vztint32_t idx)
+{
+int i;
+char *pnt=lt->value_current_sector;	/* convenient workspace mem */
+char *buf = lt->value_previous_sector;  /* convenient workspace mem */
+char *rcval = NULL;
+
+vzt_rd_block_vch_decode(lt, b);
+vzt_rd_pthread_mutex_lock(lt, &b->mutex);
+
+if((b->last_rd_value_simtime == simtime) && (b->last_rd_value_idx != ~0))
+	{
+	i = b->last_rd_value_idx;
+	}
+	else
+	{
+	int i_ok = 0;
+
+	for(i=0;i<b->num_time_ticks;i++) /* for(i=b->num_time_ticks-1; i>=0; i--) */
+		{
+		if(simtime == b->times[i])
+			{
+			i_ok = i; break;
+			}
+
+		if(simtime < b->times[i]) break;
+		i_ok = i;
+
+		/* replace with maximal bsearch if this caching doesn't work out */
+		}
+
+	b->last_rd_value_idx = i_ok;
+	b->last_rd_value_simtime = simtime;
+	}
+
+vzt_rd_fac_value(lt, b, i, idx, pnt);
+
+if(!(lt->flags[idx] & (VZT_RD_SYM_F_DOUBLE|VZT_RD_SYM_F_STRING)))
+	{
+	rcval = pnt;
+	}
+	else
+	{
+	if(lt->flags[idx] & VZT_RD_SYM_F_DOUBLE)
+		{
+		vzt_rd_double_xdr(pnt, buf);
+		rcval = buf;
+		}
+	else
+		{
+		unsigned int spnt=vzt_rd_make_sindex(pnt);
+		rcval = b->sindex[spnt];
+		}
+	}
+
+vzt_rd_pthread_mutex_unlock(lt, &b->mutex);
+return(rcval);
+}
+
+
+char *vzt_rd_value(struct vzt_rd_trace *lt, vztint64_t simtime, vztint32_t idx)
+{
+struct vzt_rd_block *b, *b2;
+char *rcval = NULL;
+
+if(lt)
+	{
+	b = lt->block_head;
+
+	if((simtime == lt->last_rd_value_simtime) && (lt->last_rd_value_block))
+		{
+		b = lt->last_rd_value_block;
+		goto b_chk;		
+		}
+		else
+		{
+		lt->last_rd_value_simtime = simtime;
+		}
+
+	while(b)
+		{
+		if((b->start > simtime) || (b->end < simtime))
+			{
+			b = b->next;
+			continue;
+			}
+
+		lt->last_rd_value_block = b;
+b_chk:		if((!b->mem)&&(!b->short_read_ignore))
+			{
+			vzt_rd_decompress_blk(lt, b, 0);
+			}
+
+		if(b->mem)
+			{
+			rcval = vzt_rd_process_block_single_factime(lt, b, simtime, idx);
+			break;
+			}
+
+		b=b->next;
+		}
+	}
+	else
+	{
+	return(NULL);
+	}
+
+if((b)&&(lt->numblocks > 2))	/* no sense freeing up when not so many blocks */
+	{
+	vztint64_t block_mem_consumed;
+
+	vzt_rd_pthread_mutex_lock(lt, &lt->mutex);			
+	block_mem_consumed = lt->block_mem_consumed;
+	vzt_rd_pthread_mutex_unlock(lt, &lt->mutex);
+
+	if(block_mem_consumed > lt->block_mem_max)
+		{
+		b2 = lt->block_head;
+		while(b2)
+			{
+			if((block_mem_consumed > lt->block_mem_max) && (b2 != b))
+				{
+				vzt_rd_pthread_mutex_lock(lt, &lt->mutex);
+				lt->block_mem_consumed -= b2->uncompressed_siz;
+				vzt_rd_pthread_mutex_unlock(lt, &lt->mutex);
+				vzt_rd_block_vch_free(lt, b2, 0);
+				}
+	
+			b2 = b2->next;
+			}
+		}
+	}
+
+return(rcval);
+}
+
+/*
+ * $Id: vzt_read.c,v 1.2 2007/04/20 02:08:19 gtkwave Exp $
+ * $Log: vzt_read.c,v $
+ * Revision 1.2  2007/04/20 02:08:19  gtkwave
+ * initial release
+ *
+ */
+
