@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 Tony Bybell.
+ * Copyright (c) 2003-2007 Tony Bybell.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -20,12 +20,24 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#if !defined _MSC_VER && !defined __MINGW32__ && !defined __CYGWIN__
+#ifdef _AIX
+#pragma alloca
+#endif
+
+#include <config.h>
+#include <wavealloca.h>
+
+#if defined(__CYGWIN__) || defined(__MINGW32__) 
+#undef HAVE_RPC_XDR_H
+#endif
+
+#if HAVE_RPC_XDR_H 
 #include <rpc/types.h>
 #include <rpc/xdr.h>
 #endif
 #include "vzt_write.h"
 
+
 /*
  * in-place sort to keep chained facs from migrating...
  */
@@ -94,6 +106,8 @@ if(lt)
 		return(BZ2_bzdopen(fd, mode));
 		}
 	}
+
+return(NULL);
 }
 
 static _VZT_WR_INLINE int vzt_gzclose(struct vzt_wr_trace *lt, void *file)
@@ -110,6 +124,8 @@ if(lt)
 		return(0);
 		}
 	}
+
+return(0);
 }
 
 static _VZT_WR_INLINE int vzt_gzflush(struct vzt_wr_trace *lt, void *file, int flush)
@@ -125,6 +141,8 @@ if(lt)
 		return(BZ2_bzflush(file));
 		}
 	}
+
+return(0);
 }
 
 static _VZT_WR_INLINE int vzt_gzwrite(struct vzt_wr_trace *lt, void *file, void* buf, unsigned len)
@@ -140,15 +158,16 @@ if(lt)
 		return(BZ2_bzwrite(file, buf, len));
 		}
 	}
+
+return(0);
 }
 
 
 /************************ splay ************************/
 
-static _VZT_WR_INLINE int cmp_l(vztint32_t i, vztint32_t j)
-{
-return((i != j) - ((i < j) << 1));
-}
+#define cmp_l(i,j) ((int)(-(i<j) + (i>j)))
+#define cmp_l_lt(i,j) (i<j)
+#define cmp_l_gt(i,j) (i>j)
 
 static int vzt_wr_dsvzt_success;
 
@@ -168,7 +187,7 @@ static vzt_wr_dsvzt_Tree * vzt_wr_dsvzt_splay (vztint32_t i, vzt_wr_dsvzt_Tree *
 	dir = cmp_l(i, t->item);
 	if (dir < 0) {
 	    if (t->left == NULL) break;
-	    if (cmp_l(i, t->left->item)<0) {
+	    if (cmp_l_lt(i, t->left->item)) {
 		y = t->left;                           /* rotate right */
 		t->left = y->right;
 		y->right = t;
@@ -180,7 +199,7 @@ static vzt_wr_dsvzt_Tree * vzt_wr_dsvzt_splay (vztint32_t i, vzt_wr_dsvzt_Tree *
 	    t = t->left;
 	} else if (dir > 0) {
 	    if (t->right == NULL) break;
-	    if (cmp_l(i, t->right->item)>0) {
+	    if (cmp_l_gt(i, t->right->item)) {
 		y = t->right;                          /* rotate left */
 		t->right = y->left;
 		y->left = t;
@@ -484,7 +503,7 @@ lt->position+=4;
 return(nmemb);
 }
 
-
+#if 0
 static int vzt_wr_emit_u64z(struct vzt_wr_trace *lt, int valueh, int valuel)
 {
 int rc;
@@ -496,7 +515,7 @@ if((rc=vzt_wr_emit_u32z(lt, valueh)))
 
 return(rc);
 }
-
+#endif
 
 static int vzt_wr_emit_stringz(struct vzt_wr_trace *lt, char *value)
 {
@@ -658,6 +677,11 @@ if(s->namlen<3) return;
 lastch--;
 while(lastch!=s->name)
 	{
+        if(*lastch=='.')
+                {
+                return; /* MTI SV [0.3] notation for implicit vars */
+                }
+
 	if(*lastch=='[')
 		{
 		*lastch=0x00;	
@@ -765,7 +789,7 @@ if((lt)&&(lt->numfacs))
 				vzt_wr_emit_u32z(lt, lt->sorted_facs[i]->rows);
 				vzt_wr_emit_u32z(lt, lt->sorted_facs[i]->msb);
 				vzt_wr_emit_u32z(lt, lt->sorted_facs[i]->lsb);
-				vzt_wr_emit_u32z(lt, lt->sorted_facs[i]->flags);
+				vzt_wr_emit_u32z(lt, lt->sorted_facs[i]->flags & VZT_WR_SYM_MASK);
 				}
 				else
 				{
@@ -1078,8 +1102,10 @@ for(cnt = 0; cnt < lt->break_header_size; cnt += sizeof(buf))
 		seg = sizeof(buf);
 		}
 
-	fread(buf, seg, 1, clone);
-	fwrite(buf, seg, 1, f2);
+	if(fread(buf, seg, 1, clone))
+		{
+		if(!fwrite(buf, seg, 1, f2)) break; /* write error! */
+		}
 	}
 
 fclose(clone);
@@ -1344,7 +1370,7 @@ if((lt->timegranule >= lt->maxgranule)||(do_finalize))
 		}
 
 		{
-		vztint32_t buf[lt->maxgranule];
+		vztint32_t * buf = alloca(lt->maxgranule * sizeof(vztint32_t));
 		memset(buf, 0, lt->maxgranule * sizeof(vztint32_t));
 		if(lt->rle) lt->rle_start = 0;
 		vzt_wr_recurse_reorder_dict(lt->dict, lt, &newval, buf, 1);
@@ -1565,8 +1591,10 @@ return(vzt_wr_emit_value_bit_string(lt, s, row, vzt_wr_expand_integer_to_bits(s-
 int vzt_wr_emit_value_double(struct vzt_wr_trace *lt, struct vzt_wr_symbol *s, unsigned int row, double value)
 {
 char xdrdata[8];
-#if !defined _MSC_VER && !defined __MINGW32__ && !defined __CYGWIN__
+#if HAVE_RPC_XDR_H 
 XDR x;
+#else
+const vztint32_t endian_matchword = 0x12345678;
 #endif
 vztint32_t msk, msk_n;
 int i;
@@ -1589,15 +1617,22 @@ while(s->aliased_to)	/* find root alias if exists */
 	s=s->aliased_to;
 	}
 
-#if !defined _MSC_VER && !defined __MINGW32__ && !defined __CYGWIN__
+#if HAVE_RPC_XDR_H 
 xdrmem_create(&x, xdrdata, sizeof(xdrdata), XDR_ENCODE);
 xdr_double(&x, &value);
 #else
 /* byte ordering in windows is reverse of XDR (on x86, that is) */
-for(i=0;i<8;i++)
-        {
-        xdrdata[i] = ((char *)&value)[7-i];
-        }
+if(*((char *)&endian_matchword) == 0x78)
+	{
+	for(i=0;i<8;i++)
+	        {
+	        xdrdata[i] = ((char *)&value)[7-i];
+	        }
+	}
+	else
+	{
+	memcpy(xdrdata, &value, sizeof(double));	/* big endian, don't bytereverse */
+	}
 #endif
 
 lt->bumptime = 1;
@@ -1946,4 +1981,11 @@ if(lt)
 	}
 }
 
+/*
+ * $Id: vzt_write.c,v 1.2 2007/04/20 02:08:19 gtkwave Exp $
+ * $Log: vzt_write.c,v $
+ * Revision 1.2  2007/04/20 02:08:19  gtkwave
+ * initial release
+ *
+ */
 
