@@ -7,12 +7,20 @@
  * of the License, or (at your option) any later version.
  */
 
+
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include <gtk/gtk.h>
 #include "gtk12compat.h"
 #include "analyzer.h"
-#include "aet.h"
+#include "symbol.h"
 #include "lx2.h"
 #include "vcd.h"
+#include "busy.h"
 #include "debug.h"
 
 char hier_grouping=1;
@@ -268,7 +276,7 @@ static void entrybox_local(char *title, int width, char *default_text, int maxch
     cleanup_e=func;
 
     /* create a new modal window */
-    window1 = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    window1 = gtk_window_new(disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     gtk_grab_add(window1);
     gtk_widget_set_usize( GTK_WIDGET (window1), width, 60);
     gtk_window_set_title(GTK_WINDOW (window1), title);
@@ -326,6 +334,8 @@ int i;
  
 if(!h_selectedtree) return;
 
+set_window_busy(widget);
+
 for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
         {
         struct symbol *s;  
@@ -404,6 +414,8 @@ for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
 		}
         }
 
+set_window_idle(widget);
+
 MaxSignalLength();
 signalarea_configure_event(signalarea, NULL);
 wavearea_configure_event(wavearea, NULL);
@@ -421,6 +433,8 @@ memcpy(&tcache,&traces,sizeof(Traces));
 traces.total=0; 
 traces.first=traces.last=NULL;
 
+set_window_busy(widget);
+
 for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
         {
         struct symbol *s;  
@@ -499,6 +513,8 @@ for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
 		}
         }
 
+set_window_idle(widget);
+
 traces.buffercount=traces.total;
 traces.buffer=traces.first;
 traces.bufferlast=traces.last;
@@ -530,6 +546,8 @@ memcpy(&tcache,&traces,sizeof(Traces));
 traces.total=0;
 traces.first=traces.last=NULL;  
 
+set_window_busy(widget);
+
 for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
         {
         struct symbol *s;  
@@ -608,6 +626,8 @@ for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
 		}
         }
  
+set_window_idle(widget);
+
 tfirst=traces.first; tlast=traces.last; /* cache for highlighting */
 
 traces.buffercount=traces.total;
@@ -839,7 +859,7 @@ void hier_searchbox(char *title, GtkSignalFunc func)
     num_rows=selected_rows=0;
 
     /* create a new modal window */
-    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    window = gtk_window_new(disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW (window), title);
     gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                        (GtkSignalFunc) destroy_callback, NULL);
@@ -996,3 +1016,12 @@ void hier_searchbox(char *title, GtkSignalFunc func)
 
     refresh_tree(current_tree);
 }
+
+/*
+ * $Id: hiersearch.c,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: hiersearch.c,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
