@@ -5,9 +5,11 @@
  * Compile this file with -DNO_GZCOMPRESS to avoid the compression code.
  */
 
-/* @(#) $Id$ */
+/* @(#) $Id: gzio.c,v 1.1.1.1 2007/04/20 01:16:32 gtkwave Exp $ */
 
-#if defined _MSC_VER || defined __MINGW32__
+#include <config.h>
+
+#ifndef HAVE_FSEEKO 
 #define fseeko fseek
 #define ftello ftell
 #endif
