@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2003-2005.
+ * Copyright (c) Tony Bybell 2003-2006.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include "lx2.h"
 
@@ -17,12 +18,13 @@
 #include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
-#include "aet.h"
+#include "symbol.h"
 #include "vcd.h"
 #include "lxt.h"
 #include "vzt.h"
 #include "lxt2_read.h"
 #include "debug.h"
+#include "busy.h"
 
 
 /*
@@ -47,6 +49,8 @@ struct Node *n;
 struct symbol *s, *prevsymroot=NULL, *prevsym=NULL;
 signed char scale;
 unsigned int numalias = 0;
+struct symbol *sym_block = NULL;
+struct Node *node_block = NULL;
 
 lx2 = lxt2_rd_init(fname);
 if(!lx2)
@@ -65,6 +69,8 @@ exponent_to_time_scale(scale);
 numfacs=lxt2_rd_get_num_facs(lx2);
 mvlfacs=(struct fac *)calloc_2(numfacs,sizeof(struct fac));
 lx2_table=(struct lx2_entry *)calloc_2(numfacs, sizeof(struct lx2_entry));
+sym_block = (struct symbol *)calloc_2(numfacs, sizeof(struct symbol));
+node_block=(struct Node *)calloc_2(numfacs,sizeof(struct Node));
 
 for(i=0;i<numfacs;i++)
 	{
@@ -138,8 +144,8 @@ for(i=0;i<numfacs;i++)
 
 	if((f->len>1)&& (!(f->flags&(LXT2_RD_SYM_F_INTEGER|LXT2_RD_SYM_F_DOUBLE|LXT2_RD_SYM_F_STRING))) )
 		{
-		sprintf(buf, "%s[%d:%d]", mvlfacs[i].name,mvlfacs[i].msb, mvlfacs[i].lsb);
-		str=malloc_2(strlen(buf)+1);
+		int len = sprintf(buf, "%s[%d:%d]", mvlfacs[i].name,mvlfacs[i].msb, mvlfacs[i].lsb);
+		str=malloc_2(len+1);
 		if(!alt_hier_delimeter)
 			{
 			strcpy(str, buf);
@@ -148,7 +154,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+                s=&sym_block[i];
+                symadd_name_exists_sym_exists(s,str,0);
 		prevsymroot = prevsym = NULL;
 		}
 	else if ( 
@@ -159,8 +166,8 @@ for(i=0;i<numfacs;i++)
 			(mvlfacs[i].msb!=-1)&&(mvlfacs[i].lsb!=-1))
 		)
 		{
-		sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
-		str=malloc_2(strlen(buf)+1);
+		int len = sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
+		str=malloc_2(len+1);
 		if(!alt_hier_delimeter)
 			{
 			strcpy(str, buf);
@@ -169,7 +176,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+                s=&sym_block[i];
+                symadd_name_exists_sym_exists(s,str,0);
 		if((prevsym)&&(i>0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name)))	/* allow chaining for search functions.. */
 			{
 			prevsym->vec_root = prevsymroot;
@@ -193,7 +201,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, mvlfacs[i].name, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+                s=&sym_block[i];
+                symadd_name_exists_sym_exists(s,str,0);
 		prevsymroot = prevsym = NULL;
 
 		if(f->flags&LXT2_RD_SYM_F_INTEGER)
@@ -214,7 +223,7 @@ for(i=0;i<numfacs;i++)
                 curnode=s;   
                 }
 
-        n=(struct Node *)calloc_2(1,sizeof(struct Node));
+	n=&node_block[i];
         n->nname=s->name;
         n->mvlfac = mvlfacs+i;
 	mvlfacs[i].working_node = n;
@@ -305,7 +314,7 @@ if(fast_tree_sort)
                 build_tree_from_name(facs[i]->name, i);
                 }
 /* SPLASH */                            splash_sync(4, 5);
-        treeprune(treeroot);
+        treegraft(treeroot);
         
         fprintf(stderr, LXT2_RDLOAD"Sorting facility hierarchy tree.\n");
         treesort(treeroot, NULL);
@@ -327,7 +336,7 @@ if(fast_tree_sort)
 		curnode=curnode->nextinaet;
 		while((ch=(*subst)))
 			{	
-			if(ch==hier_delimeter) { *subst=0x01; }	/* forces sort at hier boundaries */
+			if(ch==hier_delimeter) { *subst=VCDNAM_HIERSORT; }	/* forces sort at hier boundaries */
 			subst++;
 			}
 		}
@@ -343,7 +352,7 @@ if(fast_tree_sort)
 		subst=facs[i]->name;
 		while((ch=(*subst)))
 			{	
-			if(ch==0x01) { *subst=hier_delimeter; }	/* restore back to normal */
+			if(ch==VCDNAM_HIERSORT) { *subst=hier_delimeter; }	/* restore back to normal */
 			subst++;
 			}
 		}
@@ -359,7 +368,8 @@ if(fast_tree_sort)
 		build_tree_from_name(facs[i]->name, i);
 		}
 /* SPLASH */                            splash_sync(5, 5);
-	treeprune(treeroot);
+	treegraft(treeroot);
+	treesort(treeroot, NULL);
 	}
 
 min_time = first_cycle; max_time=last_cycle;
@@ -410,7 +420,16 @@ struct HistEnt *htemp = histent_calloc();
 struct lx2_entry *l2e = lx2_table+(*facidx);
 struct fac *f = mvlfacs+(*facidx);
 
-/* fprintf(stderr, "%Ld %d %s\n", *time, *facidx, *value); */
+static int busycnt = 0;
+
+busycnt++;
+if(busycnt==WAVE_BUSY_ITER)
+        {
+        busy_window_refresh();
+        busycnt = 0;
+        }
+
+/* fprintf(stderr, "%lld %d %s\n", *time, *facidx, *value); */
 
 if(!(f->flags&(LXT2_RD_SYM_F_DOUBLE|LXT2_RD_SYM_F_STRING)))
 	{
@@ -495,6 +514,7 @@ switch(is_lx2)
 	case LXT2_IS_AET2: import_ae2_trace(np); return;
 #endif
 	case LXT2_IS_VZT:  import_vzt_trace(np); return;
+	case LXT2_IS_VLIST: import_vcd_trace(np); return;
 	default: break; /* fallthrough */
 	}
 
@@ -604,6 +624,7 @@ switch(is_lx2)
         case LXT2_IS_AET2: ae2_set_fac_process_mask(np); return;
 #endif
         case LXT2_IS_VZT:  vzt_set_fac_process_mask(np); return;
+	case LXT2_IS_VLIST: vcd_set_fac_process_mask(np); return;
         default: break; /* fallthrough */
         }
 
@@ -637,6 +658,7 @@ switch(is_lx2)
         case LXT2_IS_AET2: ae2_import_masked(); return;
 #endif
         case LXT2_IS_VZT:  vzt_import_masked(); return;
+        case LXT2_IS_VLIST:  vcd_import_masked(); return;
         default: break; /* fallthrough */
         }
 
@@ -657,7 +679,9 @@ if(cnt>100)
 	}
 
 
+set_window_busy(NULL);
 lxt2_rd_iter_blocks(lx2, lx2_callback, NULL);
+set_window_idle(NULL);
 
 for(txidx=0;txidx<numfacs;txidx++)
 	{
@@ -729,3 +753,12 @@ for(txidx=0;txidx<numfacs;txidx++)
 		}
 	}
 }
+
+/*
+ * $Id: lx2.c,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: lx2.c,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
