@@ -17,7 +17,7 @@
 #include "vcd.h"
 #include "ae2.h"
 
-enum LXT2_Loader_Type_Encodings { LXT2_IS_INACTIVE, LXT2_IS_LXT2, LXT2_IS_VZT, LXT2_IS_AET2 };
+enum LXT2_Loader_Type_Encodings { LXT2_IS_INACTIVE, LXT2_IS_LXT2, LXT2_IS_VZT, LXT2_IS_AET2, LXT2_IS_VLIST };
 
 struct lx2_entry
 {
@@ -34,3 +34,12 @@ void lx2_set_fac_process_mask(nptr np);
 void lx2_import_masked(void);
 
 #endif
+
+/*
+ * $Id: lx2.h,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: lx2.h,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
