@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2001-2004.
+ * Copyright (c) Tony Bybell 2001-2006.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include "zlib.h"
 #include "bzlib.h"
@@ -24,13 +25,12 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include "aet.h"
+#include "symbol.h"
 #include "vcd.h"
 #include "lxt.h"
 #include "bsearch.h"
 #include "debug.h"
 
-
 /****************************************************************************/
 
 /*
@@ -377,9 +377,11 @@ return(rv);
 
 static TimeType bsearch_mvl_timechain(int key)
 {
+void *bsearch_dummy_rv;
+
 max_compare_time_tc=-1; max_compare_pos_tc=-1; 
 
-bsearch((void *)&key, (void *)positional_information, total_cycles, sizeof(int), compar_mvl_timechain);
+bsearch_dummy_rv = bsearch((void *)&key, (void *)positional_information, total_cycles, sizeof(int), compar_mvl_timechain);
 if((max_compare_pos_tc<=0)||(max_compare_time_tc<0)) 
         {
         max_compare_pos_tc=0; /* aix bsearch fix */
@@ -438,7 +440,7 @@ DEBUG(printf(LXTHDR"section offs for name decompression = %08x of len %d\n", off
 DEBUG(printf(LXTHDR"Decompressed size is %d bytes (vs %d)\n", rc, total_mem));	
 if(rc!=total_mem) { fprintf(stderr, LXTHDR"decompression size disparity  %d bytes (vs %d)\n", rc, total_mem); exit(255); }
 
-dict_string_mem_array = (char **)calloc(dict_num_entries, sizeof(char *));
+dict_string_mem_array = (char **)calloc_2(dict_num_entries, sizeof(char *));
 pnt = decmem;
 for(i=0;i<dict_num_entries;i++)
 	{
@@ -572,7 +574,7 @@ for(i=0;i<numfacs;i++)
 	offs+=0x10;
 	}
 
-lt_buf = malloc_2(lt_len ? lt_len : 1);
+lt_buf = malloc_2(lt_len>255 ? lt_len : 256);	/* in order to keep trivial LXT files from overflowing their buffer */
 
 if(zfacgeometry_size)
 	{
@@ -732,7 +734,7 @@ if(time_table_offset)
 	DEBUG(printf(LXTHDR"First cycle: %d\n", first_cycle));
 	last_cycle=get_64(offs+8);
 	DEBUG(printf(LXTHDR"Last cycle: %d\n", last_cycle));
-	DEBUG(printf(LXTHDR"Total cycles (actual): %Ld\n", last_cycle-first_cycle+1));
+	DEBUG(printf(LXTHDR"Total cycles (actual): %lld\n", last_cycle-first_cycle+1));
 	
 	/* rebuild time table from its deltas... */
 
@@ -837,8 +839,7 @@ if(zchg_size)
 		fprintf(stderr, LXTHDR"Compressed change records detected, making tempfile...\n");
 		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
 
-		read(fd, &testbyte, 2);
-		is_bz2 = (testbyte[0]=='B')&&(testbyte[1]=='Z');
+		is_bz2 = (read(fd, &testbyte, 2))&&(testbyte[0]=='B')&&(testbyte[1]=='Z');
 
 		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
 
@@ -881,7 +882,7 @@ if(zchg_size)
 	
 		fd = fd2;
 		mm=mmap(NULL, zchg_predec_size, PROT_READ, MAP_SHARED, fd, 0);	
-		
+
 		mm=(void *)((char *)mm-4); /* because header and version don't exist in packed change records */
 		}
 		else
@@ -905,8 +906,7 @@ if(zchg_size)
 #else		
 		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
 
-		read(fd, &testbyte, 2);
-		is_bz2 = (testbyte[0]=='B')&&(testbyte[1]=='Z');
+		is_bz2 = (read(fd, &testbyte, 2))&&(testbyte[0]=='B')&&(testbyte[1]=='Z');
 
 		if(change_field_offset != lseek(fd, change_field_offset, SEEK_SET)) { fprintf(stderr, LXTHDR"lseek error at offset %08x\n", (unsigned int)change_field_offset); exit(255); }
 #endif
@@ -1252,7 +1252,7 @@ if(!sync_table_offset)
 		else
 #endif
 		{
-		free((char *)mm+4);
+		free_2((char *)mm+4);
 		}
 
 	fd = recfd;            
@@ -1432,6 +1432,8 @@ struct Node *n;
 struct symbol *s, *prevsymroot=NULL, *prevsym=NULL;
 off_t tagpnt;
 int tag;
+struct symbol *sym_block = NULL;
+struct Node *node_block = NULL;
 
 fd=open(fname, O_RDONLY);
 if(fd<0)
@@ -1518,7 +1520,7 @@ if(exclude_offset)
 		bs = get_64(offset); offset+=8;
 		be = get_64(offset); offset+=8;
 
-		bt = calloc(1, sizeof(struct blackout_region_t));
+		bt = calloc_2(1, sizeof(struct blackout_region_t));
 		bt->bstart = bs;
 		bt->bend = be;
 		bt->next = blackout_regions;
@@ -1552,7 +1554,7 @@ if(!facname_offset)
 
 numfacs=get_32(facname_offset);
 DEBUG(printf(LXTHDR"Number of facs: %d\n", numfacs));
-mvlfacs=(struct fac *)calloc(numfacs,sizeof(struct fac));
+mvlfacs=(struct fac *)calloc_2(numfacs,sizeof(struct fac));
 
 if(initial_value_offset)
 	{
@@ -1597,6 +1599,9 @@ if(!hier_was_explicitly_set)    /* set default hierarchy split char */
         hier_delimeter='.';
         }
 
+sym_block = (struct symbol *)calloc_2(numfacs, sizeof(struct symbol));
+node_block = (struct Node *)calloc_2(numfacs,sizeof(struct Node));
+
 for(i=0;i<numfacs;i++)
         {
 	char buf[4096];
@@ -1620,8 +1625,8 @@ for(i=0;i<numfacs;i++)
 
 	if((f->len>1)&& (!(f->flags&(LT_SYM_F_INTEGER|LT_SYM_F_DOUBLE|LT_SYM_F_STRING))) )
 		{
-		sprintf(buf, "%s[%d:%d]", mvlfacs[i].name,mvlfacs[i].msb, mvlfacs[i].lsb);
-		str=malloc_2(strlen(buf)+1);
+		int len = sprintf(buf, "%s[%d:%d]", mvlfacs[i].name,mvlfacs[i].msb, mvlfacs[i].lsb);
+		str=malloc_2(len+1);
 		if(!alt_hier_delimeter)
 			{
 			strcpy(str, buf);
@@ -1630,7 +1635,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+                s=&sym_block[i];
+                symadd_name_exists_sym_exists(s,str,0);
 		prevsymroot = prevsym = NULL;
 		}
 	else if ( 
@@ -1641,8 +1647,8 @@ for(i=0;i<numfacs;i++)
 			(mvlfacs[i].msb!=-1)&&(mvlfacs[i].lsb!=-1))
 		)
 		{
-		sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
-		str=malloc_2(strlen(buf)+1);
+		int len = sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
+		str=malloc_2(len+1);
 		if(!alt_hier_delimeter)
 			{
 			strcpy(str, buf);
@@ -1651,7 +1657,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+                s=&sym_block[i];
+                symadd_name_exists_sym_exists(s,str,0);
 		if((prevsym)&&(i>0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name)))	/* allow chaining for search functions.. */
 			{
 			prevsym->vec_root = prevsymroot;
@@ -1675,7 +1682,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, mvlfacs[i].name, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+                s=&sym_block[i];
+                symadd_name_exists_sym_exists(s,str,0);
 		prevsymroot = prevsym = NULL;
 
 		if(f->flags&LT_SYM_F_INTEGER)
@@ -1696,7 +1704,7 @@ for(i=0;i<numfacs;i++)
                 curnode=s;   
                 }
 
-        n=(struct Node *)calloc_2(1,sizeof(struct Node));
+	n=&node_block[i];
         n->nname=s->name;
         n->mvlfac = mvlfacs+i;
 
@@ -1707,13 +1715,13 @@ for(i=0;i<numfacs;i++)
 		ext->lsi = mvlfacs[i].lsb;
 		n->ext = ext;
 		}
-                 
+
         n->head.time=-1;        /* mark 1st node as negative time */
         n->head.v.h_val=AN_X;
         s->n=n;
         }
 
-free(mvlfacs[0].name);		/* the start of the big decompression buffer */
+free_2(mvlfacs[0].name);	/* the start of the big decompression buffer */
 for(i=0;i<numfacs;i++)
 	{
 	mvlfacs[i].name = NULL;
@@ -1732,7 +1740,7 @@ for(i=0;i<numfacs;i++)
 	curnode=curnode->nextinaet;
 	while((ch=(*subst)))
 		{	
-		if(ch==hier_delimeter) { *subst=0x01; }	/* forces sort at hier boundaries */
+		if(ch==hier_delimeter) { *subst=VCDNAM_HIERSORT; }	/* forces sort at hier boundaries */
 		subst++;
 		}
 	}
@@ -1748,7 +1756,7 @@ for(i=0;i<numfacs;i++)
 	subst=facs[i]->name;
 	while((ch=(*subst)))
 		{	
-		if(ch==0x01) { *subst=hier_delimeter; }	/* restore back to normal */
+		if(ch==VCDNAM_HIERSORT) { *subst=hier_delimeter; }	/* restore back to normal */
 		subst++;
 		}
 
@@ -1767,7 +1775,8 @@ for(i=0;i<numfacs;i++)
 build_tree_from_name(facs[i]->name, i);
 }
 /* SPLASH */                            splash_sync(5, 5);
-treeprune(treeroot);
+treegraft(treeroot);
+treesort(treeroot, NULL);
 fprintf(stderr, "built.\n\n");
 
 #ifdef DEBUG_FACILITIES
@@ -1858,7 +1867,7 @@ offs=f->lastchange;
 tmval=LLDescriptor(-1);
 prevtmval = LLDescriptor(-1);
 len = np->mvlfac->len;
-         
+
 histent_tail = htemp = histent_calloc();
 if(len>1)
 	{
@@ -1945,7 +1954,7 @@ while(offs)
 
 		if(len>1)
 			{
-			DEBUG(fprintf(stderr, "!!! DELTA = %Ld\n", delta));
+			DEBUG(fprintf(stderr, "!!! DELTA = %lld\n", delta));
 			DEBUG(fprintf(stderr, "!!! offsminus1 = %08x\n", offsminus1));
 
 			if(!lxt_clock_compress_to_z)
@@ -2030,7 +2039,7 @@ while(offs)
 				DEBUG(fprintf(stderr, "!!! RLE0 = '%08x'\n", rle_delta[0]));
 				DEBUG(fprintf(stderr, "!!! RLE1 = '%08x'\n", rle_delta[1]));
 
-				DEBUG(fprintf(stderr, "!!! time_offsminus1 = %Ld\n", time_offsminus1));
+				DEBUG(fprintf(stderr, "!!! time_offsminus1 = %lld\n", time_offsminus1));
 
 				tmval = time_offsminus1 + (delta * reps);
 				for(rcnt=0;rcnt<reps;rcnt++)
@@ -2041,7 +2050,7 @@ while(offs)
 						((j/2)+(j&0))*rle_delta[1] + 
 						((j/2)+(j&1))*rle_delta[0];
 
-					DEBUG(fprintf(stderr, "!!! %Ld -> '%08x'\n", tmval, res));
+					DEBUG(fprintf(stderr, "!!! %lld -> '%08x'\n", tmval, res));
 
 					for(k=0;k<len;k++)
 						{
@@ -2207,7 +2216,7 @@ while(offs)
 
 			if(double_is_native)
 				{
-				htemp->v.h_vector = ((unsigned char *)mm+offs_dbl);
+				htemp->v.h_vector = ((char *)mm+offs_dbl);
 				DEBUG(printf(LXTHDR"Added double '%.16g'\n", *((double *)(mm+offs_dbl))));
 				}
 				else
@@ -2227,7 +2236,7 @@ while(offs)
 			htemp = histent_calloc();
 			htemp->flags = HIST_REAL|HIST_STRING;
 
-			htemp->v.h_vector = ((unsigned char *)mm+offs_str);
+			htemp->v.h_vector = ((char *)mm+offs_str);
 			DEBUG(printf(LXTHDR"Added string '%s'\n", (unsigned char *)mm+offs_str));
 			htemp->time = tmval;
 			htemp->next = histent_head;
@@ -2337,3 +2346,12 @@ np->head.next = histent_head;
 np->curr = histent_tail;
 np->numhist++;
 }
+
+/*
+ * $Id: lxt.c,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: lxt.c,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
