@@ -10,6 +10,8 @@
 #ifndef __MFMAIN_H__
 #define __MFMAIN_H__
 
+#include "busy.h"
+
 #define HAVE_PANED_PACK	/* undefine this if you have an older GTK */
 
 struct logfile_chain
@@ -19,6 +21,7 @@ char *name;
 };
 
 extern char *whoami;
+extern char *stems_name, *aet_name;
 
 GtkWidget *create_text(void);
 GtkWidget *create_zoom_buttons(void);
@@ -31,12 +34,39 @@ GtkWidget *create_time_box(void);
 GtkWidget *create_wavewindow(void);
 GtkWidget *create_signalwindow(void);
 
+/* Get/set the current size of the window.  */
+extern void get_window_size (int *x, int *y);
+extern void set_window_size (int x, int y);
+
+/* Get/set the x/y pos of the window */
+void get_window_xypos(int *root_x, int *root_y);
+void set_window_xypos(int root_x, int root_y);
+
+/* stems helper activation */
+int stems_are_active(void);
+void activate_stems_reader(char *stems_name);
+
+
+extern GtkWidget *mainwindow;
 extern GtkWidget *signalwindow;
 extern GtkWidget *wavewindow;
 extern char paned_pack_semantics;    /* 1 for paned_pack, 0 for paned_add */
 extern int initial_window_x, initial_window_y; /* inital window sizes */
 extern int initial_window_xpos, initial_window_ypos; /* inital window position */
+extern int xpos_delta, ypos_delta;
+
 extern int splash_disable;
 extern int possibly_use_rc_defaults;
 
+extern int stems_type;
+
 #endif
+
+/*
+ * $Id: main.h,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: main.h,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
