@@ -24,12 +24,25 @@
 #include "fgetdynamic.h"
 #include "strace.h"
 #include "debug.h"
-#include "aet.h"
+#include "symbol.h"
 #include "main.h"
 
 void get_main_menu (GtkWidget *, GtkWidget **menubar);
-void file_quit_cmd_callback (GtkWidget *widget, gpointer data);
+int file_quit_cmd_callback (GtkWidget *widget, gpointer data);
+int set_wave_menu_accelerator(char *str);
+
+int execute_script(char *name);
+FILE *script_handle;
 
 extern char *filesel_writesave;
 
 #endif
+
+/*
+ * $Id: menu.h,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: menu.h,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
