@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include "currenttime.h"
 #include "pixmaps.h"
 #include "debug.h"
@@ -145,3 +146,11 @@ gtk_widget_show(table2);
 return(table);
 }
    
+/*
+ * $Id: pagebuttons.c,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: pagebuttons.c,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
