@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include "pixmaps.h"
 
 
@@ -2809,3 +2810,12 @@ wave_info_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &wave_info_mask,
 wave_alert_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &wave_alert_mask,
 	&style->bg[GTK_STATE_NORMAL], (gchar **)wave_alert);
 }
+
+/*
+ * $Id: pixmaps.c,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: pixmaps.c,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
