@@ -33,15 +33,15 @@ extern GdkBitmap *zoom_larrow_mask;
 extern GdkPixmap *zoom_rarrow_pixmap;
 extern GdkBitmap *zoom_rarrow_mask;
 
-GdkPixmap *prev_page_pixmap;
-GdkBitmap *prev_page_mask;
-GdkPixmap *next_page_pixmap;
-GdkBitmap *next_page_mask;
+extern GdkPixmap *prev_page_pixmap;
+extern GdkBitmap *prev_page_mask;
+extern GdkPixmap *next_page_pixmap;
+extern GdkBitmap *next_page_mask;
 
-GdkPixmap *wave_info_pixmap;
-GdkBitmap *wave_info_mask;
-GdkPixmap *wave_alert_pixmap;
-GdkBitmap *wave_alert_mask;
+extern GdkPixmap *wave_info_pixmap;
+extern GdkBitmap *wave_info_mask;
+extern GdkPixmap *wave_alert_pixmap;
+extern GdkBitmap *wave_alert_mask;
 
 #ifdef WAVE_USE_GTK2
 
@@ -50,8 +50,17 @@ GdkBitmap *wave_alert_mask;
 
 void make_splash_pixmaps(GtkWidget *window);
 
-GdkPixmap *wave_splash_pixmap;
-GdkBitmap *wave_splash_mask;
+extern GdkPixmap *wave_splash_pixmap;
+extern GdkBitmap *wave_splash_mask;
 #endif
 
 #endif
+
+/*
+ * $Id: pixmaps.h,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: pixmaps.h,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
