@@ -16,11 +16,18 @@
  * C. 
  */
 
+
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include "currenttime.h"
 #include "analyzer.h"
-#include "aet.h"
+#include "symbol.h"
 #include "bsearch.h"
-#include "alloca.h"
+#include "wavealloca.h"
 #include "debug.h"
 #include "strace.h"
 #include "print.h"
@@ -680,16 +687,16 @@ ps_MaxSignalLength(void)
 					}
 				} else {
 					char           *str;
-					hptr            hptr;
+					hptr            h_ptr;
 
-					if ((hptr = bsearch_node(t->n.nd, tims.marker))) {
+					if ((h_ptr = bsearch_node(t->n.nd, tims.marker))) {
 						if (!t->n.nd->ext) {
 							str = (char *) calloc_2(1, 3 * sizeof(char));
 							str[0] = '=';
 							if (t->flags & TR_INVERT) {
-								str[1] = AN_STR_INV[hptr->v.h_val];
+								str[1] = AN_STR_INV[h_ptr->v.h_val];
 							} else {
-								str[1] = AN_STR[hptr->v.h_val];
+								str[1] = AN_STR[h_ptr->v.h_val];
 							}
 							t->asciivalue = str;
 							vlen = gdk_string_measure(signalfont, str);
@@ -697,18 +704,18 @@ ps_MaxSignalLength(void)
 						} else {
 							char           *str2;
 
-							if (hptr->flags & HIST_REAL) {
-								if (!(hptr->flags & HIST_STRING)) {
+							if (h_ptr->flags & HIST_REAL) {
+								if (!(h_ptr->flags & HIST_STRING)) {
 									str =
-										convert_ascii_real((double *) hptr->
+										convert_ascii_real((double *) h_ptr->
 												   v.h_vector);
 								} else {
 									str =
-										convert_ascii_string((char *) hptr->
+										convert_ascii_string((char *) h_ptr->
 												     v.h_vector);
 								}
 							} else {
-								str = convert_ascii_vec(t, hptr->v.h_vector);
+								str = convert_ascii_vec(t, h_ptr->v.h_vector);
 							}
 							if (str) {
 								int             slen;
@@ -1218,12 +1225,12 @@ pr_draw_hptr_trace(pr_context * prc, Trptr t, hptr h, int which, int dodraw, int
 static void 
 pr_draw_hptr_trace_vector_analog(pr_context * prc, Trptr t, hptr h, int which, int num_extension)
 {
-	TimeType        x0, x1, newtime, width;
+	TimeType        x0, x1, newtime;
 	int             y0, y1, yu, liney, ytext, yt0, yt1;
 	TimeType        tim, h2tim;
 	hptr            h2, h3;
         int             endcnt = 0;
-	int             pixlen, ysiz;
+	int             ysiz;
 	int             type;
 	int             lasttype = -1;
 	double          mynan = strtod("NaN", NULL);
@@ -1261,7 +1268,7 @@ pr_draw_hptr_trace_vector_analog(pr_context * prc, Trptr t, hptr h, int which, i
 				if(!(h3->flags&HIST_STRING) && h3->v.h_vector)
 					tv = *(double *)h3->v.h_vector;
 		} else {
-			tv=convert_real_vec(t,h3->v.h_vector);
+			if(h3->time <= tims.last) tv=convert_real_vec(t,h3->v.h_vector);
 		}
 		if (!isnan(tv))	{
 			if (isnan(tmin) || tv < tmin)
@@ -1316,13 +1323,13 @@ pr_draw_hptr_trace_vector_analog(pr_context * prc, Trptr t, hptr h, int which, i
 			if(!(h->flags&HIST_STRING) && h->v.h_vector)
 				tv = *(double *)h->v.h_vector;
 		} else {
-			tv=convert_real_vec(t,h->v.h_vector);
+			if(h->time <= tims.last) tv=convert_real_vec(t,h->v.h_vector);
 		}
 		if(h2->flags&HIST_REAL) {
 			if(!(h2->flags&HIST_STRING) && h2->v.h_vector)
 				tv2 = *(double *)h2->v.h_vector;
 		} else {
-			tv2=convert_real_vec(t,h2->v.h_vector);
+			if(h2->time <= tims.last) tv2=convert_real_vec(t,h2->v.h_vector);
 		}
 		if(isnan(tv))
 			yt0 = yu;
@@ -1458,7 +1465,7 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 
 		/* draw trans */
 		type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.h_vector) : AN_0;
-		if (x0 > -1)
+		if (x0 > -1) {
 			if (use_roundcaps) {
 				if (type == AN_Z) {
 					if (lasttype != -1) {
@@ -1504,7 +1511,7 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 					     x0, y0,
 					     x0, y1);
 			}
-
+		}
 
 		if (x0 != x1) {
 			if (type == AN_Z) {
@@ -1597,11 +1604,11 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 static void 
 pr_draw_vptr_trace_analog(pr_context * prc, Trptr t, vptr v, int which, int num_extension)
 {
-	TimeType        x0, x1, newtime, width;
+	TimeType        x0, x1, newtime;
 	int             y0, y1, yu, liney, ytext, yt0, yt1;
 	TimeType        tim, h2tim;
 	vptr            h, h2, h3;
-	int             pixlen, ysiz;
+	int             ysiz;
 	int             type;
 	int             lasttype = -1;
 	double          mynan = strtod("NaN", NULL);
@@ -1810,7 +1817,7 @@ pr_draw_vptr_trace(pr_context * prc, Trptr t, vptr v, int which)
 
 		/* draw trans */
 		type = vtype2(t, h);
-		if (x0 > -1)
+		if (x0 > -1) {
 			if (use_roundcaps) {
 				if (type == 2) {
 					if (lasttype != -1) {
@@ -1856,7 +1863,7 @@ pr_draw_vptr_trace(pr_context * prc, Trptr t, vptr v, int which)
 					     x0, y0,
 					     x0, y1);
 			}
-
+		}
 
 		if (x0 != x1) {
 			if (type == 2) {
@@ -2184,3 +2191,12 @@ print_mif_image(FILE * wave, gdouble px, gdouble py)
 	prc.fullpage = ps_fullpage;
 	print_image(&prc);
 }
+
+/*
+ * $Id: print.c,v 1.2 2007/04/20 02:08:13 gtkwave Exp $
+ * $Log: print.c,v $
+ * Revision 1.2  2007/04/20 02:08:13  gtkwave
+ * initial release
+ *
+ */
+
