@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2005.
+ * Copyright (c) Tony Bybell 1999-2007.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,6 +7,13 @@
  * of the License, or (at your option) any later version.
  */
 
+
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -17,13 +24,15 @@
 #include <sys/types.h>
 #include "analyzer.h"
 #include "currenttime.h"
-#include "aet.h"  
+#include "symbol.h"  
 #include "vcd.h"
-#include "alloca.h"
+#include "wavealloca.h"
 #include "fgetdynamic.h"
 #include "debug.h"
 #include "main.h"
+#include "menu.h"
 #include "color.h"
+#include "vlist.h"
 #include "rc.h"
 
 #ifndef _MSC_VER
@@ -45,6 +54,15 @@ int possibly_use_rc_defaults = 1;		/* if not -d option, use white screen, etc on
 /*
  * functions that set the individual rc variables..
  */
+static int f_accel(char *str)
+{
+DEBUG(printf("f_accel(\"%s\")\n",str));
+
+if(strlen(str)) { set_wave_menu_accelerator(str); }
+
+return(0);
+}
+
 static int f_alt_hier_delimeter(char *str)
 {
 DEBUG(printf("f_alt_hier_delimeter(\"%s\")\n",str));
@@ -111,6 +129,13 @@ cursor_snap=(val<=0)?0:val;
 return(0);
 }
 
+static int f_disable_mouseover(char *str)
+{
+DEBUG(printf("f_disable_mouseover(\"%s\")\n",str));
+disable_mouseover=atoi_64(str)?1:0;
+return(0);
+}
+
 static int f_disable_tooltips(char *str)
 {
 DEBUG(printf("f_disable_tooltips(\"%s\")\n",str));
@@ -201,6 +226,20 @@ force_toolbars=atoi_64(str)?1:0;
 return(0);
 }
 
+static int f_hide_sst(char *str)
+{
+DEBUG(printf("f_hide_sst(\"%s\")\n",str));
+hide_sst=atoi_64(str)?1:0;
+return(0);
+}
+
+static int f_sst_expanded(char *str)
+{
+DEBUG(printf("f_sst_expanded(\"%s\")\n",str));
+sst_expanded=atoi_64(str)?1:0;
+return(0);
+}
+
 static int f_hier_delimeter(char *str)
 {
 DEBUG(printf("f_hier_delimeter(\"%s\")\n",str));
@@ -352,6 +391,13 @@ use_full_precision=atoi_64(str)?1:0;
 return(0);
 }
 
+static int f_use_frequency_display(char *str)
+{
+DEBUG(printf("f_use_frequency_display(\"%s\")\n",str));
+use_frequency_delta=atoi_64(str)?1:0;
+return(0);
+}
+
 static int f_use_maxtime_display(char *str)
 {
 DEBUG(printf("f_use_maxtime_display(\"%s\")\n",str));
@@ -410,6 +456,15 @@ else if(vector_padding>16) vector_padding=16;
 return(0);
 }
 
+static int f_vlist_compression(char *str)
+{
+DEBUG(printf("f_vlist_compression(\"%s\")\n",str));
+vlist_compression_depth=atoi_64(str);
+if(vlist_compression_depth<0) vlist_compression_depth = -1;
+if(vlist_compression_depth>9) vlist_compression_depth = 9;
+return(0);
+}
+
 static int f_wave_scrolling(char *str)
 {
 DEBUG(printf("f_wave_scrolling(\"%s\")\n",str));
@@ -483,6 +538,13 @@ color_make(w)
 color_make(wfill)
 color_make(dash)
 color_make(dashfill)
+color_make(white)
+color_make(black)
+color_make(ltgray)
+color_make(normal)
+color_make(mdgray)
+color_make(dkgray)
+color_make(dkblue)
 
 
 /*
@@ -490,6 +552,7 @@ color_make(dashfill)
  */ 
 static struct rc_entry rcitems[]=
 {
+{ "accel", f_accel },
 { "alt_hier_delimeter", f_alt_hier_delimeter },
 { "append_vcd_hier", f_append_vcd_hier },
 { "atomic_vectors", f_atomic_vectors },
@@ -500,41 +563,50 @@ static struct rc_entry rcitems[]=
 { "color_1", f_color_1 },
 { "color_back", f_color_back },
 { "color_baseline", f_color_baseline },
+{ "color_black", f_color_black },
 { "color_dash", f_color_dash },
 { "color_dashfill", f_color_dashfill },
+{ "color_dkblue", f_color_dkblue },
+{ "color_dkgray", f_color_dkgray },
 { "color_grid", f_color_grid },
 { "color_high", f_color_high },
 { "color_low", f_color_low },
+{ "color_ltgray", f_color_ltgray },
 { "color_mark", f_color_mark },
+{ "color_mdgray", f_color_mdgray },
 { "color_mid", f_color_mid },
+{ "color_normal", f_color_normal },
 { "color_time", f_color_time },
 { "color_timeb", f_color_timeb },
 { "color_trans", f_color_trans },
+{ "color_u", f_color_u },
+{ "color_ufill", f_color_ufill },
 { "color_umark", f_color_umark },
 { "color_value", f_color_value },
 { "color_vbox", f_color_vbox },
 { "color_vtrans", f_color_vtrans },
-{ "color_x", f_color_x },
-{ "color_xfill", f_color_xfill },
-{ "color_u", f_color_u },
-{ "color_ufill", f_color_ufill },
 { "color_w", f_color_w },
 { "color_wfill", f_color_wfill },
+{ "color_white", f_color_white },
+{ "color_x", f_color_x },
+{ "color_xfill", f_color_xfill },
 { "constant_marker_update", f_constant_marker_update },
 { "convert_to_reals", f_convert_to_reals },
 { "cursor_snap", f_cursor_snap },
+{ "disable_mouseover", f_disable_mouseover },
 { "disable_tooltips", f_disable_tooltips },
 { "do_initial_zoom_fit", f_do_initial_zoom_fit },
 { "dynamic_resizing", f_dynamic_resizing },
 { "enable_fast_exit", f_enable_fast_exit },
 { "enable_ghost_marker", f_enable_ghost_marker },
-{ "enable_horiz_grid", f_enable_horiz_grid}, 
+{ "enable_horiz_grid", f_enable_horiz_grid }, 
 { "enable_vcd_autosave", f_enable_vcd_autosave },
-{ "enable_vert_grid", f_enable_vert_grid}, 
-{ "fontname_logfile", f_fontname_logfile}, 
-{ "fontname_signals", f_fontname_signals}, 
-{ "fontname_waves", f_fontname_waves}, 
-{ "force_toolbars", f_force_toolbars}, 
+{ "enable_vert_grid", f_enable_vert_grid }, 
+{ "fontname_logfile", f_fontname_logfile }, 
+{ "fontname_signals", f_fontname_signals }, 
+{ "fontname_waves", f_fontname_waves }, 
+{ "force_toolbars", f_force_toolbars }, 
+{ "hide_sst", f_hide_sst },
 { "hier_delimeter", f_hier_delimeter },
 { "hier_grouping", f_hier_grouping },
 { "hier_max_level", f_hier_max_level },
@@ -550,7 +622,9 @@ static struct rc_entry rcitems[]=
 { "show_base_symbols", f_show_base_symbols },
 { "show_grid", f_show_grid },
 { "splash_disable", f_splash_disable },
+{ "sst_expanded", f_sst_expanded },
 { "use_big_fonts", f_use_big_fonts },
+{ "use_frequency_display", f_use_frequency_display },
 { "use_full_precision", f_use_full_precision },
 { "use_maxtime_display", f_use_maxtime_display },
 { "use_nonprop_fonts", f_use_nonprop_fonts },
@@ -560,6 +634,7 @@ static struct rc_entry rcitems[]=
 { "vcd_preserve_glitches", f_vcd_preserve_glitches },
 { "vcd_warning_filesize", f_vcd_warning_filesize },
 { "vector_padding", f_vector_padding },
+{ "vlist_compression", f_vlist_compression },
 { "wave_scrolling", f_wave_scrolling },
 { "zoom_base", f_zoom_base },
 { "zoom_center", f_zoom_center },
@@ -573,6 +648,7 @@ f_splash_disable 	("off");
 f_zoom_pow10_snap	("on");
 f_hier_max_level	("1");
 f_cursor_snap		("8");
+f_use_frequency_display ("off");
 f_use_maxtime_display	("off");
 f_use_roundcaps		("on");
 f_use_nonprop_fonts	("on");
@@ -602,12 +678,30 @@ f_color_umark		("ff8080");
 f_color_mark		("ffff80");
 f_color_time		("ffffff");
 f_color_timeb		("000000");
+f_color_white		("ffffff");
+f_color_black		("000000");
+f_color_ltgray		("f5f5f5");
+f_color_normal		("e6e6e6");
+f_color_mdgray		("eeeeee");
+f_color_dkgray		("cccccc");
+f_color_dkblue		("4464ac");
 }
 
 
 void read_rc_file(char *override_rc)
 {
 FILE *handle;
+int i;
+int num_rcitems = sizeof(rcitems)/sizeof(struct rc_entry);
+
+for(i=0;i<(num_rcitems-1);i++)
+	{
+	if(strcmp(rcitems[i].name, rcitems[i+1].name) > 0)
+		{
+		fprintf(stderr, "rcitems misordering: '%s' vs '%s'\n", rcitems[i].name, rcitems[i+1].name);
+		exit(255);
+		}
+	}
 
 /* move defaults first and only go whitescreen if instructed to do so */
 if(possibly_use_rc_defaults) vanilla_rc();
@@ -624,7 +718,7 @@ if(!(handle=fopen(rcname,"rb")))
 	char *rcpath;
 	
 	home=getpwuid(geteuid())->pw_dir;
-	rcpath=alloca(strlen(home)+1+strlen(rcname)+1);
+	rcpath=(char *)alloca(strlen(home)+1+strlen(rcname)+1);
 	strcpy(rcpath,home);
 	strcat(rcpath,"/");
 	strcat(rcpath,rcname);
@@ -709,3 +803,12 @@ fclose(handle);
 errno=0;
 return;
 }
+
+/*
+ * $Id: rc.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: rc.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
