@@ -7,7 +7,26 @@
  * of the License, or (at your option) any later version.
  */
 
-#ifdef __linux__
+#include <config.h>
+
+#ifdef _AIX
+#ifndef _GNUC_
+#include <sys/types.h>
+#include <stdlib.h>
+#define REGEX_MALLOC
+#ifndef STDC_HEADERS
+#define STDC_HEADERS
+#endif
+#endif
+#endif
+
+#ifdef __CYGWIN__
+#include <sys/types.h>
+#include <stdlib.h>
+#define REGEX_MAY_COMPILE
+#include "gnu_regex.c"
+#else
+#if defined __linux__
 #include <sys/types.h>
 #include <stdlib.h>
 #include <regex.h>
@@ -25,30 +44,43 @@
 #define REGEX_MAY_COMPILE
 #include "gnu_regex.c"
 #endif
+#endif
 
-static regex_t preg;
-static int regex_ok=0;
+#include "regex_wave.h"
+#include "debug.h"
+
+static regex_t preg[WAVE_REGEX_TOTAL];
+static int regex_ok[WAVE_REGEX_TOTAL]; /* automatically zeroed as static */
 
 /*
  * compile a regular expression into a regex_t and
  * dealloc any previously valid ones
  */
-int wave_regex_compile(char *regex)
+int wave_regex_compile(char *regex, int which)
 {
 int comp_rc;
 
-if(regex_ok) { regfree(&preg); } /* free previous regex_t ancillary data if valid */
-comp_rc=regcomp(&preg, regex, REG_ICASE|REG_NOSUB);
-return(regex_ok=(comp_rc)?0:1);
+if(regex_ok[which]) { regfree(&preg[which]); } /* free previous regex_t ancillary data if valid */
+comp_rc=regcomp(&preg[which], regex, REG_ICASE|REG_NOSUB);
+return(regex_ok[which]=(comp_rc)?0:1);
 }
 
 
 /*
  * do match
  */
-int wave_regex_match(char *str)
+int wave_regex_match(char *str, int which)
 {
-int rc=regexec(&preg, str, 0, NULL, 0);
+int rc;
+
+if(regex_ok[which])
+	{
+	rc = regexec(&preg[which], str, 0, NULL, 0);
+	}
+	else
+	{
+	rc = 1; /* fail on malformed regex */
+	}
 
 return((rc)?0:1);
 }
@@ -60,12 +92,12 @@ return((rc)?0:1);
  */
 void *wave_regex_alloc_compile(char *regex)
 {
-regex_t *mreg = (regex_t *)malloc(sizeof(regex_t));
+regex_t *mreg = (regex_t *)malloc_2(sizeof(regex_t));
 int comp_rc=regcomp(mreg, regex, REG_ICASE|REG_NOSUB);
 
 if(comp_rc)
 	{
-	free(mreg);
+	free_2(mreg);
 	mreg=NULL;
 	}
 
@@ -94,7 +126,15 @@ regex_t *mreg = (regex_t *)pnt;
 if(mreg)
 	{
 	regfree(mreg);
-	free(mreg);
+	free_2(mreg);
 	}
 }
 
+/*
+ * $Id: regex.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: regex.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
