@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -840,3 +841,12 @@ if((match=(char **)bsearch((void *)str, (void *)cnames, C_ARRAY_SIZE, sizeof(cha
 	return(rc&0x00ffffff);
 	}
 }
+
+/*
+ * $Id: rgb.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: rgb.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
