@@ -7,14 +7,22 @@
  * of the License, or (at your option) any later version.
  */
 
+
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include <gtk/gtk.h>
 #include "gtk12compat.h"
 #include "analyzer.h"
-#include "aet.h"
+#include "symbol.h"
 #include "vcd.h"
 #include "lx2.h"
 #include "ghw.h"
 #include "debug.h"
+#include "busy.h"
 
 static GtkWidget *window1;
 static GtkWidget *entry_a;
@@ -68,7 +76,7 @@ static void entrybox_local(char *title, int width, char *default_text, int maxch
     cleanup_e=func;
 
     /* create a new modal window */
-    window1 = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    window1 = gtk_window_new(disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     gtk_grab_add(window1);
     gtk_widget_set_usize( GTK_WIDGET (window1), width, 60);
     gtk_window_set_title(GTK_WINDOW (window1), title);
@@ -218,6 +226,7 @@ gfloat interval;
 if(is_insert_running) return;
 is_insert_running = ~0;
 gtk_grab_add(widget);
+set_window_busy(widget);
 
 symc=NULL;
 
@@ -339,6 +348,8 @@ wavearea_configure_event(wavearea, NULL);
 
 gtk_progress_set_value (GTK_PROGRESS (pdata->pbar), 0.0);
 pdata->oldvalue = -1.0;
+
+set_window_idle(widget);
 gtk_grab_remove(widget);
 is_insert_running=0;
 }
@@ -355,7 +366,7 @@ gfloat interval;
 if(is_replace_running) return;
 is_replace_running = ~0;
 gtk_grab_add(widget);
-
+set_window_busy(widget);
 
 tfirst=NULL; tlast=NULL;
 symc=NULL;
@@ -488,6 +499,8 @@ wavearea_configure_event(wavearea, NULL);
 
 gtk_progress_set_value (GTK_PROGRESS (pdata->pbar), 0.0);
 pdata->oldvalue = -1.0;  
+
+set_window_idle(widget);
 gtk_grab_remove(widget);
 is_replace_running=0;
 }
@@ -502,6 +515,7 @@ gfloat interval;
 if(is_append_running) return;
 is_append_running = ~0;
 gtk_grab_add(widget);
+set_window_busy(widget);
 
 symc=NULL;
 
@@ -607,6 +621,8 @@ wavearea_configure_event(wavearea, NULL);
 
 gtk_progress_set_value (GTK_PROGRESS (pdata->pbar), 0.0);
 pdata->oldvalue = -1.0; 
+
+set_window_idle(widget);
 gtk_grab_remove(widget);   
 is_append_running=0;
 }
@@ -663,7 +679,7 @@ entry_suffixed=wave_alloca(strlen(entry_text)+strlen(regex_type[regex_which])+1+
 if(regex_which<2) strcpy(entry_suffixed, "\\<");	/* match on word boundary */
 strcat(entry_suffixed,entry_text);
 strcat(entry_suffixed,regex_type[regex_which]);
-wave_regex_compile(entry_suffixed);
+wave_regex_compile(entry_suffixed, WAVE_REGEX_SEARCH);
 for(i=0;i<numfacs;i++)
 	{
 	facs[i]->selected=0;
@@ -683,7 +699,7 @@ for(i=0;i<numfacs;i++)
 		}
 	pdata->oldvalue = i;
 
-	if(wave_regex_match(facs[i]->name))
+	if(wave_regex_match(facs[i]->name, WAVE_REGEX_SEARCH))
 	if((!is_ghw)||(strcmp(WAVE_GHW_DUMMYFACNAME, facs[i]->name)))
 		{
 		if(!facs[i]->vec_root)
@@ -789,7 +805,7 @@ void searchbox(char *title, GtkSignalFunc func)
     num_rows=selected_rows=0;
 
     /* create a new modal window */
-    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    window = gtk_window_new(disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW (window), title);
     gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                        (GtkSignalFunc) destroy_callback, NULL);
@@ -1044,3 +1060,12 @@ void searchbox(char *title, GtkSignalFunc func)
 
     if(strlen(searchbox_text)) enter_callback(entry,NULL);
 }
+
+/*
+ * $Id: search.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: search.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
