@@ -7,12 +7,14 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include "gtk12compat.h"
 #include <gtk/gtk.h>
+#include "menu.h"
 #include "debug.h"
 #include "pixmaps.h"
 
-static GtkWidget *window;
+static GtkWidget *window = NULL;
 static void (*cleanup)(GtkWidget *, gpointer);
 
 static void ok_callback(GtkWidget *widget, GtkWidget *nothing)
@@ -20,6 +22,7 @@ static void ok_callback(GtkWidget *widget, GtkWidget *nothing)
   DEBUG(printf("OK\n"));
   gtk_grab_remove(window);
   gtk_widget_destroy(window);
+  window = NULL;
   if(cleanup)cleanup(NULL,(gpointer)1);
 }
 
@@ -28,6 +31,7 @@ static void destroy_callback(GtkWidget *widget, GtkWidget *nothing)
   DEBUG(printf("Cancel\n"));
   gtk_grab_remove(window);
   gtk_widget_destroy(window);
+  window = NULL;
   if(cleanup)cleanup(NULL,NULL);
 }
 
@@ -39,10 +43,18 @@ void simplereqbox(char *title, int width, char *default_text,
     GtkWidget *label, *separator;
     GtkWidget *pixmapwid1;
 
+    if(window) return; /* only should happen with GtkPlug */
+
     cleanup=WAVE_GTK_SFUNCAST(func);
 
+    if(script_handle)
+	{
+	if(cleanup)cleanup(NULL,(gpointer)1);
+	return;
+	}
+
     /* create a new modal window */
-    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    window = gtk_window_new(disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     gtk_grab_add(window);
     gtk_widget_set_usize( GTK_WIDGET (window), width, 55 + 52);
     gtk_window_set_title(GTK_WINDOW (window), title);
@@ -104,3 +116,12 @@ void simplereqbox(char *title, int width, char *default_text,
 
     gtk_widget_show(window);
 }
+
+/*
+ * $Id: simplereq.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: simplereq.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
