@@ -7,8 +7,9 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <gtk/gtk.h>
-#include "aet.h"
+#include "symbol.h"
 #include "lxt2_read.h"
 #include "lx2.h"
    
@@ -20,11 +21,21 @@ a hierarchy first */
 static GtkWidget *text=NULL;
 static GtkWidget *vscrollbar;
 
+#if defined(WAVE_USE_GTK2) && !defined(GTK_ENABLE_BROKEN)
+static GtkTextIter iter;
+static GtkTextTag *bold_tag;
+#endif
+
+
 void status_text(char *str)
 {
 if(text)
 	{
-	gtk_text_insert (GTK_TEXT (text), NULL, &text->style->black, NULL, str, -1);
+#if defined(WAVE_USE_GTK2) && !defined(GTK_ENABLE_BROKEN)
+        gtk_text_buffer_insert (GTK_TEXT_VIEW (text)->buffer, &iter, str, -1);
+#else
+        gtk_text_insert (GTK_TEXT (text), NULL, &text->style->black, NULL, str, -1);
+#endif
 	}
 	else
 	{
@@ -41,7 +52,14 @@ char buf[128];
 
 if(is_vcd)
 	{
-	status_text("VCD loaded successfully.\n");
+	if(partial_vcd)
+		{
+		status_text("VCD loading interactively.\n");
+		}
+		else
+		{
+		status_text("VCD loaded successfully.\n");
+		}
 	}
 else
 if(is_lxt)
@@ -61,6 +79,7 @@ if(is_lx2)
 		case LXT2_IS_LXT2: status_text("LXT2 loaded successfully.\n"); break;
 		case LXT2_IS_AET2: status_text("AET2 loaded successfully.\n"); break;
 		case LXT2_IS_VZT:  status_text("VZT loaded successfully.\n"); break;
+		case LXT2_IS_VLIST:  status_text("VCD loaded successfully.\n"); break;
 		}
 	}
 
@@ -69,12 +88,22 @@ status_text(buf);
 
 if((is_vcd)||(is_ghw))
 	{
-	sprintf(buf,"[%d] regions found.\n",regions);
-	status_text(buf);
+	if(!partial_vcd)
+		{
+		sprintf(buf,"[%d] regions found.\n",regions);
+		status_text(buf);
+		}
 	}
 else
 	{
-	sprintf(buf,"Regions loaded on demand.\n");
+	if(is_lx2 == LXT2_IS_VLIST)
+		{
+		sprintf(buf,"Regions formed on demand.\n");
+		}
+		else
+		{
+		sprintf(buf,"Regions loaded on demand.\n");
+		}
 	status_text(buf);
 	}
 }
@@ -95,7 +124,14 @@ table = gtk_table_new (1, 16, FALSE);
    
 /* Put a text widget in the upper left hand corner. Note the use of
 * GTK_SHRINK in the y direction */
+#if defined(WAVE_USE_GTK2) && !defined(GTK_ENABLE_BROKEN)
+text = gtk_text_view_new ();
+gtk_text_buffer_get_start_iter (gtk_text_view_get_buffer(GTK_TEXT_VIEW (text)), &iter);
+bold_tag = gtk_text_buffer_create_tag (GTK_TEXT_VIEW (text)->buffer, "bold",
+                                      "weight", PANGO_WEIGHT_BOLD, NULL);
+#else
 text = gtk_text_new (NULL, NULL);
+#endif
 gtk_table_attach (GTK_TABLE (table), text, 0, 14, 0, 1,
 		      	GTK_FILL | GTK_EXPAND,
 		      	GTK_FILL | GTK_SHRINK | GTK_EXPAND, 0, 0);
@@ -103,7 +139,16 @@ gtk_widget_set_usize(GTK_WIDGET(text), 100, 50);
 gtk_widget_show (text);
 
 /* And a VScrollbar in the upper right */
+#if defined(WAVE_USE_GTK2) && !defined(GTK_ENABLE_BROKEN)
+{
+GtkTextViewClass *tc = (GtkTextViewClass*)GTK_OBJECT_GET_CLASS(GTK_OBJECT(text));
+ 
+tc->set_scroll_adjustments(GTK_TEXT_VIEW (text), NULL, NULL);
+vscrollbar = gtk_vscrollbar_new (GTK_TEXT_VIEW (text)->vadjustment);  
+}
+#else 
 vscrollbar = gtk_vscrollbar_new ((GTK_TEXT (text))->vadj);
+#endif
 gtk_table_attach (GTK_TABLE (table), vscrollbar, 15, 16, 0, 1,
 			GTK_FILL, GTK_FILL | GTK_SHRINK | GTK_EXPAND, 0, 0);
 gtk_widget_show (vscrollbar);
@@ -116,3 +161,11 @@ gtk_tooltips_set_tip_2(tooltips, text, "Status Window", NULL);
 return(table);
 }
    
+/*
+ * $Id: status.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: status.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
