@@ -1,12 +1,13 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2004.
+ * Copyright (c) Tony Bybell 1999-2007.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  */
-
+#include <config.h>
+#include <ctype.h>
 #include "gtk12compat.h"
 #include "strace.h"
 #include "currenttime.h"
@@ -27,7 +28,7 @@ static GtkSignalFunc cleanup;
 static char *logical[]=
 	{"AND", "OR", "XOR", "NAND", "NOR", "XNOR"};
 
-static char *stype[]=
+static char *stype[WAVE_STYPE_COUNT]=
 	{"Don't Care", "High", "Z (Mid)", "X", "Low", "String",
 	 "Rising Edge", "Falling Edge", "Any Edge"};
 
@@ -109,6 +110,30 @@ static struct item_mark_string item_mark_end_strings[]=
 	   { "Named Marker Z" }
 	};
 
+/*
+ * naive nonoptimized case insensitive strstr
+ */
+char *strstr_i(char *hay, char *needle)
+{
+char *ht, *nt;
+
+while (*hay)
+	{
+	int offs = 0;
+	ht = hay; nt = needle;
+	while(*nt)
+		{
+		if(toupper(*(ht+offs)) != toupper(*nt)) break;
+		offs++;
+		nt++;
+		}
+
+	if(!*nt) return(hay);
+	hay++;
+	}
+
+return(*hay ? hay : NULL);
+}
 
 /*
  * trap timescale overflows
@@ -139,7 +164,7 @@ s=straces;
 
 while(s)
 	{
-	for(i=0;i<9;i++)
+	for(i=0;i<WAVE_STYPE_COUNT;i++)
 		{
 		if(s->back[i]) free_2(s->back[i]);
 		}
@@ -323,7 +348,7 @@ void tracesearchbox(char *title, GtkSignalFunc func)
     numtraces=0;
 
     /* create a new window */
-    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    window = gtk_window_new(disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW (window), title);
     gtk_widget_set_usize( GTK_WIDGET (window), 420, -1); 
     gtk_signal_connect(GTK_OBJECT (window), "delete_event",
@@ -416,7 +441,7 @@ void tracesearchbox(char *title, GtkSignalFunc func)
     menu = gtk_menu_new ();
     group=NULL;
 
-    for(i=0;i<9;i++)
+    for(i=0;i<WAVE_STYPE_COUNT;i++)
 	{
 	struct strace_back *b;
 
@@ -754,9 +779,8 @@ while(s)
 				break;
 
 			case ST_RISE:
+				if((str[0]=='1')||(str[0]=='H')) s->search_result=1;	
 				totaltraces++;
-				if(((str[0]=='1')||(str[0]=='H'))&&(adjust(s->his.h->time,shift_timebase)==maxbase)) 
-					s->search_result=1;	
 				break;
 
 			case ST_LOW:
@@ -766,8 +790,7 @@ while(s)
 
 			case ST_FALL:
 				totaltraces++;
-				if(((str[0]=='0')||(str[0]=='L'))&&(adjust(s->his.h->time,shift_timebase)==maxbase))
- 					s->search_result=1;
+				if((str[0]=='0')||(str[0]=='L')) s->search_result=1;
 				break;
 
 			case ST_MID:
@@ -783,13 +806,13 @@ while(s)
 
 			case ST_ANY:
 				totaltraces++;
-				if(adjust(s->his.h->time,shift_timebase)==maxbase)s->search_result=1;
+				s->search_result=1;
 				break;
 		
 			case ST_STRING:
 				totaltraces++;
 				if(s->string)
-				if(strstr(s->string,str)) s->search_result=1;
+				if(strstr_i(s->string,str)) s->search_result=1;
 				break;
 
 			default:
@@ -916,14 +939,13 @@ while(s)
 
 			case ST_ANY:
 				totaltraces++;
-				if(adjust(s->his.v->time,shift_timebase)==maxbase)
-					s->search_result=1;
+				s->search_result=1;
 				break;
 		
 			case ST_STRING:
 				totaltraces++;
 				if(s->string)
-				if(strstr(chval, s->string)) s->search_result=1;
+				if(strstr_i(chval, s->string)) s->search_result=1;
 				break;
 
 			default:
@@ -1138,7 +1160,7 @@ while(s)
 			case ST_STRING:
 				totaltraces++;
 				if(s->string)
-				if(strstr(s->string,str)) s->search_result=1;
+				if(strstr_i(s->string,str)) s->search_result=1;
 				break;
 
 			default:
@@ -1272,7 +1294,7 @@ while(s)
 			case ST_STRING:
 				totaltraces++;
 				if(s->string)
-				if(strstr(chval, s->string)) s->search_result=1;
+				if(strstr_i(chval, s->string)) s->search_result=1;
 				break;
 
 			default:
@@ -1519,7 +1541,7 @@ int mprintf(const char *fmt, ... )
 int len;
 int rc;
 va_list args;
-struct mprintf_buff_t *bt = (struct mprintf_buff_t *)calloc(1, sizeof(struct mprintf_buff_t));
+struct mprintf_buff_t *bt = (struct mprintf_buff_t *)calloc_2(1, sizeof(struct mprintf_buff_t));
 char buff[65537];
                                 
 va_start(args, fmt);
@@ -1537,7 +1559,8 @@ if(!mprintf_buff_current)
         mprintf_buff_current->next = bt;
         mprintf_buff_current = bt;
         }
-                        
+
+va_end(args);                        
 return(rc);
 }
 
@@ -1555,7 +1578,7 @@ if(mprintf_buff_head)
 		{
 		free_2(mb->str);
 		mbt = mb->next;
-		free(mb);
+		free_2(mb);
 		mb = mbt;
 		}
 
@@ -1661,3 +1684,12 @@ if(timearray)
 	mprintf("!!\n");	/* mark end of strace region */
 	}
 }
+
+/*
+ * $Id: strace.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: strace.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
