@@ -23,7 +23,7 @@ enum strace_directions
 
 enum st_stype
         {ST_DC, ST_HIGH, ST_MID, ST_X, ST_LOW, ST_STRING,
-         ST_RISE, ST_FALL, ST_ANY};
+         ST_RISE, ST_FALL, ST_ANY, WAVE_STYPE_COUNT};
 
 struct strace_defer_free
 {
@@ -51,7 +51,7 @@ union
         vptr    v;
       	} his;
 
-struct strace_back *back[9];    /* dealloc these too! */   
+struct strace_back *back[WAVE_STYPE_COUNT];    /* dealloc these too! */   
 };
 
 
@@ -92,3 +92,11 @@ void delete_mprintf(void);
 
 #endif
 
+/*
+ * $Id: strace.h,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: strace.h,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
