@@ -7,13 +7,30 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <gtk/gtk.h>
 #include "gtk12compat.h"
-#include "aet.h"
+#include "symbol.h"
 #include "debug.h"
 
 GtkWidget *from_entry=NULL, *to_entry=NULL;
 
+void update_endcap_times_for_partial_vcd(void)
+{
+char str[40];
+
+if(from_entry)
+	{
+	reformat_time(str, tims.first, time_dimension);
+	gtk_entry_set_text(GTK_ENTRY(from_entry),str);
+	}
+
+if(to_entry)
+	{
+	reformat_time(str, tims.last, time_dimension);
+	gtk_entry_set_text(GTK_ENTRY(to_entry),str);
+	}
+}
 
 void time_update(void)
 {
@@ -152,3 +169,11 @@ gtk_widget_show(box2);
 return(mainbox);
 }
    
+/*
+ * $Id: timeentry.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: timeentry.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
