@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2005.
+ * Copyright (c) Tony Bybell 2005-6.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,9 +7,10 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <gtk/gtk.h>
 #include "gtk12compat.h"
-#include "aet.h"
+#include "symbol.h"
 #include "translate.h"
 #include "debug.h"
 
@@ -327,7 +328,7 @@ if(num_file_filters == FILE_FILTER_MAX)
 	return;
 	}
 
-fileselbox("Select Filter File",&fcurr,GTK_SIGNAL_FUNC(add_filter_callback_2), GTK_SIGNAL_FUNC(NULL),"*.txt");
+fileselbox("Select Filter File",&fcurr,GTK_SIGNAL_FUNC(add_filter_callback_2), GTK_SIGNAL_FUNC(NULL),NULL, 0);
 }
 
 /*
@@ -355,7 +356,7 @@ void trans_searchbox(char *title)
     current_filter = 0;
 
     /* create a new modal window */
-    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    window = gtk_window_new(disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW (window), title);
     gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                        (GtkSignalFunc) destroy_callback, NULL);
@@ -503,3 +504,12 @@ if(num_file_filters < FILE_FILTER_MAX)
 		}
 	}
 }
+
+/*
+ * $Id: translate.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: translate.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
