@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2004.
+ * Copyright (c) Tony Bybell 1999-2006.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,11 +7,22 @@
  * of the License, or (at your option) any later version.
  */
 
+
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include "tree.h"
 #include "vcd.h"
 
 struct tree *treeroot=NULL;
 static char *module=NULL;
+static int module_len;
+
+static struct tree *terminals_tchain = NULL;	/* splice in tree children last */
+
 
 char hier_delimeter='.';		/* default is dot unless aet is 
 					   selected, then it's slash */
@@ -29,7 +40,7 @@ enum TreeBuildTypes { MAKETREE_FLATTEN, MAKETREE_LEAF, MAKETREE_NODE };
  */
 void init_tree(void)
 {
-treeroot=(struct tree *)calloc_2(1,sizeof(struct tree));
+/* treeroot=(struct tree *)calloc_2(1,sizeof(struct tree)); */
 module=(char *)malloc_2(longestname+1);
 }
 
@@ -39,7 +50,7 @@ module=(char *)malloc_2(longestname+1);
  * hierarchy name.  return ptr to next name if it exists
  * else NULL
  */
-static char *get_module_name(char *s)
+static const char *get_module_name(const char *s)
 {
 char ch;
 char *pnt;
@@ -48,119 +59,95 @@ pnt=module;
 
 for(;;)
 	{
-	ch=*s;
+	ch=*(s++);
 
-	if(ch==hier_delimeter) 
+	if((ch==hier_delimeter) || (ch == '|'))
 		{
 		*(pnt)=0;	
-		s++;
+		module_len = pnt - module;
 		return(s);		
 		}
-	if(!ch)
+
+	if(!(*(pnt++)=ch))
 		{
-		*(pnt)=0;
+		module_len = pnt - module;
 		return(NULL);	/* nothing left to extract */		
 		}
-
-	s++;
-	*(pnt++)=ch;
 	}
 }
 
 
 /*
- * build the tree.  to keep the logic simple, we have
- * dead leaves at the ends.
+ * adds back netnames
  */
-void build_tree_from_name(char *s, int which)
+int treegraft(struct tree *t)
 {
-struct tree *t;
+struct tree *tx = terminals_tchain;
+struct tree *t2;
+struct tree *par;
 
-if(s==NULL) return;
-if(!strlen(s)) return;
-
-t=treeroot;
-while(s)
+while(tx)
 	{
-	s=get_module_name(s);
+	t2 = tx->next;
 
-	if(!t->name)
+	par = tx->child;
+	tx->child = NULL;
+
+	if(par)
 		{
-		char *newname;
-		struct tree *t2;
-
-		newname=(char *)malloc_2(strlen(module)+1);
-		strcpy(newname,module);
-		t->name=newname;		
-		if(!s) t->which=which; else t->which=-1;
-
-		t2=(struct tree *)calloc_2(1,sizeof(struct tree));
-		t->child=t2;			
-		/* t2->parent=t; */
-		t=t2;
-
-		continue;
-		}
-
-	if(!strcmp(t->name, module)) 
-		{
-		t=t->child;
-		continue;
-		}
-		else
-		{	
-		struct tree *t2, *t3;
-		char *newname;
-
-		newname=(char *)malloc_2(strlen(module)+1);
-		strcpy(newname,module);
-		t2=(struct tree *)calloc_2(1,sizeof(struct tree));
-		memcpy(t2, t, sizeof(struct tree));
-
-		t->name=newname;
-		t->next=t2;
-		if(!s) t->which=which; else t->which=-1;
-
-		t3=(struct tree *)calloc_2(1,sizeof(struct tree));
-		t->child=t3;
-		/* t3->parent=t; */
-		t=t3;		
-
-		continue;
-		}
-	}
-}
-
-
-/*
- * remove dead leaves from the tree
- */
-int treeprune(struct tree *t)
-{
-while(t)
-	{
-	if(t->name)
-		{
-		if(t->child)
+		if(par->child)
 			{
-			if(!(treeprune(t->child)))
-				{
-				t->child=NULL; /* remove empty leaf */
-				}
+			tx->next = par->child;
+			par->child = tx;
+			}
+			else
+			{
+			par->child = tx;
+			tx->next = NULL;
 			}
 		}
 		else
 		{
-		free_2(t);		       /* free empty leaf */
-		return(0);
+		tx->next = t->next;
+		t->next = tx;
 		}
 
-	t=t->next;
+	tx = t2;
 	}
+
 return(1);
 }
 
 
+/*
+ * unswizzle extended names in tree
+ */ 
+void treenamefix_str(char *s)
+{
+while(*s)
+	{
+	if(*s==VCDNAM_ESCAPE) *s=hier_delimeter;
+	s++;
+	}
+}
+
+void treenamefix(struct tree *t)
+{
+struct tree *tnext;
+if(t->child) treenamefix(t->child);
+
+tnext = t->next;
+
+while(tnext)
+	{
+	if(tnext->name) treenamefix_str(tnext->name);
+	tnext=tnext->next;
+	}
+
+if(t->name) treenamefix_str(t->name);
+}
+
+
 /*
  * for debugging purposes only
  */
@@ -183,7 +170,7 @@ while(t)
 	if(t->which>=0) /* for when valid netnames like A.B.C, A.B.C.D exist (not legal excluding texsim) */
 			/* otherwise this would be an 'else' */
 		{
-		printf("%d) %s\n", t->which, s2);
+		printf("%3d) %s\n", t->which, s2);
 		}
 
 	free_2(s2);
@@ -265,56 +252,6 @@ return(sibling);
 }
 
 
-void maketree(GtkCTreeNode *subtree, struct tree *t)
-{
-GtkCTreeNode *sibling=NULL, *sibling_test;
-struct tree *t2;
-
-if(!hier_grouping)
-	{
-	t2=t;
-	while(t2)
-		{
-		sibling_test=maketree_nodes(subtree, t2, sibling, MAKETREE_FLATTEN);	
-		sibling=sibling_test?sibling_test:sibling;
-		t2=t2->next;
-		}
-	}
-	else
-	{
-	t2=t;
-	while(t2)
-		{
-		if(!t2->child)
-			{
-			sibling_test=maketree_nodes(subtree, t2, sibling, MAKETREE_LEAF);
-			if(sibling_test)
-				{
-				maketree(sibling=sibling_test, t2->child);
-				}
-			}
-	
-		t2=t2->next;
-		}
-
-	t2=t;
-	while(t2)
-		{
-		if(t2->child)
-			{
-			sibling_test=maketree_nodes(subtree, t2, sibling, MAKETREE_NODE);
-			if(sibling_test)
-				{
-				maketree(sibling=sibling_test, t2->child);
-				}
-			}
-	
-		t2=t2->next;
-		}
-	}
-}
-
-
 /*
  * return least significant member name of a hierarchy
  * (used for tree and hier vec_root search hits)
@@ -348,7 +285,7 @@ return(pnt?pnt:nam);
  * sort the hier tree..should be faster than
  * moving numfacs longer strings around
  */
-int fast_tree_sort = 1;
+int fast_tree_sort = 1;	/* XXX : resync code should fix any remaining problems */
 
 static int tree_qsort_cmp(const void *v1, const void *v2)
 {
@@ -427,7 +364,8 @@ while(t)
 	if(t->which>=0) /* for when valid netnames like A.B.C, A.B.C.D exist (not legal excluding texsim) */
 			/* otherwise this would be an 'else' */
 		{
-		facs2[facs2_pos--] = facs[t->which];
+		facs2[facs2_pos] = facs[t->which];
+		t->which = facs2_pos--;
 		}
 
 	t=t->next;
@@ -437,7 +375,7 @@ while(t)
 
 void order_facs_from_treesort(struct tree *t, void *v)
 {
-struct symbol ***f = (struct symbol ***)v; /* eliminate compiler warning in tree.h as aet.h refs tree.h */
+struct symbol ***f = (struct symbol ***)v; /* eliminate compiler warning in tree.h as symbol.h refs tree.h */
 
 facs2=(struct symbol **)malloc_2(numfacs*sizeof(struct symbol *));
 facs2_pos = numfacs-1;
@@ -454,3 +392,230 @@ free_2(*f);
 *f = facs2;
 facs2 = NULL;
 }
+
+
+void build_tree_from_name(const char *s, int which)
+{
+struct tree *t, *nt;
+struct tree *tchain;
+struct tree *prevt;
+
+if(s==NULL || !s[0]) return;
+
+t = treeroot;
+
+if(t)
+	{
+	prevt = NULL;
+	while(s)
+		{
+rs:		s=get_module_name(s);
+
+		if(t && !strcmp(t->name, module))
+			{
+			prevt = t;
+			t = t->child;
+			continue;
+			}
+
+		tchain = t;
+		if(s && t)
+			{
+		      	nt = t->next;
+		      	while(nt)
+				{
+				if(nt && !strcmp(nt->name, module))
+					{
+					prevt = nt;
+					t = nt->child;
+					goto rs;
+					}
+
+				nt = nt->next;
+				}
+			}
+
+		nt=(struct tree *)calloc_2(1,sizeof(struct tree)+module_len);
+		memcpy(nt->name, module, module_len);
+
+		if(s)
+			{
+			nt->which=-1;
+
+			if(prevt)				/* make first in chain */
+				{
+				nt->next = prevt->child;
+				prevt->child = nt;
+				}
+				else				/* make second in chain as it's toplevel */
+				{
+				nt->next = tchain->next;	
+				tchain->next = nt;
+				}
+			}
+			else
+			{
+			nt->child = prevt;			/* parent */
+			nt->which = which;
+			nt->next = terminals_tchain;
+			terminals_tchain = nt;
+			return;
+			}
+	
+		/* blindly clone fac from next part of hier on down */
+		t = nt;
+		while(s)
+			{
+			s=get_module_name(s);
+		
+			nt=(struct tree *)calloc_2(1,sizeof(struct tree)+module_len);
+			memcpy(nt->name, module, module_len);
+
+			if(s)
+				{
+				nt->which = -1;
+				t->child = nt;
+				t = nt;
+				}
+				else
+				{
+				nt->child = t;			/* parent */
+				nt->which = which;
+				nt->next = terminals_tchain;
+				terminals_tchain = nt;
+				}
+			}
+		}
+	}
+else	
+	{
+	/* blindly create first fac in the tree (only ever called once) */
+	while(s)
+		{
+		s=get_module_name(s);
+
+		nt=(struct tree *)calloc_2(1,sizeof(struct tree)+module_len);
+		memcpy(nt->name, module, module_len);
+
+		if(!s) nt->which=which; else nt->which=-1;
+
+		if(treeroot)
+			{
+			t->child = nt;
+			t = nt;
+			}
+			else
+			{
+			treeroot = t = nt;
+			}
+		}
+	
+	}
+
+}
+
+
+/* ######################## */
+/* ## compatibility code ## */
+/* ######################## */
+
+/*
+ * tree widgets differ between GTK2 and GTK1 so we need two different
+ * maketree() routines
+ */
+#if WAVE_USE_GTK2
+
+/*
+ * GTK2: build the tree.
+ */
+void maketree(GtkCTreeNode *subtree, struct tree *t)
+{
+GtkCTreeNode *sibling=NULL, *sibling_test;
+struct tree *t2;
+
+/* 
+ * TG reworked treesearch widget so there is no need to 
+ * process anything other than nodes.  Leaves are handled
+ * in the filtered list below the node expand/contract
+ * tree
+ */
+t2=t;
+while(t2)
+	{
+	if(t2->child)
+		{
+		sibling_test=maketree_nodes(subtree, t2, sibling, MAKETREE_NODE);
+		if(sibling_test)
+			{
+			maketree(sibling=sibling_test, t2->child);
+			}
+		}
+
+	t2=t2->next;
+	}
+}
+
+#else
+
+/*
+ * GTK1: build the tree.
+ */
+void maketree(GtkCTreeNode *subtree, struct tree *t)
+{
+GtkCTreeNode *sibling=NULL, *sibling_test;
+struct tree *t2;
+
+if(!hier_grouping)
+	{
+	t2=t;
+	while(t2)
+		{
+		sibling_test=maketree_nodes(subtree, t2, sibling, MAKETREE_FLATTEN);	
+		sibling=sibling_test?sibling_test:sibling;
+		t2=t2->next;
+		}
+	}
+	else
+	{
+	t2=t;
+	while(t2)
+		{
+		if(!t2->child)
+			{
+			sibling_test=maketree_nodes(subtree, t2, sibling, MAKETREE_LEAF);
+			if(sibling_test)
+				{
+				maketree(sibling=sibling_test, t2->child);
+				}
+			}
+	
+		t2=t2->next;
+		}
+
+	t2=t;
+	while(t2)
+		{
+		if(t2->child)
+			{
+			sibling_test=maketree_nodes(subtree, t2, sibling, MAKETREE_NODE);
+			if(sibling_test)
+				{
+				maketree(sibling=sibling_test, t2->child);
+				}
+			}
+	
+		t2=t2->next;
+		}
+	}
+}
+
+#endif
+
+/*
+ * $Id: tree.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: tree.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
