@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2005.
+ * Copyright (c) Tony Bybell 1999-2007.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,20 @@
  * coredump fix for bad VCD	04apr02ajb
  * min/maxid speedup            27feb03ajb
  * bugfix on min/maxid speedup  06jul03ajb
+ * escaped hier modification    20feb06ajb
+ * added real_parameter vartype 04aug06ajb
+ * added in/out port vartype    31jan07ajb
+ * use gperf for port vartypes  19feb07ajb
+ * MTI SV implicit-var fix      05apr07ajb
+ * MTI SV len=0 is real var     05apr07ajb
  */
+
+/* AIX may need this for alloca to work */ 
+#if defined _AIX
+  #pragma alloca
+#endif
+
+#include <config.h>
 #include "vcd.h"
 
 #undef VCD_BSEARCH_IS_PERFECT		/* bsearch is imperfect under linux, but OK under AIX */
@@ -58,11 +71,13 @@ static TimeType current_time=-1;
 static int num_glitches=0;
 static int num_glitch_regions=0;
 
-static char vcd_hier_delimeter[2]={0, 0};   /* fill in after rc reading code */
+char vcd_hier_delimeter[2]={0, 0};   /* fill in after rc reading code */
 
 static struct vcdsymbol *pv=NULL, *rootv=NULL;
 static char *vcdbuf=NULL, *vst=NULL, *vend=NULL;
 
+static int escaped_names_found = 0;
+
 /******************************************************************/
 
 void strcpy_vcdalt(char *too, char *from, char delim)
@@ -79,11 +94,32 @@ do
 	} while((*(too++)=ch));
 }
 
+
+int strcpy_delimfix(char *too, char *from)
+{
+char ch;
+int found = 0;
+
+do
+	{
+	ch=*(from++);
+	if(ch==hier_delimeter)
+		{
+		ch=VCDNAM_ESCAPE;
+		found = 1;
+		}
+	} while((*(too++)=ch));
+
+if(found) escaped_names_found = found;
+
+return(found);
+}
+
 /******************************************************************/
 
-static struct slist *slistroot=NULL, *slistcurr=NULL;
-static char *slisthier=NULL;
-static int slisthier_len=0;
+struct slist *slistroot=NULL, *slistcurr=NULL;
+char *slisthier=NULL;
+int slisthier_len=0;
 
 /******************************************************************/
 
@@ -94,7 +130,7 @@ enum Tokens   { T_VAR, T_END, T_SCOPE, T_UPSCOPE,
 		T_TIMESCALE, T_VERSION, T_VCDCLOSE,
 		T_EOF, T_STRING, T_UNKNOWN_KEY };
 
-char *tokens[]={ "var", "end", "scope", "upscope",
+static char *tokens[]={ "var", "end", "scope", "upscope",
 		 "comment", "date", "dumpall", "dumpoff", "dumpon",
 		 "dumpvars", "enddefinitions",
 		 "dumpports", "dumpportsoff", "dumpportson", "dumpportsall",
@@ -115,20 +151,6 @@ static struct vcdsymbol *vcdsymroot=NULL, *vcdsymcurr=NULL;
 static struct vcdsymbol **sorted=NULL;
 static struct vcdsymbol **indexed=NULL;
 
-enum VarTypes { V_EVENT, V_PARAMETER,
-		V_INTEGER, V_REAL, V_REG, V_SUPPLY0,
-		V_SUPPLY1, V_TIME, V_TRI, V_TRIAND, V_TRIOR,
-		V_TRIREG, V_TRI0, V_TRI1, V_WAND, V_WIRE, V_WOR, V_PORT,
-		V_END, V_LB, V_COLON, V_RB, V_STRING };
-
-char *vartypes[]={ "event", "parameter",
-		"integer", "real", "reg", "supply0",
-		"supply1", "time", "tri", "triand", "trior",
-		"trireg", "tri0", "tri1", "wand", "wire", "wor", "port",
-		"$end", "", "", "", ""};
-
-#define NUM_VTOKENS 19
-
 static int numsyms=0;
 
 /******************************************************************/
@@ -447,10 +469,10 @@ return(T_UNKNOWN_KEY);
 
 
 static int var_prevch=0;
-static int get_vartoken_patched(void)
+static int get_vartoken_patched(int match_kw)
 {
 int ch;
-int i, len=0;
+int len=0;
 
 if(!var_prevch)
 	{
@@ -488,12 +510,13 @@ for(yytext[len++]=ch;;yytext[len++]=ch)
 	}
 yytext[len]=0;	/* terminator */
 
-for(i=0;i<NUM_VTOKENS;i++)
+if(match_kw)
 	{
-	if(!strcmp(yytext,vartypes[i]))
+	int vt = vcd_keyword_code(yytext, len);
+	if(vt != V_STRING)
 		{
 		if(ch<0) { free_2(varsplit); varsplit=NULL; }
-		return(i);
+		return(vt);
 		}
 	}
 
@@ -502,14 +525,14 @@ if(ch<0) { free_2(varsplit); varsplit=NULL; }
 return(V_STRING);
 }
 
-static int get_vartoken(void)
+static int get_vartoken(int match_kw)
 {
 int ch;
-int i, len=0;
+int len=0;
 
 if(varsplit)
 	{
-	int rc=get_vartoken_patched();
+	int rc=get_vartoken_patched(match_kw);
 	if(rc!=V_END) return(rc);
 	var_prevch=0;
 	}
@@ -534,6 +557,11 @@ if(ch=='[') return(V_LB);
 if(ch==':') return(V_COLON);
 if(ch==']') return(V_RB);
 
+if(ch=='#')     /* for MTI System Verilog '$var reg 64 >w #implicit-var###VarElem:ram_di[0.0] [63:0] $end' style declarations */
+        {       /* debussy simply escapes until the space */
+        yytext[len++]= '\\';
+        }
+
 for(yytext[len++]=ch;;yytext[len++]=ch)
 	{
 	if(len==T_MAX_STR)
@@ -571,11 +599,12 @@ if((varsplit)&&(yytext[len-1]==']'))
 	varsplit=NULL;
 	}
 
-for(i=0;i<NUM_VTOKENS;i++)
+if(match_kw)
 	{
-	if(!strcmp(yytext,vartypes[i]))
+	int vt = vcd_keyword_code(yytext, len);
+	if(vt != V_STRING)
 		{
-		return(i);
+		return(vt);
 		}
 	}
 
@@ -633,7 +662,7 @@ for(;;)
 if(hdr) DEBUG(fprintf(stderr,"\n"));
 }
 
-static char *build_slisthier(void)
+char *build_slisthier(void)
 {
 struct slist *s;
 int len=0;
@@ -671,6 +700,7 @@ while(s)
 		}
 	s=s->next;
 	}
+
 return(slisthier);
 }
 
@@ -1090,7 +1120,7 @@ for(;;)
 				s=(struct slist *)calloc_2(1,sizeof(struct slist));
 				s->len=yylen;
 				s->str=(char *)malloc_2(yylen+1);
-				strcpy(s->str,yytext);
+				strcpy(s->str, yytext);
 
 				if(slistcurr)
 					{
@@ -1151,7 +1181,7 @@ for(;;)
 
 			var_prevch=0;
 			varsplit=NULL;
-			vtok=get_vartoken();
+			vtok=get_vartoken(1);
 			if(vtok>V_PORT) goto bail;
 
 			v=(struct vcdsymbol *)calloc_2(1,sizeof(struct vcdsymbol));
@@ -1160,7 +1190,7 @@ for(;;)
 
 			if(vtok==V_PORT)
 				{
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok==V_STRING)
 					{
 					v->size=atoi_64(yytext);
@@ -1169,11 +1199,11 @@ for(;;)
 					else 
 					if(vtok==V_LB)
 					{
-					vtok=get_vartoken();
+					vtok=get_vartoken(1);
 					if(vtok==V_END) goto err;
 					if(vtok!=V_STRING) goto err;
 					v->msi=atoi_64(yytext);
-					vtok=get_vartoken();
+					vtok=get_vartoken(0);
 					if(vtok==V_RB)
 						{
 						v->lsi=v->msi;
@@ -1182,10 +1212,10 @@ for(;;)
 						else
 						{
 						if(vtok!=V_COLON) goto err;
-						vtok=get_vartoken();
+						vtok=get_vartoken(0);
 						if(vtok!=V_STRING) goto err;
 						v->lsi=atoi_64(yytext);
-						vtok=get_vartoken();
+						vtok=get_vartoken(0);
 						if(vtok!=V_RB) goto err;
 
 						if(v->msi>v->lsi)
@@ -1209,7 +1239,7 @@ for(;;)
                                 if(v->nid < vcd_minid) vcd_minid = v->nid;
                                 if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				if(slisthier_len)
 					{
@@ -1222,7 +1252,16 @@ for(;;)
 						}
 						else
 						{
-						strcpy(v->name+slisthier_len+1,yytext);
+						if((strcpy_delimfix(v->name+slisthier_len+1,yytext)) && (yytext[0] != '\\'))
+							{
+							char *sd=(char *)malloc_2(slisthier_len+1+yylen+2);
+							strcpy(sd,slisthier);
+							strcpy(sd+slisthier_len,vcd_hier_delimeter);
+							sd[slisthier_len+1] = '\\';
+							strcpy(sd+slisthier_len+2,v->name+slisthier_len+1);
+							free_2(v->name);
+							v->name = sd;
+							}
 						}
 					}
 					else
@@ -1234,7 +1273,14 @@ for(;;)
 						}
 						else
 						{
-						strcpy(v->name,yytext);
+						if((strcpy_delimfix(v->name,yytext)) && (yytext[0] != '\\'))
+							{
+							char *sd=(char *)malloc_2(yylen+2);
+							sd[0] = '\\';
+							strcpy(sd+1,v->name);
+							free_2(v->name);
+							v->name = sd;
+							}
 						}
 					}
 
@@ -1259,10 +1305,9 @@ for(;;)
 				}
 				else	/* regular vcd var, not an evcd port var */
 				{
-				vtok=get_vartoken();
+				vtok=get_vartoken(1);
 				if(vtok==V_END) goto err;
 				v->size=atoi_64(yytext);
-				if(!v->size) v->size=1;
 				vtok=get_strtoken();
 				if(vtok==V_END) goto err;
 				v->id=(char *)malloc_2(yylen+1);
@@ -1272,7 +1317,7 @@ for(;;)
                                 if(v->nid < vcd_minid) vcd_minid = v->nid;
                                 if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				if(slisthier_len)
 					{
@@ -1285,7 +1330,16 @@ for(;;)
 						}
 						else
 						{
-						strcpy(v->name+slisthier_len+1,yytext);
+						if((strcpy_delimfix(v->name+slisthier_len+1,yytext)) && (yytext[0] != '\\'))
+							{
+                                                        char *sd=(char *)malloc_2(slisthier_len+1+yylen+2);
+                                                        strcpy(sd,slisthier);
+                                                        strcpy(sd+slisthier_len,vcd_hier_delimeter);
+                                                        sd[slisthier_len+1] = '\\';
+                                                        strcpy(sd+slisthier_len+2,v->name+slisthier_len+1);
+                                                        free_2(v->name);
+                                                        v->name = sd;
+							}
 						}
 					}
 					else
@@ -1297,7 +1351,14 @@ for(;;)
 						}
 						else
 						{
-						strcpy(v->name,yytext);
+                                                if((strcpy_delimfix(v->name,yytext)) && (yytext[0] != '\\'))
+                                                        {
+                                                        char *sd=(char *)malloc_2(yylen+2);
+                                                        sd[0] = '\\';
+                                                        strcpy(sd+1,v->name);
+                                                        free_2(v->name);
+                                                        v->name = sd;
+                                                        }
 						}
 					}
 
@@ -1320,27 +1381,29 @@ for(;;)
 					}
                                 pv=v;
 				
-				vtok=get_vartoken();
+				vtok=get_vartoken(1);
 				if(vtok==V_END) goto dumpv;
 				if(vtok!=V_LB) goto err;
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				v->msi=atoi_64(yytext);
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok==V_RB)
 					{
 					v->lsi=v->msi;
 					goto dumpv;
 					}
 				if(vtok!=V_COLON) goto err;
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_STRING) goto err;
 				v->lsi=atoi_64(yytext);
-				vtok=get_vartoken();
+				vtok=get_vartoken(0);
 				if(vtok!=V_RB) goto err;
 				}
 
 			dumpv:
+                        if(v->size == 0) { v->vartype = V_REAL; } /* MTI fix */
+
 			if((v->vartype==V_REAL)||((convert_to_reals)&&((v->vartype==V_INTEGER)||(v->vartype==V_PARAMETER))))
 				{
 				v->vartype=V_REAL;
@@ -1503,6 +1566,7 @@ for(;;)
 				if(v->id) free_2(v->id);
 				if(v->value) free_2(v->value);
 				free_2(v);
+				pv = NULL;
 				}
 
 			bail:
@@ -1560,7 +1624,7 @@ for(;;)
 			dumping_off=1;
 			if((!blackout_regions)||((blackout_regions)&&(blackout_regions->bstart<=blackout_regions->bend)))
 				{
-				struct blackout_region_t *bt = calloc(1, sizeof(struct blackout_region_t));
+				struct blackout_region_t *bt = calloc_2(1, sizeof(struct blackout_region_t));
 
 				bt->bstart = current_time;
 				bt->next = blackout_regions;
@@ -2150,7 +2214,7 @@ for(i=0;i<numfacs;i++)
         curnode=curnode->nextinaet;
         while((ch=(*subst)))
                 {
-                if(ch==hier_delimeter) { *subst=0x01; } /* forces sort at hier boundaries */
+                if(ch==hier_delimeter) { *subst=VCDNAM_HIERSORT; } /* forces sort at hier boundaries */
                 subst++;
                 }
         }
@@ -2165,7 +2229,7 @@ for(i=0;i<numfacs;i++)
         subst=facs[i]->name;
         while((ch=(*subst)))
                 {
-                if(ch==0x01) { *subst=hier_delimeter; } /* restore back to normal */
+                if(ch==VCDNAM_HIERSORT) { *subst=hier_delimeter; } /* restore back to normal */
                 subst++;
                 }
         
@@ -2180,8 +2244,25 @@ init_tree();
 for(i=0;i<numfacs;i++)
 {                       
 build_tree_from_name(facs[i]->name, i);
+
+if(escaped_names_found)
+	{
+        char *subst, ch;
+        subst=facs[i]->name;
+        while((ch=(*subst)))
+                {
+                if(ch==VCDNAM_ESCAPE) { *subst=hier_delimeter; } /* restore back to normal */
+                subst++;
+                }
+	}
 }                       
-treeprune(treeroot);
+treegraft(treeroot);
+treesort(treeroot, NULL);
+
+if(escaped_names_found) 
+	{
+	treenamefix(treeroot);
+	}
 }
 
 /*******************************************************************************/
@@ -2296,13 +2377,15 @@ if (((flen>2)&&(!strcmp(fname+flen-3,".gz")))||
 			fprintf(stderr, "Warning! File size is %d MB.  This might fail to load.\n"
 					"Consider converting it to lxt, lxt2, or vzt database formats instead.  (See\n"
 					"the vcd2lxt(1), vcd2lxt2(1), and vzt2vzt(1) manpages for more information.)\n"
-					"To disable this warning, set rc variable vcd_warning_filesize to zero.\n\n",
+					"To disable this warning, set rc variable vcd_warning_filesize to zero.\n"
+					"Alternatively, use the -o, --optimize command line option to convert to LXT2.\n\n",
 						(int)(vcd_fsiz/(1024*1024)));
 			}
 		}
 		else
 		{
 		vcd_handle=stdin;
+		splash_disable = 1;
 		}
 	vcd_is_compressed=0;
 	}
@@ -2340,6 +2423,12 @@ if(vcd_fsiz)
         splash_sync(vcd_fsiz, vcd_fsiz);
 	vcd_fsiz = 0;
         }
+else
+if(vcd_is_compressed)
+	{
+        splash_sync(1,1);
+	vcd_fsiz = 0;
+	}
 
 vcd_build_symbols();
 vcd_sortfacs();
@@ -2362,3 +2451,12 @@ return(max_time);
 }
 
 /*******************************************************************************/
+
+/*
+ * $Id: vcd.c,v 1.2 2007/04/20 02:08:17 gtkwave Exp $
+ * $Log: vcd.c,v $
+ * Revision 1.2  2007/04/20 02:08:17  gtkwave
+ * initial release
+ *
+ */
+
