@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2003-2005.
+ * Copyright (c) Tony Bybell 2003-2006.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include "vzt.h"
 #include "lx2.h"
@@ -18,13 +19,13 @@
 #include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
-#include "aet.h"
+#include "symbol.h"
 #include "vcd.h"
 #include "lxt.h"
 #include "lxt2_read.h"
 #include "vzt_read.h"
 #include "debug.h"
-
+#include "busy.h"
 
 /*
  * globals
@@ -46,6 +47,8 @@ struct Node *n;
 struct symbol *s, *prevsymroot=NULL, *prevsym=NULL;
 signed char scale;
 unsigned int numalias = 0;
+struct symbol *sym_block = NULL;
+struct Node *node_block = NULL;
 
 vzt = vzt_rd_init_smp(fname, num_cpus);
 if(!vzt)
@@ -64,6 +67,8 @@ exponent_to_time_scale(scale);
 numfacs=vzt_rd_get_num_facs(vzt);
 mvlfacs=(struct fac *)calloc_2(numfacs,sizeof(struct fac));
 vzt_table=(struct lx2_entry *)calloc_2(numfacs, sizeof(struct lx2_entry));
+sym_block = (struct symbol *)calloc_2(numfacs, sizeof(struct symbol));
+node_block=(struct Node *)calloc_2(numfacs,sizeof(struct Node));
 
 for(i=0;i<numfacs;i++)
 	{
@@ -137,8 +142,9 @@ for(i=0;i<numfacs;i++)
 
 	if((f->len>1)&& (!(f->flags&(VZT_RD_SYM_F_INTEGER|VZT_RD_SYM_F_DOUBLE|VZT_RD_SYM_F_STRING))) )
 		{
-		sprintf(buf, "%s[%d:%d]", mvlfacs[i].name,mvlfacs[i].msb, mvlfacs[i].lsb);
-		str=malloc_2(strlen(buf)+1);
+		int len=sprintf(buf, "%s[%d:%d]", mvlfacs[i].name,mvlfacs[i].msb, mvlfacs[i].lsb);
+		str=malloc_2(len+1);
+
 		if(!alt_hier_delimeter)
 			{
 			strcpy(str, buf);
@@ -147,7 +153,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+		s=&sym_block[i];
+	        symadd_name_exists_sym_exists(s,str,0);
 		prevsymroot = prevsym = NULL;
 		}
 	else if ( 
@@ -158,8 +165,8 @@ for(i=0;i<numfacs;i++)
 			(mvlfacs[i].msb!=-1)&&(mvlfacs[i].lsb!=-1))
 		)
 		{
-		sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
-		str=malloc_2(strlen(buf)+1);
+		int len = sprintf(buf, "%s[%d]", mvlfacs[i].name,mvlfacs[i].msb);
+		str=malloc_2(len+1);
 		if(!alt_hier_delimeter)
 			{
 			strcpy(str, buf);
@@ -168,7 +175,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+		s=&sym_block[i];
+	        symadd_name_exists_sym_exists(s,str,0);
 		if((prevsym)&&(i>0)&&(!strcmp(mvlfacs[i].name, mvlfacs[i-1].name)))	/* allow chaining for search functions.. */
 			{
 			prevsym->vec_root = prevsymroot;
@@ -192,7 +200,8 @@ for(i=0;i<numfacs;i++)
 			{
 			strcpy_vcdalt(str, mvlfacs[i].name, alt_hier_delimeter);
 			}
-	        s=symadd_name_exists(str,hash(str));
+		s=&sym_block[i];
+	        symadd_name_exists_sym_exists(s,str,0);
 		prevsymroot = prevsym = NULL;
 
 		if(f->flags&VZT_RD_SYM_F_INTEGER)
@@ -213,7 +222,7 @@ for(i=0;i<numfacs;i++)
                 curnode=s;   
                 }
 
-        n=(struct Node *)calloc_2(1,sizeof(struct Node));
+        n=&node_block[i];
         n->nname=s->name;
         n->mvlfac = mvlfacs+i;
 	mvlfacs[i].working_node = n;
@@ -304,7 +313,7 @@ if(fast_tree_sort)
                 build_tree_from_name(facs[i]->name, i);
                 }
 /* SPLASH */                            splash_sync(4, 5);  
-        treeprune(treeroot);
+        treegraft(treeroot);
                                 
         fprintf(stderr, VZT_RDLOAD"Sorting facility hierarchy tree.\n");
         treesort(treeroot, NULL);
@@ -326,7 +335,7 @@ if(fast_tree_sort)
 		curnode=curnode->nextinaet;
 		while((ch=(*subst)))
 			{	
-			if(ch==hier_delimeter) { *subst=0x01; }	/* forces sort at hier boundaries */
+			if(ch==hier_delimeter) { *subst=VCDNAM_HIERSORT; }	/* forces sort at hier boundaries */
 			subst++;
 			}
 		}
@@ -342,7 +351,7 @@ if(fast_tree_sort)
 		subst=facs[i]->name;
 		while((ch=(*subst)))
 			{	
-			if(ch==0x01) { *subst=hier_delimeter; }	/* restore back to normal */
+			if(ch==VCDNAM_HIERSORT) { *subst=hier_delimeter; }	/* restore back to normal */
 			subst++;
 			}
 		}
@@ -358,7 +367,8 @@ if(fast_tree_sort)
 		build_tree_from_name(facs[i]->name, i);
 		}
 /* SPLASH */                            splash_sync(5, 5);  
-	treeprune(treeroot);
+	treegraft(treeroot);
+	treesort(treeroot, NULL);
 	}
 
 min_time = first_cycle; max_time=last_cycle;
@@ -409,7 +419,16 @@ struct HistEnt *htemp = histent_calloc();
 struct lx2_entry *l2e = vzt_table+(*facidx);
 struct fac *f = mvlfacs+(*facidx);
 
-/* fprintf(stderr, "%Ld %d %s\n", *time, *facidx, *value); */
+static int busycnt = 0;
+
+busycnt++; 
+if(busycnt==WAVE_BUSY_ITER)
+	{
+	busy_window_refresh();
+	busycnt = 0;
+	}
+
+/* fprintf(stderr, "%lld %d %s\n", *time, *facidx, *value); */
 
 if(!(f->flags&(VZT_RD_SYM_F_DOUBLE|VZT_RD_SYM_F_STRING)))
 	{
@@ -628,8 +647,9 @@ if(cnt>100)
 	fprintf(stderr, VZT_RDLOAD"Extracting %d traces\n", cnt);
 	}
 
-
+set_window_busy(NULL);
 vzt_rd_iter_blocks(vzt, vzt_callback, NULL);
+set_window_idle(NULL);
 
 for(txidx=0;txidx<numfacs;txidx++)
 	{
@@ -701,3 +721,12 @@ for(txidx=0;txidx<numfacs;txidx++)
 		}
 	}
 }
+
+/*
+ * $Id: vzt.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: vzt.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
