@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2005.
+ * Copyright (c) Tony Bybell 1999-2007.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -7,16 +7,20 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include "gtk12compat.h"
 #include "currenttime.h"
 #include "pixmaps.h"
-#include "aet.h"
+#include "symbol.h"
 #include "bsearch.h"
 #include "color.h"
 #include "strace.h"
 #include "debug.h"
+#include "main.h"
 
+#if !defined _ISOC99_SOURCE
 #define _ISOC99_SOURCE 1
+#endif
 #include <math.h>
 
 static void rendertimebar(void);
@@ -97,11 +101,35 @@ static GdkGC    *gc_w     =NULL;
 static GdkGC    *gc_dashfill =NULL;
 static GdkGC    *gc_dash  =NULL;
 
+static char made_sgc_contexts=0;
+
+GdkGC	*gc_white =NULL;
+GdkGC	*gc_black =NULL;
+GdkGC	*gc_ltgray=NULL;
+GdkGC	*gc_normal=NULL;
+GdkGC	*gc_mdgray=NULL;
+GdkGC	*gc_dkgray=NULL;
+GdkGC	*gc_dkblue=NULL;
+
 static const GdkModifierType   bmask[4]= {0, GDK_BUTTON1_MASK, 0, GDK_BUTTON3_MASK };		        /* button 1, 3 press/rel encodings */
 static const GdkModifierType m_bmask[4]= {0, GDK_BUTTON1_MOTION_MASK, 0, GDK_BUTTON3_MOTION_MASK };	/* button 1, 3 motion encodings */
 
 /******************************************************************/
 
+static void update_dual()
+{
+if(dual_ctx && !dual_race_lock)
+        {
+        dual_ctx[dual_id].zoom = tims.zoom;
+        dual_ctx[dual_id].marker = tims.marker;
+        dual_ctx[dual_id].baseline = tims.baseline;
+        dual_ctx[dual_id].left_margin_time = tims.start;
+        dual_ctx[dual_id].use_new_times = 1;
+        }
+}
+
+/******************************************************************/
+
 /*
  * gtk_draw_line() acceleration for win32 by doing draw combining
  */
@@ -181,14 +209,44 @@ if(seg_trans)
 TimeType cook_markertime(TimeType marker, gint x, gint y)
 {
 int i, num_traces_displayable;                
-Trptr t;
+Trptr t = NULL;
 TimeType lft, rgh;
 char lftinv, rghinv;
 gdouble xlft, xrgh;
 gdouble xlftd, xrghd;
+TimeType closest_named = MAX_HISTENT_TIME;
+int closest_which = -1;
+gint xold = x, yold = y;
 
 if(!cursor_snap) return(marker);
 
+/* potential snapping to a named marker time */
+for(i=0;i<26;i++)
+	{
+	if(named_markers[i] != -1)
+		{
+		TimeType dlt;
+
+		if((named_markers[i]>=tims.start)&&(named_markers[i]<=tims.end)&&(named_markers[i]<=tims.last))
+			{
+			if(marker < named_markers[i])
+				{
+				dlt = named_markers[i] - marker;			
+				}
+				else
+				{		
+				dlt = marker - named_markers[i];			
+				}
+			
+			if(dlt < closest_named)
+				{
+				closest_named = dlt;
+				closest_which = i;
+				}
+			}
+		}
+	}
+
 num_traces_displayable=wavearea->allocation.height/(fontheight);
 num_traces_displayable--;   /* for the time trace that is always there */
 
@@ -200,11 +258,16 @@ if(y>num_traces_displayable) y=num_traces_displayable;
 t=topmost_trace;
 for(i=0;i<y;i++)
 	{
-	if(!t) return(marker);
+	if(!t) goto bot;
 	t=GiveNextTrace(t);
 	}
 
-if((!t)||(t->flags&(TR_BLANK|TR_EXCLUDE))) return(marker);
+if(!t) goto bot;
+if((t->flags&(TR_BLANK|TR_EXCLUDE)))
+	{
+	t = NULL;
+	goto bot;
+	}
 
 if(t->flags & TR_ANALOG_BLANK_STRETCH)	/* seek to real analog trace is present... */
 	{
@@ -223,14 +286,14 @@ if(t->flags & TR_ANALOG_BLANK_STRETCH)	/* seek to real analog trace is present..
 			}
 		}
 	}
-if(!t) return(marker);
+if(!t) goto bot;
 
 if(t->vector)
 	{
 	vptr v = bsearch_vector(t->n.vec, marker - t->shift);
 	vptr v2 = v ? v->next : NULL;
 
-	if((!v)||(!v2)) return(marker);	/* should never happen */
+	if((!v)||(!v2)) goto bot;	/* should never happen */
 
 	lft = v->time;
 	rgh = v2->time;
@@ -240,12 +303,13 @@ if(t->vector)
 	hptr h = bsearch_node(t->n.nd, marker - t->shift);
 	hptr h2 = h ? h->next : NULL;
 
-	if((!h)||(!h2)) return(marker);	/* should never happen */
+	if((!h)||(!h2)) goto bot;	/* should never happen */
 
 	lft = h->time;
 	rgh = h2->time;
 	}
 
+
 lftinv = (lft < (tims.start - t->shift))||(lft >= (tims.end - t->shift))||(lft >= (tims.last - t->shift));
 rghinv = (rgh < (tims.start - t->shift))||(rgh >= (tims.end - t->shift))||(rgh >= (tims.last - t->shift));
 
@@ -260,13 +324,50 @@ if(xrghd<(gdouble)0.0) xrghd = ((gdouble)0.0) - xrghd;
 
 if(xlftd<=xrghd)
 	{
-	if((!lftinv)&&(xlftd<=cursor_snap)) marker = lft + t->shift;
+	if((!lftinv)&&(xlftd<=cursor_snap)) 
+		{
+		if(closest_which >= 0)
+		        {
+        		if((closest_named * pxns) < xlftd)
+                		{
+                		marker = named_markers[closest_which];
+				goto xit;
+                		}
+			}
+
+		marker = lft + t->shift;
+		goto xit;
+		}
 	}
 	else
 	{
-	if((!rghinv)&&(xrghd<=cursor_snap)) marker = rgh + t->shift;
+	if((!rghinv)&&(xrghd<=cursor_snap)) 
+		{
+		if(closest_which >= 0)
+		        {
+        		if((closest_named * pxns) < xrghd)
+                		{
+                		marker = named_markers[closest_which];
+				goto xit;
+                		}
+			}
+
+		marker = rgh + t->shift;
+		goto xit;
+		}
 	}
 
+bot:
+if(closest_which >= 0)
+	{
+	if((closest_named * pxns) <= cursor_snap)
+		{
+		marker = named_markers[closest_which];
+		}
+	}
+
+xit:
+move_mouseover(t, xold, yold, marker);
 return(marker);
 }
 
@@ -398,6 +499,8 @@ if(m1x>m2x)		/* ensure m1x <= m2x for partitioned refresh */
 	}
 
 if(m1x==-1) m1x=m2x;	/* make both markers same if no ghost marker or v.v. */
+
+update_dual();
 }
 
 
@@ -557,7 +660,7 @@ if(signalpixmap)
 	DEBUG(printf("Wave VSlider Moved to %d\n",trtarget));
 
 		gdk_draw_rectangle(signalpixmap, 
-			signalarea->style->bg_gc[GTK_STATE_PRELIGHT], TRUE, 0, 0,
+			gc_ltgray, TRUE, 0, 0,
 	            	signal_fill_width, signalarea->allocation.height);
 	
 		sync_marker();
@@ -575,11 +678,11 @@ if(signalpixmap)
 	}
 }
 
-static void button_press_release_common(void)
+void button_press_release_common(void)
 {
 MaxSignalLength();
 gdk_draw_rectangle(signalpixmap, 
-	signalarea->style->bg_gc[GTK_STATE_PRELIGHT], TRUE, 0, 0,
+	gc_ltgray, TRUE, 0, 0,
         signal_fill_width, signalarea->allocation.height);
 sync_marker();
 RenderSigs((int)(GTK_ADJUSTMENT(wave_vslider)->value),0);
@@ -594,6 +697,7 @@ static void button_motion_common(gint xin, gint yin, int pressrel, int is_button
 {
 gdouble x,offset,pixstep;
 TimeType newcurr;
+
 if(xin<0) xin=0;
 if(xin>(wavewidth-1)) xin=(wavewidth-1);
 
@@ -652,6 +756,9 @@ int scrolled;
 gint xi, yi;
 #endif
 
+int dummy_x, dummy_y;
+get_window_xypos(&dummy_x, &dummy_y);
+
 if(event->is_hint)   
         {
 	WAVE_GDK_GET_POINTER(event->window, &x, &y, &xi, &yi, &state);
@@ -971,10 +1078,28 @@ if((event->button)&&(event->button==in_button_press))
 	update_markertime(time_trunc(tims.marker));
 	}
 
+move_mouseover(NULL, 0, 0, LLDescriptor(0));
 tims.timecache=0;
 return(TRUE);
 }
 
+
+void make_sigarea_gcs(GtkWidget *signalarea)
+{
+if(!made_sgc_contexts)
+	{
+	gc_white = alloc_color(signalarea, color_white, signalarea->style->white_gc);
+	gc_black = alloc_color(signalarea, color_black, signalarea->style->black_gc);
+	gc_ltgray= alloc_color(signalarea, color_ltgray, signalarea->style->bg_gc[GTK_STATE_PRELIGHT]);
+	gc_normal= alloc_color(signalarea, color_normal, signalarea->style->bg_gc[GTK_STATE_NORMAL]);
+	gc_mdgray= alloc_color(signalarea, color_mdgray, signalarea->style->bg_gc[GTK_STATE_INSENSITIVE]);
+	gc_dkgray= alloc_color(signalarea, color_dkgray, signalarea->style->bg_gc[GTK_STATE_ACTIVE]);
+	gc_dkblue= alloc_color(signalarea, color_dkblue, signalarea->style->bg_gc[GTK_STATE_SELECTED]);
+
+	made_sgc_contexts=~0;
+	}
+}
+
 gint wavearea_configure_event(GtkWidget *widget, GdkEventConfigure *event)
 {
 DEBUG(printf("WaveWin Configure Event h: %d, w: %d\n",widget->allocation.height,
@@ -1098,14 +1223,14 @@ GTK_WIDGET_SET_FLAGS( wavearea, GTK_CAN_FOCUS );
 
 gtk_table_attach (GTK_TABLE (table), wavearea, 0, 9, 0, 9,
                         GTK_FILL | GTK_EXPAND,
-                        GTK_FILL | GTK_EXPAND | GTK_SHRINK, 3, 3);
+                        GTK_FILL | GTK_EXPAND, 3, 2);
 
 wave_vslider=gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
 vadj=GTK_ADJUSTMENT(wave_vslider);
 gtk_signal_connect(GTK_OBJECT(wave_vslider), "value_changed",
                         GTK_SIGNAL_FUNC(service_vslider), NULL);
 vscroll=gtk_vscrollbar_new(vadj);
-GTK_WIDGET_SET_FLAGS(vscroll, GTK_CAN_FOCUS);
+/* GTK_WIDGET_SET_FLAGS(vscroll, GTK_CAN_FOCUS); */
 gtk_widget_show(vscroll);
 gtk_table_attach (GTK_TABLE (table), vscroll, 9, 10, 0, 9,
                         GTK_FILL,
@@ -1116,12 +1241,12 @@ hadj=GTK_ADJUSTMENT(wave_hslider);
 gtk_signal_connect(GTK_OBJECT(wave_hslider), "value_changed",
                         GTK_SIGNAL_FUNC(service_hslider), NULL);
 hscroll=gtk_hscrollbar_new(hadj);
-GTK_WIDGET_SET_FLAGS(hscroll, GTK_CAN_FOCUS);
-
+/* GTK_WIDGET_SET_FLAGS(hscroll, GTK_CAN_FOCUS); */
 gtk_widget_show(hscroll);
+
 gtk_table_attach (GTK_TABLE (table), hscroll, 0, 9, 9, 10,
                         GTK_FILL,
-                        GTK_FILL | GTK_SHRINK, 3, 3);
+                        GTK_FILL | GTK_SHRINK, 3, 4);
 gtk_widget_show(table);
 
 frame=gtk_frame_new("Waves");
@@ -1148,12 +1273,9 @@ xsrc=(gint)hadj->value;
 num_traces_displayable=signalarea->allocation.height/(fontheight);
 num_traces_displayable--;   /* for the time trace that is always there */
 
-gdk_draw_rectangle(signalpixmap, signalarea->style->bg_gc[GTK_STATE_ACTIVE], TRUE, 
-		0, -1, signal_fill_width, fontheight); 
-gdk_draw_line(signalpixmap, signalarea->style->white_gc, 
-		0, fontheight-1, signal_fill_width-1, fontheight-1);
-gdk_draw_string(signalpixmap, signalfont, 
-	signalarea->style->fg_gc[0], 3+xsrc, fontheight-4, "Time");
+gdk_draw_rectangle(signalpixmap, gc_dkgray, TRUE, 0, -1, signal_fill_width, fontheight); 
+gdk_draw_line(signalpixmap, gc_white, 0, fontheight-1, signal_fill_width-1, fontheight-1);
+gdk_draw_string(signalpixmap, signalfont, gc_black, 3+xsrc, fontheight-4, "Time");
 
 t=traces.first;
 trwhich=0;
@@ -1222,20 +1344,20 @@ if(!(t->flags&TR_HIGHLIGHT))
 		{
 		if(dobackground==2)
 			{
-			gdk_draw_rectangle(signalpixmap, signalarea->style->bg_gc[GTK_STATE_NORMAL], TRUE, 
+			gdk_draw_rectangle(signalpixmap, gc_normal, TRUE, 
 				0, retval,
 		            	signal_fill_width, fontheight-1);
 			}
 			else
 			{
-			gdk_draw_rectangle(signalpixmap, signalarea->style->bg_gc[GTK_STATE_PRELIGHT], TRUE, 
+			gdk_draw_rectangle(signalpixmap, gc_ltgray, TRUE, 
 				0, retval,
 		            	signal_fill_width, fontheight-1);
 			}
 		}
 
 	gdk_draw_line(signalpixmap, 
-		signalarea->style->white_gc,
+		gc_white,
 		0, liney,
 		signal_fill_width-1, liney);
 
@@ -1250,14 +1372,14 @@ if(!(t->flags&TR_HIGHLIGHT))
 
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->fg_gc[0],
+				        gc_black,
 				        left_justify_sigs?3:3+max_signal_name_pixel_width-
 						combined, 
 					texty,
 				        t->name);
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->fg_gc[0],
+				        gc_black,
 				        left_justify_sigs?3+baselen:3+max_signal_name_pixel_width-
 						bufxlen, 
 					texty,
@@ -1267,7 +1389,7 @@ if(!(t->flags&TR_HIGHLIGHT))
 				{
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->fg_gc[0],
+				        gc_black,
 				        left_justify_sigs?3:3+max_signal_name_pixel_width-
 						gdk_string_measure(signalfont, t->name), 
 					texty,
@@ -1278,7 +1400,7 @@ if(!(t->flags&TR_HIGHLIGHT))
 		if((t->asciivalue)&&(!(t->flags&TR_EXCLUDE)))
 			gdk_draw_string(signalpixmap,
 				signalfont,
-	        		signalarea->style->fg_gc[0],
+	        		gc_black,
 	        		max_signal_name_pixel_width+6,
 				texty,
 	        		t->asciivalue);
@@ -1287,7 +1409,7 @@ if(!(t->flags&TR_HIGHLIGHT))
 		{
 		if(t->name)
 			{
-			int comment_color = (t->flags&TR_COLLAPSED) ? GTK_STATE_INSENSITIVE : GTK_STATE_SELECTED;
+			GdkGC *comment_color = (t->flags&TR_COLLAPSED) ? gc_mdgray : gc_dkblue;
 
 			if(bufxlen)
 				{
@@ -1296,14 +1418,14 @@ if(!(t->flags&TR_HIGHLIGHT))
 
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->bg_gc[comment_color],
+				        comment_color,
 				        left_justify_sigs?3:3+max_signal_name_pixel_width-
 						combined, 
 					texty,
 				        t->name);
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->bg_gc[comment_color],
+				        comment_color,
 				        left_justify_sigs?3+baselen:3+max_signal_name_pixel_width-
 						bufxlen, 
 					texty,
@@ -1313,7 +1435,7 @@ if(!(t->flags&TR_HIGHLIGHT))
 				{
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->bg_gc[comment_color],
+				        comment_color,
 				        left_justify_sigs?3:3+max_signal_name_pixel_width-
 						gdk_string_measure(signalfont, t->name), 
 					texty,
@@ -1324,11 +1446,11 @@ if(!(t->flags&TR_HIGHLIGHT))
 	}
 	else
 	{
-	gdk_draw_rectangle(signalpixmap, signalarea->style->bg_gc[GTK_STATE_SELECTED], TRUE, 
+	gdk_draw_rectangle(signalpixmap, gc_dkblue, TRUE, 
 		0, retval,
             	signal_fill_width, fontheight-1);
 	gdk_draw_line(signalpixmap, 
-		signalarea->style->white_gc,
+		gc_white,
 		0, liney,
 		signal_fill_width-1, liney);
 
@@ -1343,14 +1465,14 @@ if(!(t->flags&TR_HIGHLIGHT))
 
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->white_gc,
+				        gc_white,
 				        left_justify_sigs?3:3+max_signal_name_pixel_width-
 						combined, 
 					texty,
 				        t->name);
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->white_gc,
+				        gc_white,
 				        left_justify_sigs?3+baselen:3+max_signal_name_pixel_width-
 						bufxlen, 
 					texty,
@@ -1360,7 +1482,7 @@ if(!(t->flags&TR_HIGHLIGHT))
 				{
 				gdk_draw_string(signalpixmap,
 					signalfont,
-				        signalarea->style->white_gc,
+				        gc_white,
 				        left_justify_sigs?3:3+max_signal_name_pixel_width-
 						gdk_string_measure(signalfont, t->name), 
 					texty,
@@ -1371,7 +1493,7 @@ if(!(t->flags&TR_HIGHLIGHT))
 		if((t->asciivalue)&&(!(t->flags&TR_EXCLUDE)))
 			gdk_draw_string(signalpixmap,
 		      	signalfont,
-		       	signalarea->style->white_gc,
+		       	gc_white,
 		        max_signal_name_pixel_width+6,
 			texty,
 		        t->asciivalue);
@@ -1381,7 +1503,7 @@ if(!(t->flags&TR_HIGHLIGHT))
 		if(t->name)
 		gdk_draw_string(signalpixmap,
 		      	signalfont,
-		        (dobackground==2)?signalarea->style->bg_gc[GTK_STATE_PRELIGHT]:signalarea->style->bg_gc[GTK_STATE_ACTIVE],
+		        (dobackground==2)?gc_ltgray:gc_dkgray,
 		        left_justify_sigs?3:3+max_signal_name_pixel_width-
 				gdk_string_measure(signalfont, t->name), 
 			texty,
@@ -1470,8 +1592,8 @@ if(t->name)
 			else
 			{
 			char *str;
-			hptr hptr;
-			if((hptr=bsearch_node(t->n.nd,tims.marker)))
+			hptr h_ptr;
+			if((h_ptr=bsearch_node(t->n.nd,tims.marker)))
 				{
 				if(!t->n.nd->ext)
 					{
@@ -1479,11 +1601,11 @@ if(t->name)
 					str[0]='=';
 					if(t->flags&TR_INVERT)
 						{
-						str[1]=AN_STR_INV[hptr->v.h_val];
+						str[1]=AN_STR_INV[h_ptr->v.h_val];
 						}
 						else
 						{
-						str[1]=AN_STR[hptr->v.h_val];
+						str[1]=AN_STR[h_ptr->v.h_val];
 						}
 					t->asciivalue=str;
 					vlen=gdk_string_measure(signalfont,str);
@@ -1492,20 +1614,20 @@ if(t->name)
 					{
 					char *str2;
 
-					if(hptr->flags&HIST_REAL)
+					if(h_ptr->flags&HIST_REAL)
 						{
-						if(!(hptr->flags&HIST_STRING))
+						if(!(h_ptr->flags&HIST_STRING))
 							{
-							str=convert_ascii_real((double *)hptr->v.h_vector);
+							str=convert_ascii_real((double *)h_ptr->v.h_vector);
 							}
 							else
 							{
-							str=convert_ascii_string((char *)hptr->v.h_vector);
+							str=convert_ascii_string((char *)h_ptr->v.h_vector);
 							}
 						}
 						else
 						{
-		                        	str=convert_ascii_vec(t,hptr->v.h_vector);
+		                        	str=convert_ascii_vec(t,h_ptr->v.h_vector);
 						}
 
 					if(str)
@@ -1572,12 +1694,29 @@ if(!in_button_press)
 	
 		oldusize=signalwindow->allocation.width;
 		if(oldusize!=max_signal_name_pixel_width)
-			{
-			int os;
-			os=max_signal_name_pixel_width;
-			os=(os<48)?48:os;
-			gtk_widget_set_usize(GTK_WIDGET(signalwindow), 
-					os+30, -1);
+			{ /* keep signalwindow from expanding arbitrarily large */
+#ifdef WAVE_USE_GTK2
+			int wx, wy;
+			get_window_size(&wx, &wy);
+
+			if((3*max_signal_name_pixel_width) < (2*wx))	/* 2/3 width max */
+#else
+			if((3*max_signal_name_pixel_width) < (2*(wavewidth + signalwindow->allocation.width)))
+#endif
+				{
+				int os;
+				os=max_signal_name_pixel_width;
+				os=(os<48)?48:os;
+				gtk_widget_set_usize(GTK_WIDGET(signalwindow), 
+						os+30, -1);
+				}
+				else
+				{
+				int os;
+				os=48;
+				gtk_widget_set_usize(GTK_WIDGET(signalwindow), 
+						os+30, -1);
+				}
 			}
 		}
 	}
@@ -1624,8 +1763,8 @@ if((t->name)&&(!(t->flags&(TR_BLANK|TR_ANALOG_BLANK_STRETCH))))
 			else
 			{
 			char *str;
-			hptr hptr;
-			if((hptr=bsearch_node(t->n.nd,tims.marker)))
+			hptr h_ptr;
+			if((h_ptr=bsearch_node(t->n.nd,tims.marker)))
 				{
 				if(!t->n.nd->ext)
 					{
@@ -1633,11 +1772,11 @@ if((t->name)&&(!(t->flags&(TR_BLANK|TR_ANALOG_BLANK_STRETCH))))
 					str[0]='=';
 					if(t->flags&TR_INVERT)
 						{
-						str[1]=AN_STR_INV[hptr->v.h_val];
+						str[1]=AN_STR_INV[h_ptr->v.h_val];
 						}
 						else
 						{
-						str[1]=AN_STR[hptr->v.h_val];
+						str[1]=AN_STR[h_ptr->v.h_val];
 						}
 					t->asciivalue=str;
 					}
@@ -1645,20 +1784,20 @@ if((t->name)&&(!(t->flags&(TR_BLANK|TR_ANALOG_BLANK_STRETCH))))
 					{
 					char *str2;
 
-					if(hptr->flags&HIST_REAL)
+					if(h_ptr->flags&HIST_REAL)
 						{
-						if(!(hptr->flags&HIST_STRING))
+						if(!(h_ptr->flags&HIST_STRING))
 							{
-							str=convert_ascii_real((double *)hptr->v.h_vector);
+							str=convert_ascii_real((double *)h_ptr->v.h_vector);
 							}
 							else
 							{
-							str=convert_ascii_string((char *)hptr->v.h_vector);
+							str=convert_ascii_string((char *)h_ptr->v.h_vector);
 							}
 						}
 						else
 						{
-		                        	str=convert_ascii_vec(t,hptr->v.h_vector);
+		                        	str=convert_ascii_vec(t,h_ptr->v.h_vector);
 						}
 
 					if(str)
@@ -1892,6 +2031,8 @@ gdk_draw_rectangle(wavepixmap, gc_timeb, TRUE,
 		0, -1, wavewidth, fontheight); 
 rendertimes();
 rendertraces();
+
+update_dual();
 }
 
 static void rendertraces(void)
@@ -2217,7 +2358,7 @@ tims.end+=shift_timebase;
 
 static void draw_hptr_trace_vector_analog(Trptr t, hptr h, int which, int num_extension)
 {
-TimeType x0, x1, newtime, width;
+TimeType x0, x1, newtime;
 int y0, y1, yu, liney, yt0, yt1;
 TimeType tim, h2tim;
 hptr h2, h3;
@@ -2260,7 +2401,7 @@ if(h3->flags&HIST_REAL)
 	}
 	else
 	{
-	tv=convert_real_vec(t,h3->v.h_vector);
+	if(h3->time <= tims.last) tv=convert_real_vec(t,h3->v.h_vector);
 	}
 if (!isnan(tv))
 	{
@@ -2319,6 +2460,7 @@ if(x1>wavewidth)
 /* draw trans */
 type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.h_vector) : AN_0;
 tv = tv2 = mynan;
+
 if(h->flags&HIST_REAL)
 	{
 	if(!(h->flags&HIST_STRING) && h->v.h_vector)
@@ -2326,8 +2468,9 @@ if(h->flags&HIST_REAL)
 	}
 	else
 	{
-	tv=convert_real_vec(t,h->v.h_vector);
+	if(h->time <= tims.last) tv=convert_real_vec(t,h->v.h_vector);
 	}
+
 if(h2->flags&HIST_REAL)
 	{
 	if(!(h2->flags&HIST_STRING) && h2->v.h_vector)
@@ -2335,8 +2478,9 @@ if(h2->flags&HIST_REAL)
 	}
 	else
 	{
-	tv2=convert_real_vec(t,h2->v.h_vector);
+	if(h2->time <= tims.last) tv2=convert_real_vec(t,h2->v.h_vector);
 	}
+
 if(isnan(tv))
 	{
 	yt0 = yu;
@@ -2495,7 +2639,7 @@ if(x1>wavewidth)
 
 /* draw trans */
 type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.h_vector) : AN_0;
-if(x0>-1)
+if(x0>-1) {
 if(use_roundcaps)
 	{
 	if (type == AN_Z) {
@@ -2556,7 +2700,7 @@ if(use_roundcaps)
 		x0, y0,
 		x0, y1);
 	}
-
+}
 		
 if(x0!=x1)
 	{
@@ -2687,7 +2831,7 @@ tims.end+=shift_timebase;
 
 static void draw_vptr_trace_analog(Trptr t, vptr v, int which, int num_extension)
 {
-TimeType x0, x1, newtime, width;
+TimeType x0, x1, newtime;
 int y0, y1, yu, liney, yt0, yt1;
 TimeType tim, h2tim;
 vptr h, h2, h3;
@@ -2937,7 +3081,7 @@ if(x1>wavewidth)
 /* draw trans */
 type = vtype2(t,h);
 
-if(x0>-1)
+if(x0>-1) {
 if(use_roundcaps)
 	{
 	if (type == AN_Z) 
@@ -3007,6 +3151,7 @@ if(use_roundcaps)
 			x0, y0,
 			x0, y1);
 		}
+}
 
 if(x0!=x1)
 	{
@@ -3118,3 +3263,12 @@ wave_gdk_draw_line_flush(wavepixmap);
 tims.start+=shift_timebase;
 tims.end+=shift_timebase;
 }
+
+/*
+ * $Id: wavewindow.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: wavewindow.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
