@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include <config.h>
 #include "pixmaps.h"
 #include "currenttime.h"
 #include "debug.h"
@@ -477,3 +478,11 @@ gtk_widget_show(table2);
 return table;
 }
 
+/*
+ * $Id: zoombuttons.c,v 1.2 2007/04/20 02:08:18 gtkwave Exp $
+ * $Log: zoombuttons.c,v $
+ * Revision 1.2  2007/04/20 02:08:18  gtkwave
+ * initial release
+ *
+ */
+
